@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.chared.c,v 3.71 2002/03/08 17:36:45 christos Exp $ */
+/* $Header: /src/pub/tcsh/ed.chared.c,v 3.77 2004/11/21 04:38:03 christos Exp $ */
 /*
  * ed.chared.c: Character editing functions.
  */
@@ -72,7 +72,7 @@
 
 #include "sh.h"
 
-RCSID("$Id: ed.chared.c,v 3.71 2002/03/08 17:36:45 christos Exp $")
+RCSID("$Id: ed.chared.c,v 3.77 2004/11/21 04:38:03 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -116,7 +116,7 @@ static	void	 c_alternativ_key_map	__P((int));
 void	 c_insert		__P((int));
 void	 c_delafter		__P((int));
 void	 c_delbefore		__P((int));
-static 	int	 c_to_class		__P((int));
+static 	int	 c_to_class		__P((Char));
 static	Char	*c_prev_word		__P((Char *, Char *, int));
 static	Char	*c_next_word		__P((Char *, Char *, int));
 static	Char	*c_number		__P((Char *, int *, int));
@@ -146,8 +146,9 @@ static	CCRETVAL v_csearch_back		__P((int, int, int));
 #if defined(DSPMBYTE)
 static	void 	 e_charfwd_mbyte	__P((int));
 static	void 	 e_charback_mbyte	__P((int));
-static  int extdel;
-static  int extins = 0;
+static  int	 extdel;
+static  int	 extins = 0;
+extern  bool	 dspmbyte_utf8;
 #endif
 
 static void
@@ -206,9 +207,18 @@ c_delafter(num)
 	    for (wkcp = Cursor ; wkcp < Cursor + num; wkcp++) {
 		if (extdel == 0)
 		    extdel = Ismbyte1(*wkcp); /* check to 1st. byte */
+		else if (dspmbyte_utf8 && Ismbyte1(*wkcp))
+		    extdel = 1;
+		else if (dspmbyte_utf8 && Ismbyte2(*wkcp))
+		    extdel++;
 		else
 		    extdel = 0;	/* if 2nd. byte, force set to 0 */
 	    }
+	    while (extdel && num < LastChar - Cursor && Ismbyte2(Cursor[num])) {
+		num++;
+		if (!dspmbyte_utf8)
+		    break;
+	    }
 	}
 #endif
 	if (VImode) {
@@ -225,20 +235,6 @@ c_delafter(num)
 	    for (cp = Cursor; cp <= LastChar; cp++)
 		*cp = cp[num];
 	LastChar -= num;
-#if defined(DSPMBYTE)
-	if (_enable_mbdisp && extdel && Ismbyte2(*Cursor)) {
-	    if( VImode ) {
-		UndoSize++;
-		*kp++ = *Cursor; /* Save deleted chars into undobuf */
-	    }
-	    for (cp = Cursor; cp <= LastChar; cp++)
-		*cp = cp[1];
-	    LastChar--;
-	    e_redisp( 1 );
-	}
-	else
-	    extdel = 0;
-#endif
     }
 #ifdef notdef
     else {
@@ -259,7 +255,6 @@ c_delbefore(num)		/* delete before dot, with bounds checking */
 
 #if defined(DSPMBYTE)
     Char *nowcur, *wkcp;
-    Char delc;
 
     extdel = 0;
 #endif
@@ -270,7 +265,25 @@ c_delbefore(num)		/* delete before dot, with bounds checking */
     if (num > 0) {			/* if I can delete anything */
 #if defined(DSPMBYTE)
 	nowcur = Cursor - num;
-	delc = *nowcur;
+#endif
+
+#if defined(DSPMBYTE)
+	if (_enable_mbdisp) {
+	    for (wkcp = InputBuf; wkcp < nowcur; wkcp++) {
+		if(extdel == 0)
+		    extdel = Ismbyte1(*wkcp); /* check to 1st. byte */
+		else if (dspmbyte_utf8 && Ismbyte1(*wkcp))
+		    extdel = 1;
+		else if (dspmbyte_utf8 && Ismbyte2(*wkcp))
+		    extdel++;
+		else
+		    extdel = 0;	/* if 2nd. byte, force set to 0 */
+	    }
+	    if (extdel && Ismbyte2(*nowcur)) {
+		num += extdel;
+	    } else
+		extdel = 0;
+	}
 #endif
 	if (VImode) {
 	    kp = UndoBuf;		/* Set Up for VI undo command */
@@ -286,29 +299,6 @@ c_delbefore(num)		/* delete before dot, with bounds checking */
 	    for (cp = Cursor - num; cp <= LastChar; cp++)
 		*cp = cp[num];
 	LastChar -= num;
-#if defined(DSPMBYTE)
-	if (_enable_mbdisp) {
-	    for (wkcp = InputBuf; wkcp < nowcur; wkcp++) {
-		if(extdel == 0)
-		    extdel = Ismbyte1(*wkcp); /* check to 1st. byte */
-		else
-		    extdel = 0;	/* if 2nd. byte, force set to 0 */
-	    }
-	    if (extdel && Ismbyte2(delc)) {
-		if( VImode ) {
-		    UndoSize++;
-		    UndoPtr--;
-		    *kp++ = *(nowcur-1);
-				/* Save deleted chars into undobuf */
-		}
-		for (cp = nowcur - 1; cp <= LastChar; cp++)
-		    *cp = cp[1];
-		LastChar--;
-	    }
-	}
-	else
-	    extdel = 0;
-#endif
     }
 }
 
@@ -362,7 +352,7 @@ c_preword(p, low, n, delim)
  */
 static int
 c_to_class(ch)
-int  ch;
+Char ch;
 {
     if (Isspace(ch))
         return C_CLASS_WHITE;
@@ -846,7 +836,9 @@ c_delfini()		/* Finish up delete action */
 	Cursor = ActionPos;
 #if defined(DSPMBYTE)
 	if (_enable_mbdisp && extdel) {
-	    Cursor--;
+	    Cursor -= extdel;
+	    if (Cursor < InputBuf)
+		Cursor = InputBuf;
 	    e_redisp(1);
 	}
 #endif
@@ -1021,6 +1013,109 @@ c_push_kill(start, end)
     *kp = '\0';
 }
 
+#if defined(DSPMBYTE)
+
+static int
+Utf8Len(cp, w)
+    Char *cp;
+    int *w;
+{
+    Char c;
+    int n, u, f = 0;
+
+    c = *cp++;
+    if (!Ismbyte1(c)) {
+        *w = 1;
+	return 1;
+    }
+    n = 1;
+    if ((c & 0xfe) == 0xfc) {
+	n = 5;
+	u = c & 0x01;
+    } else if ((c & 0xfc) == 0xf8) {
+	n = 4;
+	u = c & 0x03;
+    } else if ((c & 0xf8) == 0xf0) {
+	n = 3;
+	u = c & 0x07;
+    } else if ((c & 0xf0) == 0xe0) {
+	n = 2;
+	u = c & 0x0f;
+    } else {
+	n = 1;
+	u = c & 0x1f;
+    }
+    while (n-- > 0) {
+	c = *cp++;
+	f++;
+        if (!Ismbyte2(c)) {
+	    *w = -1;
+	    return f;
+        }
+        u = u << 6 | (c & 0x3f);
+    }
+    *w = (u >= 0x1100 &&
+     (u <= 0x115f ||		/* Hangul Jamo init. consonants */
+      (u >= 0x2e80 && u <= 0xa4cf && (u & ~0x0011) != 0x300a &&
+       u != 0x303f) ||                  /* CJK ... Yi */
+      (u >= 0xac00 && u <= 0xd7a3) || /* Hangul Syllables */
+      (u >= 0xdf00 && u <= 0xdfff) || /* dw combining sequence */
+      (u >= 0xf900 && u <= 0xfaff) || /* CJK Compatibility Ideographs */
+      (u >= 0xfe30 && u <= 0xfe6f) || /* CJK Compatibility Forms */
+      (u >= 0xff00 && u <= 0xff5f) || /* Fullwidth Forms */
+      (u >= 0xffe0 && u <= 0xffe6) ||
+      (u >= 0x20000 && u <= 0x2ffff))) ? 2 : 1;
+    return f + 1;
+}
+
+int
+ColLen(cp)
+    Char *cp;
+{
+    Char c;
+    int w, l = 0;
+    while ((c = *cp) != 0) {
+	if (Ismbyte1(c)) {
+	    cp += Utf8Len(cp, &w);
+	    if (w > 0)
+		l += w;
+	}
+	else {
+	    cp++;
+	    l++;
+	}
+    }
+    return l;
+}
+
+
+void
+Setutf8lit(cp, end)
+    Char *cp, *end;
+{
+    Char c;
+    int w, f;
+
+    while (end ? cp < end : *cp != '\0') {
+	c = *cp;
+	if (!Ismbyte1(c)) {
+	    cp++;
+	    continue;
+	}
+	f = Utf8Len(cp, &w);
+	if (w < 0) {
+	    cp += f;
+	    continue;
+	}
+	w = f - w;
+	for (; f-- > 0; cp++)
+	    if (w-- > 0) 
+		*cp |= LITERAL;
+    }
+}
+
+#endif
+
 static CCRETVAL
 c_get_histline()
 {
@@ -1073,6 +1168,11 @@ c_get_histline()
 	if (LastChar < InputBuf)
 	    LastChar = InputBuf;
     }
+  
+#ifdef DSPMBYTE
+    if (dspmbyte_utf8)
+	Setutf8lit(InputBuf, LastChar);
+#endif /* DSPMBYTE */
 
 #ifdef KSHVI
     if (VImode)
@@ -1153,7 +1253,8 @@ e_inc_search(dir)
 	if (GetNextChar(&ch) != 1)
 	    return(e_send_eof(0));
 
-	switch (CurrentKeyMap[(unsigned char) ch]) {
+	switch (ch > NT_NUM_KEYS
+		? F_INSERT : CurrentKeyMap[(unsigned char) ch]) {
 	case F_INSERT:
 	case F_DIGIT:
 	case F_MAGIC_SPACE:
@@ -1435,7 +1536,7 @@ v_search(dir)
 /*ARGSUSED*/
 CCRETVAL
 v_cmd_mode(c)
-    int c;
+    Char c;
 {
     USE(c);
     InsertPos = 0;
@@ -1464,7 +1565,7 @@ v_cmd_mode(c)
 /*ARGSUSED*/
 CCRETVAL
 e_unassigned(c)
-    int c;
+    Char c;
 {				/* bound to keys that arn't really assigned */
     USE(c);
     SoundBeep();
@@ -1474,7 +1575,7 @@ e_unassigned(c)
 
 CCRETVAL
 e_insert(c)
-    int c;
+    Char c;
 {
     int i;
 #if defined(DSPMBYTE)
@@ -1520,12 +1621,40 @@ e_insert(c)
 	    *Cursor++ = (Char) ' ';
 	}
 	else if (_enable_mbdisp && extins && Ismbyte2(c)) {
-	    *(Cursor-1) = savec;
-	    *Cursor++ = (Char) c;
-	    extins = 0;
-	    e_redisp(1);
-	    Refresh();
-	    ret = CC_REFRESH;
+	    if (dspmbyte_utf8) {
+		int n = 1, w;
+		if ((savec & 0xfe) == 0xfc)
+		    n = 5;
+		else if ((savec & 0xfc) == 0xf8)
+		    n = 4;
+		else if ((savec & 0xf8) == 0xf0)
+		    n = 3;
+		else if ((savec & 0xf0) == 0xe0)
+		    n = 2;
+		if (extins++ < n) {
+		    *Cursor++ = (Char )c;
+		} else {
+		    *Cursor++ = (Char )c;
+		    *(Cursor-extins) = savec;
+		    Utf8Len(Cursor - extins, &w);
+		    if (w > 0) {
+			w = extins - w;
+			while (w-- > 0)
+			    Cursor[w - extins] |= LITERAL;
+		    }
+		    extins = 0;
+		    e_redisp(1);
+		    Refresh();
+		    ret = CC_REFRESH;
+		}
+	    } else {
+		*(Cursor-1) = savec;
+		*Cursor++ = (Char)c;
+		extins = 0;
+		e_redisp(1);
+		Refresh();
+		ret = CC_REFRESH;
+	    }
 	}
 	else
 	    *Cursor++ = (Char) c;
@@ -1610,7 +1739,9 @@ DeleteBack(n)			/* delete the n characters before . */
 	    Cursor -= n;
 #if defined(DSPMBYTE)
 	if(_enable_mbdisp && extdel && Cursor > InputBuf) {
-	    Cursor--;
+	    Cursor -= extdel;
+	    if (Cursor < InputBuf)
+		Cursor = InputBuf;
 	    e_redisp(1);
 	}
 #endif
@@ -1619,7 +1750,7 @@ DeleteBack(n)			/* delete the n characters before . */
 
 CCRETVAL
 e_digit(c)			/* gray magic here */
-    int c;
+    Char c;
 {
     if (!Isdigit(c))
 	return(CC_ERROR);	/* no NULs in the input ever!! */
@@ -1653,7 +1784,7 @@ e_digit(c)			/* gray magic here */
 
 CCRETVAL
 e_argdigit(c)			/* for ESC-n */
-    int c;
+    Char c;
 {
     c &= ASCII;
 
@@ -1674,7 +1805,7 @@ e_argdigit(c)			/* for ESC-n */
 
 CCRETVAL
 v_zero(c)			/* command mode 0 for vi */
-    int c;
+    Char c;
 {
     if (DoingArg) {		/* if doing an arg, add this in... */
 	if (Argument > 1000000)
@@ -1696,7 +1827,7 @@ v_zero(c)			/* command mode 0 for vi */
 /*ARGSUSED*/
 CCRETVAL
 e_newline(c)
-    int c;
+    Char c;
 {				/* always ignore argument */
     USE(c);
   /*  PastBottom();  NOW done in ed.inputl.c */
@@ -1710,7 +1841,7 @@ e_newline(c)
 /*ARGSUSED*/
 CCRETVAL
 e_send_eof(c)
-    int c;
+    Char c;
 {				/* for when ^D is ONLY send-eof */
     USE(c);
     PastBottom();
@@ -1721,7 +1852,7 @@ e_send_eof(c)
 /*ARGSUSED*/
 CCRETVAL
 e_complete(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -1731,7 +1862,7 @@ e_complete(c)
 /*ARGSUSED*/
 CCRETVAL
 e_complete_back(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -1741,7 +1872,7 @@ e_complete_back(c)
 /*ARGSUSED*/
 CCRETVAL
 e_complete_fwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -1751,7 +1882,7 @@ e_complete_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 e_complete_all(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -1761,7 +1892,7 @@ e_complete_all(c)
 /*ARGSUSED*/
 CCRETVAL
 v_cm_complete(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor < LastChar)
@@ -1773,7 +1904,7 @@ v_cm_complete(c)
 /*ARGSUSED*/
 CCRETVAL
 e_toggle_hist(c)
-    int c;
+    Char c;
 {
     struct Hist *hp;
     int     h;
@@ -1817,6 +1948,11 @@ e_toggle_hist(c)
 	    LastChar = InputBuf;
     }
 
+#ifdef DSPMBYTE
+    if (dspmbyte_utf8)
+	Setutf8lit(InputBuf, LastChar);
+#endif /* DSPMBYTE */
+
 #ifdef KSHVI
     if (VImode)
 	Cursor = InputBuf;
@@ -1830,7 +1966,7 @@ e_toggle_hist(c)
 /*ARGSUSED*/
 CCRETVAL
 e_up_hist(c)
-    int c;
+    Char c;
 {
     Char    beep = 0;
 
@@ -1860,7 +1996,7 @@ e_up_hist(c)
 /*ARGSUSED*/
 CCRETVAL
 e_down_hist(c)
-    int c;
+    Char c;
 {
     USE(c);
     UndoAction = TCSHOP_NOP;
@@ -1917,7 +2053,7 @@ c_hsetpat()
 /*ARGSUSED*/
 CCRETVAL
 e_up_search_hist(c)
-    int c;
+    Char c;
 {
     struct Hist *hp;
     int h;
@@ -1986,7 +2122,7 @@ e_up_search_hist(c)
 /*ARGSUSED*/
 CCRETVAL
 e_down_search_hist(c)
-    int c;
+    Char c;
 {
     struct Hist *hp;
     int h;
@@ -2040,7 +2176,7 @@ e_down_search_hist(c)
 /*ARGSUSED*/
 CCRETVAL
 e_helpme(c)
-    int c;
+    Char c;
 {
     USE(c);
     PastBottom();
@@ -2051,7 +2187,7 @@ e_helpme(c)
 /*ARGSUSED*/
 CCRETVAL
 e_correct(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2061,7 +2197,7 @@ e_correct(c)
 /*ARGSUSED*/
 CCRETVAL
 e_correctl(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2071,10 +2207,9 @@ e_correctl(c)
 /*ARGSUSED*/
 CCRETVAL
 e_run_fg_editor(c)
-    int c;
+    Char c;
 {
     struct process *pp;
-    extern bool tellwhat;
 
     USE(c);
     if ((pp = find_stop_ed()) != NULL) {
@@ -2103,7 +2238,7 @@ e_run_fg_editor(c)
 /*ARGSUSED*/
 CCRETVAL
 e_list_choices(c)
-    int c;
+    Char c;
 {
     USE(c);
     PastBottom();
@@ -2114,7 +2249,7 @@ e_list_choices(c)
 /*ARGSUSED*/
 CCRETVAL
 e_list_all(c)
-    int c;
+    Char c;
 {
     USE(c);
     PastBottom();
@@ -2125,7 +2260,7 @@ e_list_all(c)
 /*ARGSUSED*/
 CCRETVAL
 e_list_glob(c)
-    int c;
+    Char c;
 {
     USE(c);
     PastBottom();
@@ -2136,7 +2271,7 @@ e_list_glob(c)
 /*ARGSUSED*/
 CCRETVAL
 e_expand_glob(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2146,7 +2281,7 @@ e_expand_glob(c)
 /*ARGSUSED*/
 CCRETVAL
 e_normalize_path(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2156,7 +2291,7 @@ e_normalize_path(c)
 /*ARGSUSED*/
 CCRETVAL
 e_normalize_command(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2166,7 +2301,7 @@ e_normalize_command(c)
 /*ARGSUSED*/
 CCRETVAL
 e_expand_vars(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -2176,7 +2311,7 @@ e_expand_vars(c)
 /*ARGSUSED*/
 CCRETVAL
 e_which(c)
-    int c;
+    Char c;
 {				/* do a fast command line which(1) */
     USE(c);
     PastBottom();
@@ -2187,7 +2322,7 @@ e_which(c)
 /*ARGSUSED*/
 CCRETVAL
 e_last_item(c)
-    int c;
+    Char c;
 {				/* insert the last element of the prev. cmd */
     Char *cp;
     struct Hist *hp;
@@ -2227,7 +2362,7 @@ e_last_item(c)
 /*ARGSUSED*/
 CCRETVAL
 e_dabbrev_expand(c)
-    int c;
+    Char c;
 {				/* expand to preceding word matching prefix */
     Char *cp, *ncp, *bp;
     struct Hist *hp;
@@ -2312,7 +2447,7 @@ e_dabbrev_expand(c)
 /*ARGSUSED*/
 CCRETVAL
 e_yank_kill(c)
-    int c;
+    Char c;
 {				/* almost like GnuEmacs */
     int len;
     Char *kp, *cp;
@@ -2344,7 +2479,7 @@ e_yank_kill(c)
 /*ARGSUSED*/
 CCRETVAL
 e_yank_pop(c)
-    int c;
+    Char c;
 {				/* almost like GnuEmacs */
     int m_bef_c, del_len, ins_len;
     Char *kp, *cp;
@@ -2405,7 +2540,7 @@ e_yank_pop(c)
 /*ARGSUSED*/
 CCRETVAL
 v_delprev(c) 		/* Backspace key in insert mode */
-    int c;
+    Char c;
 {
     int rc;
 
@@ -2418,7 +2553,9 @@ v_delprev(c) 		/* Backspace key in insert mode */
 	    Cursor -= Argument;
 #if defined(DSPMBYTE)
 	if (_enable_mbdisp && extdel) {
-	    Cursor--;
+	    Cursor -= extdel;
+	    if (Cursor < InputBuf)
+		Cursor = InputBuf;
 	    e_redisp(c);
 	}
 #endif
@@ -2431,7 +2568,7 @@ v_delprev(c) 		/* Backspace key in insert mode */
 /*ARGSUSED*/
 CCRETVAL
 e_delprev(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor > InputBuf) {
@@ -2442,7 +2579,9 @@ e_delprev(c)
 	    Cursor -= Argument;
 #if defined(DSPMBYTE)
 	if (_enable_mbdisp && extdel && Cursor > InputBuf) {
-	    Cursor--;
+	    Cursor -= extdel;
+	    if (Cursor < InputBuf)
+		Cursor = InputBuf;
 	    e_redisp(c);
 	}
 #endif
@@ -2456,7 +2595,7 @@ e_delprev(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delwordprev(c)
-    int c;
+    Char c;
 {
     Char *cp;
 
@@ -2507,7 +2646,7 @@ e_delwordprev(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delnext(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar) {/* if I'm at the end */
@@ -2531,14 +2670,14 @@ e_delnext(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delnext_eof(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar) {/* if I'm at the end */
 	if (!VImode) {
 	    if (Cursor == InputBuf) {	
 		/* if I'm also at the beginning */
-		so_write(STReof, 4);/* then do a EOF */
+		so_write(eSTReof, 4);/* then do a EOF */
 		flush();
 		return(CC_EOF);
 	    }
@@ -2561,7 +2700,7 @@ e_delnext_eof(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delnext_list(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar) {	/* if I'm at the end */
@@ -2580,12 +2719,12 @@ e_delnext_list(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delnext_list_eof(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar) {	/* if I'm at the end */
 	if (Cursor == InputBuf) {	/* if I'm also at the beginning */
-	    so_write(STReof, 4);/* then do a EOF */
+	    so_write(eSTReof, 4);/* then do a EOF */
 	    flush();
 	    return(CC_EOF);
 	}
@@ -2606,13 +2745,13 @@ e_delnext_list_eof(c)
 /*ARGSUSED*/
 CCRETVAL
 e_list_eof(c)
-    int c;
+    Char c;
 {
     CCRETVAL rv;
 
     USE(c);
     if (Cursor == LastChar && Cursor == InputBuf) {
-	so_write(STReof, 4);	/* then do a EOF */
+	so_write(eSTReof, 4);	/* then do a EOF */
 	flush();
 	rv = CC_EOF;
     }
@@ -2627,7 +2766,7 @@ e_list_eof(c)
 /*ARGSUSED*/
 CCRETVAL
 e_delwordnext(c)
-    int c;
+    Char c;
 {
     Char *cp;
 
@@ -2649,7 +2788,7 @@ e_delwordnext(c)
 /*ARGSUSED*/
 CCRETVAL
 e_toend(c)
-    int c;
+    Char c;
 {
     USE(c);
     Cursor = LastChar;
@@ -2665,7 +2804,7 @@ e_toend(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tobeg(c)
-    int c;
+    Char c;
 {
     USE(c);
     Cursor = InputBuf;
@@ -2686,7 +2825,7 @@ e_tobeg(c)
 /*ARGSUSED*/
 CCRETVAL
 e_killend(c)
-    int c;
+    Char c;
 {
     USE(c);
     c_push_kill(Cursor, LastChar); /* copy it */
@@ -2698,7 +2837,7 @@ e_killend(c)
 /*ARGSUSED*/
 CCRETVAL
 e_killbeg(c)
-    int c;
+    Char c;
 {
     USE(c);
     c_push_kill(InputBuf, Cursor); /* copy it */
@@ -2710,7 +2849,7 @@ e_killbeg(c)
 /*ARGSUSED*/
 CCRETVAL
 e_killall(c)
-    int c;
+    Char c;
 {
     USE(c);
     c_push_kill(InputBuf, LastChar); /* copy it */
@@ -2722,7 +2861,7 @@ e_killall(c)
 /*ARGSUSED*/
 CCRETVAL
 e_killregion(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (!Mark)
@@ -2744,7 +2883,7 @@ e_killregion(c)
 /*ARGSUSED*/
 CCRETVAL
 e_copyregion(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (!Mark)
@@ -2762,7 +2901,7 @@ e_copyregion(c)
 /*ARGSUSED*/
 CCRETVAL
 e_charswitch(cc)
-    int cc;
+    Char cc;
 {
     Char c;
 
@@ -2785,7 +2924,7 @@ e_charswitch(cc)
 /*ARGSUSED*/
 CCRETVAL
 e_gcharswitch(cc)
-    int cc;
+    Char cc;
 {				/* gosmacs style ^T */
     Char c;
 
@@ -2815,6 +2954,9 @@ e_charback_mbyte(argument)
     }
     else {
 	while (0 < argument && Cursor > InputBuf) {
+	    /* utf8 has more then one mbyte2 byte */
+	    while (dspmbyte_utf8 && Cursor - 1 > InputBuf && Ismbyte2(*(Cursor - 2)) && Ismbyte2(*(Cursor - 1)))
+		Cursor--;
 	    if (Cursor - 1 != InputBuf &&
 		Ismbyte1(*(Cursor - 2)) && Ismbyte2(*(Cursor - 1))) {
 		Cursor--;
@@ -2829,7 +2971,7 @@ e_charback_mbyte(argument)
 /*ARGSUSED*/
 CCRETVAL
 e_charback(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor > InputBuf) {
@@ -2859,7 +3001,7 @@ e_charback(c)
 /*ARGSUSED*/
 CCRETVAL
 v_wordback(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == InputBuf)
@@ -2880,7 +3022,7 @@ v_wordback(c)
 /*ARGSUSED*/
 CCRETVAL
 e_wordback(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == InputBuf)
@@ -2909,10 +3051,13 @@ e_charfwd_mbyte(argument)
 	Cursor += argument;
     else
 	while (0 < argument && Cursor < LastChar) {
+	    /* utf8 has more then one mbyte2 byte */
 	    if (Cursor + 1 != LastChar &&
 		Ismbyte1(*Cursor) && Ismbyte2(*(Cursor + 1))) {
 		Cursor++;
 	    }
+	    while (dspmbyte_utf8 && Cursor + 1 < LastChar && Ismbyte2(*Cursor) && Ismbyte2(*(Cursor + 1)))
+		Cursor++;
 	    Cursor++;
 	    argument--;
 	}
@@ -2922,7 +3067,7 @@ e_charfwd_mbyte(argument)
 /*ARGSUSED*/
 CCRETVAL
 e_charfwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor < LastChar) {
@@ -2951,7 +3096,7 @@ e_charfwd(c)
 /*ARGSUSED*/
 CCRETVAL
 e_wordfwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar)
@@ -2973,7 +3118,7 @@ e_wordfwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_wordfwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar)
@@ -2995,7 +3140,7 @@ v_wordfwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_wordbegnext(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar)
@@ -3172,7 +3317,7 @@ c_get_word(begin, end)
 /*ARGSUSED*/
 CCRETVAL
 e_uppercase(c)
-    int c;
+    Char c;
 {
     Char   *cp, *end;
 
@@ -3193,7 +3338,7 @@ e_uppercase(c)
 /*ARGSUSED*/
 CCRETVAL
 e_capitolcase(c)
-    int c;
+    Char c;
 {
     Char   *cp, *end;
 
@@ -3222,7 +3367,7 @@ e_capitolcase(c)
 /*ARGSUSED*/
 CCRETVAL
 e_lowercase(c)
-    int c;
+    Char c;
 {
     Char   *cp, *end;
 
@@ -3243,7 +3388,7 @@ e_lowercase(c)
 /*ARGSUSED*/
 CCRETVAL
 e_set_mark(c)
-    int c;
+    Char c;
 {
     USE(c);
     Mark = Cursor;
@@ -3253,7 +3398,7 @@ e_set_mark(c)
 /*ARGSUSED*/
 CCRETVAL
 e_exchange_mark(c)
-    int c;
+    Char c;
 {
     Char *cp;
 
@@ -3268,7 +3413,7 @@ e_exchange_mark(c)
 /*ARGSUSED*/
 CCRETVAL
 e_argfour(c)
-    int c;
+    Char c;
 {				/* multiply current argument by 4 */
     USE(c);
     if (Argument > 1000000)
@@ -3281,7 +3426,7 @@ e_argfour(c)
 /*ARGSUSED*/
 CCRETVAL
 e_quote(c)
-    int c;
+    Char c;
 {
     Char    ch;
     int     num;
@@ -3299,7 +3444,7 @@ e_quote(c)
 /*ARGSUSED*/
 CCRETVAL
 e_metanext(c)
-    int c;
+    Char c;
 {
     USE(c);
     MetaNext = 1;
@@ -3310,7 +3455,7 @@ e_metanext(c)
 /*ARGSUSED*/
 CCRETVAL
 e_extendnext(c)
-    int c;
+    Char c;
 {
     CurrentKeyMap = CcAltMap;
     return(CC_ARGHACK);	/* preserve argument */
@@ -3321,7 +3466,7 @@ e_extendnext(c)
 /*ARGSUSED*/
 CCRETVAL
 v_insbeg(c)
-    int c;
+    Char c;
 {				/* move to beginning of line and start vi
 				 * insert mode */
     USE(c);
@@ -3339,7 +3484,7 @@ v_insbeg(c)
 /*ARGSUSED*/
 CCRETVAL
 v_replone(c)
-    int c;
+    Char c;
 {				/* vi mode overwrite one character */
     USE(c);
     c_alternativ_key_map(0);
@@ -3353,7 +3498,7 @@ v_replone(c)
 /*ARGSUSED*/
 CCRETVAL
 v_replmode(c)
-    int c;
+    Char c;
 {				/* vi mode start overwriting */
     USE(c);
     c_alternativ_key_map(0);
@@ -3367,7 +3512,7 @@ v_replmode(c)
 /*ARGSUSED*/
 CCRETVAL
 v_substchar(c)
-    int c;
+    Char c;
 {				/* vi mode substitute for one char */
     USE(c);
     c_delafter(Argument);
@@ -3378,7 +3523,7 @@ v_substchar(c)
 /*ARGSUSED*/
 CCRETVAL
 v_substline(c)
-    int c;
+    Char c;
 {				/* vi mode replace whole line */
     USE(c);
     (void) e_killall(0);
@@ -3389,7 +3534,7 @@ v_substline(c)
 /*ARGSUSED*/
 CCRETVAL
 v_chgtoend(c)
-    int c;
+    Char c;
 {				/* vi mode change to end of line */
     USE(c);
     (void) e_killend(0);
@@ -3400,7 +3545,7 @@ v_chgtoend(c)
 /*ARGSUSED*/
 CCRETVAL
 v_insert(c)
-    int c;
+    Char c;
 {				/* vi mode start inserting */
     USE(c);
     c_alternativ_key_map(0);
@@ -3415,7 +3560,7 @@ v_insert(c)
 /*ARGSUSED*/
 CCRETVAL
 v_add(c)
-    int c;
+    Char c;
 {				/* vi mode start adding */
     USE(c);
     c_alternativ_key_map(0);
@@ -3437,7 +3582,7 @@ v_add(c)
 /*ARGSUSED*/
 CCRETVAL
 v_addend(c)
-    int c;
+    Char c;
 {				/* vi mode to add at end of line */
     USE(c);
     c_alternativ_key_map(0);
@@ -3454,9 +3599,9 @@ v_addend(c)
 /*ARGSUSED*/
 CCRETVAL
 v_change_case(cc)
-    int cc;
+    Char cc;
 {
-    char    c;
+    Char    c;
 
     USE(cc);
     if (Cursor < LastChar) {
@@ -3480,10 +3625,9 @@ v_change_case(cc)
 /*ARGSUSED*/
 CCRETVAL
 e_expand(c)
-    int c;
+    Char c;
 {
     Char *p;
-    extern bool justpr;
 
     USE(c);
     for (p = InputBuf; Isspace(*p); p++)
@@ -3499,7 +3643,7 @@ e_expand(c)
 /*ARGSUSED*/
 CCRETVAL
 e_startover(c)
-    int c;
+    Char c;
 {				/* erase all of current line, start again */
     USE(c);
     ResetInLine(0);		/* reset the input pointers */
@@ -3509,7 +3653,7 @@ e_startover(c)
 /*ARGSUSED*/
 CCRETVAL
 e_redisp(c)
-    int c;
+    Char c;
 {
     USE(c);
     ClearLines();
@@ -3520,7 +3664,7 @@ e_redisp(c)
 /*ARGSUSED*/
 CCRETVAL
 e_cleardisp(c)
-    int c;
+    Char c;
 {
     USE(c);
     ClearScreen();		/* clear the whole real screen */
@@ -3531,7 +3675,7 @@ e_cleardisp(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_int(c)
-    int c;
+    Char c;
 {			
     USE(c);
 #if defined(_MINIX) || defined(WINNT_NATIVE)
@@ -3554,18 +3698,28 @@ e_tty_int(c)
 /*ARGSUSED*/
 CCRETVAL
 e_stuff_char(c)
-     int c;
+     Char c;
 {
 #ifdef TIOCSTI
-     extern int Tty_raw_mode;
      int was_raw = Tty_raw_mode;
+#ifdef WIDE_STRINGS
+     char buf[MB_LEN_MAX];
+     size_t i, len;
+#else
      char ch = (char) c;
+#endif
 
      if (was_raw)
          (void) Cookedmode();
 
      (void) write(SHIN, "\n", 1);
+#ifdef WIDE_STRINGS
+     len = one_wctomb(buf, c & CHAR);
+     for (i = 0; i < len; i++)
+	 (void) ioctl(SHIN, TIOCSTI, (ioctl_t) &buf[i]);
+#else
      (void) ioctl(SHIN, TIOCSTI, (ioctl_t) &ch);
+#endif
 
      if (was_raw)
          (void) Rawmode();
@@ -3578,7 +3732,7 @@ e_stuff_char(c)
 /*ARGSUSED*/
 CCRETVAL
 e_insovr(c)
-    int c;
+    Char c;
 {
     USE(c);
     inputmode = (inputmode == MODE_INSERT ? MODE_REPLACE : MODE_INSERT);
@@ -3588,7 +3742,7 @@ e_insovr(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_dsusp(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3598,7 +3752,7 @@ e_tty_dsusp(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_flusho(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3608,7 +3762,7 @@ e_tty_flusho(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_quit(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3618,7 +3772,7 @@ e_tty_quit(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_tsusp(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3628,7 +3782,7 @@ e_tty_tsusp(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_stopo(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3638,7 +3792,7 @@ e_tty_stopo(c)
 /*ARGSUSED*/
 CCRETVAL
 e_expand_history(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -3649,7 +3803,7 @@ e_expand_history(c)
 /*ARGSUSED*/
 CCRETVAL
 e_magic_space(c)
-    int c;
+    Char c;
 {
     USE(c);
     *LastChar = '\0';		/* just in case */
@@ -3660,7 +3814,7 @@ e_magic_space(c)
 /*ARGSUSED*/
 CCRETVAL
 e_inc_fwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     patlen = 0;
@@ -3671,7 +3825,7 @@ e_inc_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 e_inc_back(c)
-    int c;
+    Char c;
 {
     USE(c);
     patlen = 0;
@@ -3681,7 +3835,7 @@ e_inc_back(c)
 /*ARGSUSED*/
 CCRETVAL
 e_copyprev(c)
-    int c;
+    Char c;
 {
     Char *cp, *oldc, *dp;
 
@@ -3706,7 +3860,7 @@ e_copyprev(c)
 /*ARGSUSED*/
 CCRETVAL
 e_tty_starto(c)
-    int c;
+    Char c;
 {
     USE(c);
     /* do no editing */
@@ -3716,7 +3870,7 @@ e_tty_starto(c)
 /*ARGSUSED*/
 CCRETVAL
 e_load_average(c)
-    int c;
+    Char c;
 {
     USE(c);
     PastBottom();
@@ -3734,7 +3888,7 @@ e_load_average(c)
 /*ARGSUSED*/
 CCRETVAL
 v_chgmeta(c)
-    int c;
+    Char c;
 {
     USE(c);
     /*
@@ -3747,7 +3901,7 @@ v_chgmeta(c)
 /*ARGSUSED*/
 CCRETVAL
 v_delmeta(c)
-    int c;
+    Char c;
 {
     USE(c);
     return(v_action(TCSHOP_DELETE));
@@ -3757,7 +3911,7 @@ v_delmeta(c)
 /*ARGSUSED*/
 CCRETVAL
 v_endword(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar)
@@ -3780,7 +3934,7 @@ v_endword(c)
 /*ARGSUSED*/
 CCRETVAL
 v_eword(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (Cursor == LastChar)
@@ -3802,7 +3956,7 @@ v_eword(c)
 /*ARGSUSED*/
 CCRETVAL
 v_char_fwd(c)
-    int c;
+    Char c;
 {
     Char ch;
 
@@ -3820,7 +3974,7 @@ v_char_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_char_back(c)
-    int c;
+    Char c;
 {
     Char ch;
 
@@ -3837,7 +3991,7 @@ v_char_back(c)
 /*ARGSUSED*/
 CCRETVAL
 v_charto_fwd(c)
-    int c;
+    Char c;
 {
     Char ch;
 
@@ -3852,7 +4006,7 @@ v_charto_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_charto_back(c)
-    int c;
+    Char c;
 {
     Char ch;
 
@@ -3866,7 +4020,7 @@ v_charto_back(c)
 /*ARGSUSED*/
 CCRETVAL
 v_rchar_fwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (srch_char == 0)
@@ -3879,7 +4033,7 @@ v_rchar_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_rchar_back(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (srch_char == 0)
@@ -3892,7 +4046,7 @@ v_rchar_back(c)
 /*ARGSUSED*/
 CCRETVAL
 v_undo(c)
-    int c;
+    Char c;
 {
     int  loop;
     Char *kp, *cp;
@@ -3955,7 +4109,7 @@ v_undo(c)
 /*ARGSUSED*/
 CCRETVAL
 v_ush_meta(c)
-    int c;
+    Char c;
 {
     USE(c);
     return v_search(F_UP_SEARCH_HIST);
@@ -3964,7 +4118,7 @@ v_ush_meta(c)
 /*ARGSUSED*/
 CCRETVAL
 v_dsh_meta(c)
-    int c;
+    Char c;
 {
     USE(c);
     return v_search(F_DOWN_SEARCH_HIST);
@@ -3973,7 +4127,7 @@ v_dsh_meta(c)
 /*ARGSUSED*/
 CCRETVAL
 v_rsrch_fwd(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (patlen == 0) return(CC_ERROR);
@@ -3983,7 +4137,7 @@ v_rsrch_fwd(c)
 /*ARGSUSED*/
 CCRETVAL
 v_rsrch_back(c)
-    int c;
+    Char c;
 {
     USE(c);
     if (patlen == 0) return(CC_ERROR);
@@ -3997,7 +4151,7 @@ v_rsrch_back(c)
  */
 CCRETVAL
 e_copy_to_clipboard(c)
-    int c;
+    Char c;
 {
     USE(c);
     return CC_ERROR;
@@ -4005,7 +4159,7 @@ e_copy_to_clipboard(c)
 
 CCRETVAL
 e_paste_from_clipboard(c)
-    int c;
+    Char c;
 {
     USE(c);
     return (CC_ERROR);
@@ -4013,28 +4167,28 @@ e_paste_from_clipboard(c)
 
 CCRETVAL
 e_dosify_next(c)
-    int c;
+    Char c;
 {
     USE(c);
     return (CC_ERROR);
 }
 CCRETVAL
 e_dosify_prev(c)
-    int c;
+    Char c;
 {
     USE(c);
     return (CC_ERROR);
 }
 CCRETVAL
 e_page_up(c)
-    int c;
+    Char c;
 {
     USE(c);
     return (CC_ERROR);
 }
 CCRETVAL
 e_page_down(c)
-    int c;
+    Char c;
 {
     USE(c);
     return (CC_ERROR);
