@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.h,v 3.31 2001/02/19 23:30:44 kim Exp $ */
+/* $Header: /src/pub/tcsh/ed.h,v 3.37 2004/11/21 04:38:03 christos Exp $ */
 /*
  * ed.h: Editor declarations and globals
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -58,10 +54,10 @@
 #define KEYCMD   unsigned char	/* size needed to index into CcFuncTbl */
  /* Must be unsigned 		       */
 
-typedef CCRETVAL(*PFCmd) __P((int));	/* pointer to function returning CCRETVAL */
+typedef CCRETVAL(*PFCmd) __P((Char));	/* pointer to function returning CCRETVAL */
 
 struct KeyFuncs {		/* for the "bind" shell command */
-    char   *name;		/* function name for bind command */
+    const char *name;		/* function name for bind command */
     int     func;		/* function numeric value */
     char   *desc;		/* description of function */
 };
@@ -153,21 +149,27 @@ EXTERN int HistWhich;		/* Hist_num is saved in this */
 EXTERN char Expand;		/* true if we are expanding a line */
 extern Char HistLit;		/* true if history lines are shown literal */
 EXTERN Char CurrentHistLit;	/* Literal status of current show history line */
+EXTERN int Tty_raw_mode;
 
 /*
  * These are truly extern
  */
 extern int MacroLvl;
+extern Char *litptr[];
+extern int didsetty;
 
 EXTERN Char *KeyMacro[MAXMACROLEVELS];
 
-EXTERN Char **Display;		/* display buffer seed vector */
+/* CHAR_ERR in Display and Vdisplay means the non-first column of a character
+   that is wider than one "regular" position. The cursor should never point
+   in the middle of a multiple-column character. */
+EXTERN eChar **Display;		/* display buffer seed vector */
 EXTERN int CursorV,		/* real cursor vertical (line) */
         CursorH,		/* real cursor horisontal (column) */
         TermV,			/* number of real screen lines
 				 * (sizeof(DisplayBuf) / width */
         TermH;			/* screen width */
-EXTERN Char **Vdisplay;		/* new buffer */
+EXTERN eChar **Vdisplay;	/* new buffer */
 
 /* Variables that describe terminal ability */
 EXTERN int T_Lines, T_Cols;	/* Rows and Cols of the terminal */
@@ -187,39 +189,6 @@ EXTERN Char T_HasMeta;		/* true if we have a meta key */
 #define min(x,y)	(((x)<(y))?(x):(y))
 #define max(x,y)	(((x)>(y))?(x):(y))
 
-/*
- * Terminal dependend data structures
- */
-typedef struct {
-#ifdef WINNT_NATIVE
-    int dummy;
-#else /* !WINNT_NATIVE */
-# if defined(POSIX) || defined(TERMIO)
-#  ifdef POSIX
-    struct termios d_t;
-#  else
-    struct termio d_t;
-#  endif /* POSIX */
-# else /* SGTTY */
-#  ifdef TIOCGETP
-    struct sgttyb d_t;
-#  endif /* TIOCGETP */
-#  ifdef TIOCGETC
-    struct tchars d_tc;
-#  endif /* TIOCGETC */
-#  ifdef TIOCGPAGE
-    struct ttypagestat d_pc;
-#  endif /* TIOCGPAGE */
-#  ifdef TIOCLGET
-    int d_lb;
-#  endif /* TIOCLGET */
-# endif /* POSIX || TERMIO */
-# ifdef TIOCGLTC
-    struct ltchars d_ltc;
-# endif /* TIOCGLTC */
-#endif /* WINNT_NATIVE */
-} ttydata_t;
-
 #define MODE_INSERT	0
 #define MODE_REPLACE	1
 #define MODE_REPLACE_1	2
@@ -244,12 +213,39 @@ typedef struct {
 # define M_NN		3
 #endif /* TERMIO */
 typedef struct { 
-    char *t_name;
+    const char *t_name;
     int  t_setmask;
     int  t_clrmask;
 } ttyperm_t[NN_IO][M_NN];
 
 extern ttyperm_t ttylist;
+#include "ed.term.h"
 #include "ed.decls.h"
 
+#ifndef POSIX
+/*
+ * We don't prototype these, cause some systems have them wrong!
+ */
+extern int   tgetent	__P(());
+extern char *tgetstr	__P(());
+extern int   tgetflag	__P(());
+extern int   tgetnum	__P(());
+extern char *tgoto	__P(());
+# define PUTPURE putpure
+# define PUTRAW putraw
+#else
+extern int   tgetent	__P((char *, const char *));
+extern char *tgetstr	__P((const char *, char **));
+extern int   tgetflag	__P((const char *));
+extern int   tgetnum	__P((const char *));
+extern char *tgoto	__P((const char *, int, int));
+extern void  tputs	__P((const char *, int, void (*)(int)));
+# define PUTPURE ((void (*)__P((int))) putpure)
+# define PUTRAW ((void (*)__P((int))) putraw)
+#endif
+
+#ifdef DSPMBYTE
+extern bool dspmbyte_utf8;
+#endif
+
 #endif /* _h_ed */
