@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.init.c,v 3.44 2001/02/19 23:30:44 kim Exp $ */
+/* $Header: /src/pub/tcsh/ed.init.c,v 3.50 2004/08/04 17:12:27 christos Exp $ */
 /*
  * ed.init.c: Editor initializations
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,10 +32,9 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.init.c,v 3.44 2001/02/19 23:30:44 kim Exp $")
+RCSID("$Id: ed.init.c,v 3.50 2004/08/04 17:12:27 christos Exp $")
 
 #include "ed.h"
-#include "ed.term.h"
 #include "tc.h"
 #include "ed.defns.h"
 
@@ -60,7 +55,6 @@ extern bool GotTermCaps;
 static ttydata_t extty, edtty, tstty;
 #define qutty tstty
 
-extern int insource;
 #define SHTTY (insource ? OLDSTD : SHIN)
 
 #define uc unsigned char
@@ -132,6 +126,7 @@ check_window_size(force)
 #else				/* BSDSIGS */
     (void) sigrelse(SIG_WINDOW);
 #endif /* BSDSIGS */
+    windowchg = 0;
 }
 
 sigret_t
@@ -145,7 +140,7 @@ int snum;
     if (snum > 0)
       sigset(snum, window_change);
 #endif /* UNRELSIGS */
-    check_window_size(0);
+    windowchg = 1;
 #ifndef SIGVOID
     return (snum);
 #endif 
@@ -197,7 +192,7 @@ ed_Setup(rst)
     vdisable = (unsigned char) _POSIX_VDISABLE;
 #endif /* POSIX && _PC_VDISABLE && !BSD4_4 && !WINNT_NATIVE */
 	
-    if ((imode = adrof(STRinputmode)) != NULL) {
+    if ((imode = adrof(STRinputmode)) != NULL && imode->vec != NULL) {
 	if (!Strcmp(*(imode->vec), STRinsert))
 	    inputmode = MODE_INSERT;
 	else if (!Strcmp(*(imode->vec), STRoverwrite))
@@ -517,7 +512,6 @@ Rawmode()
 # endif /* TERMIO || POSIX */
 
 	{
-	    extern int didsetty;
 	    int i;
 
 	    tty_getchar(&tstty, ttychars[TS_IO]);
