@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.inputl.c,v 3.48 2000/06/10 20:14:57 kim Exp $ */
+/* $Header: /src/pub/tcsh/ed.inputl.c,v 3.55 2004/11/20 20:30:46 christos Exp $ */
 /*
  * ed.inputl.c: Input line handling.
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.inputl.c,v 3.48 2000/06/10 20:14:57 kim Exp $")
+RCSID("$Id: ed.inputl.c,v 3.55 2004/11/20 20:30:46 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -46,9 +42,7 @@ RCSID("$Id: ed.inputl.c,v 3.48 2000/06/10 20:14:57 kim Exp $")
 
 /* ed.inputl -- routines to get a single line from the input. */
 
-extern bool tellwhat;
 extern bool MapsAreInited;
-extern bool Tty_raw_mode;
 
 /* mismatched first character */
 static Char mismatch[] = 
@@ -67,6 +61,10 @@ static bool rotate = 0;
 static int
 Repair()
 {
+#ifdef DSPMBYTE
+    if (dspmbyte_utf8)
+	Setutf8lit(InputBuf, LastChar);
+#endif
     if (NeedsRedraw) {
 	ClearLines();
 	ClearDisp();
@@ -85,7 +83,6 @@ Inputl()
 {
     CCRETVAL retval;
     KEYCMD  cmdnum = 0;
-    extern KEYCMD NumFuns;
     unsigned char tch;		/* the place where read() goes */
     Char    ch;
     int     num;		/* how many chars we have read at NL */
@@ -110,7 +107,7 @@ Inputl()
     if (GettingInput)
 	MacroLvl = -1;		/* editor was interrupted during input */
 
-    if (imode) {
+    if (imode && imode->vec != NULL) {
 	if (!Strcmp(*(imode->vec), STRinsert))
 	    inputmode = MODE_INSERT;
 	else if (!Strcmp(*(imode->vec), STRoverwrite))
@@ -229,7 +226,7 @@ Inputl()
 	    curlen = 0;
 	    curchoice = -1;
 	    matchval = 1;
-	    if (crct && (!Strcmp(*(crct->vec), STRcmd) ||
+	    if (crct && crct->vec != NULL && (!Strcmp(*(crct->vec), STRcmd) ||
 			 !Strcmp(*(crct->vec), STRall))) {
                 PastBottom();
 		copyn(Origin, InputBuf, INBUFSIZE);
@@ -284,7 +281,8 @@ Inputl()
 		    }
 		    flush();
 		}
-	    } else if (crct && !Strcmp(*(crct->vec), STRcomplete)) {
+	    } else if (crct && crct->vec != NULL &&
+		!Strcmp(*(crct->vec), STRcomplete)) {
                 if (LastChar > InputBuf && LastChar[-1] == '\n') {
                     LastChar[-1] = '\0';
                     LastChar--;
@@ -397,12 +395,12 @@ Inputl()
 	    expnum = (int) (Cursor - InputBuf);
 	    switch (matchval = tenematch(InputBuf, Cursor-InputBuf, fn)){
 	    case 1:
-		if (non_unique_match && matchbeep &&
+		if (non_unique_match && matchbeep && matchbeep->vec != NULL &&
 		    (Strcmp(*(matchbeep->vec), STRnotunique) == 0))
 		    SoundBeep();
 		break;
 	    case 0:
-		if (matchbeep) {
+		if (matchbeep && matchbeep->vec != NULL) {
 		    if (Strcmp(*(matchbeep->vec), STRnomatch) == 0 ||
 			Strcmp(*(matchbeep->vec), STRambiguous) == 0 ||
 			Strcmp(*(matchbeep->vec), STRnotunique) == 0)
@@ -417,7 +415,7 @@ Inputl()
 		    SoundBeep();
 		    break;
 		}
-		if (matchbeep) {
+		if (matchbeep && matchbeep->vec != NULL) {
 		    if ((Strcmp(*(matchbeep->vec), STRambiguous) == 0 ||
 			 Strcmp(*(matchbeep->vec), STRnotunique) == 0))
 			SoundBeep();
@@ -430,7 +428,8 @@ Inputl()
 		 * (PWP: this is the best feature addition to tcsh I have 
 		 * seen in many months.)
 		 */
-		if (autol && (Strcmp(*(autol->vec), STRambiguous) != 0 || 
+		if (autol && autol->vec != NULL && 
+		    (Strcmp(*(autol->vec), STRambiguous) != 0 || 
 				     expnum == Cursor - InputBuf)) {
 		    PastBottom();
 		    fn = (retval == CC_COMPLETE_ALL) ? LIST_ALL : LIST;
@@ -438,6 +437,10 @@ Inputl()
 		}
 		break;
 	    }
+#ifdef DSPMBYTE
+	    if (dspmbyte_utf8)
+		Setutf8lit(InputBuf, LastChar);
+#endif
 	    if (NeedsRedraw) {
 		PastBottom();
 		ClearLines();
@@ -651,7 +654,7 @@ RunCommand(str)
 static int
 GetNextCommand(cmdnum, ch)
     KEYCMD *cmdnum;
-    register Char *ch;
+    Char *ch;
 {
     KEYCMD  cmd = 0;
     int     num;
@@ -664,6 +667,8 @@ GetNextCommand(cmdnum, ch)
 	if (
 #ifdef DSPMBYTE
 	     _enable_mbdisp &&
+#else
+	     MB_LEN_MAX == 1 &&
 #endif
 	     !adrof(STRnokanji) && (*ch & META)) {
 	    MetaNext = 0;
@@ -682,7 +687,11 @@ GetNextCommand(cmdnum, ch)
 	if (*ch < NT_NUM_KEYS)
 	    cmd = CurrentKeyMap[*ch];
 	else
+#ifdef WINNT_NATIVE
 	    cmd = CurrentKeyMap[(unsigned char) *ch];
+#else
+	    cmd = F_INSERT;
+#endif
 	if (cmd == F_XKEY) {
 	    XmapVal val;
 	    CStr cstr;
@@ -712,11 +721,16 @@ GetNextCommand(cmdnum, ch)
 
 int
 GetNextChar(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register int num_read;
+    int num_read;
     int     tried = 0;
+#ifdef WIDE_STRINGS
+    char cbuf[MB_LEN_MAX];
+    size_t cbp;
+#else
     unsigned char tcp;
+#endif
 
     for (;;) {
 	if (MacroLvl < 0) {
@@ -737,36 +751,71 @@ GetNextChar(cp)
     if (Rawmode() < 0)		/* make sure the tty is set up correctly */
 	return 0;		/* oops: SHIN was closed */
 
-#ifdef WINNT
+#ifdef WINNT_NATIVE
     __nt_want_vcode = 1;
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
+#ifdef SIG_WINDOW
+    if (windowchg)
+	(void) check_window_size(0);	/* for window systems */
+#endif /* SIG_WINDOW */
+#ifdef WIDE_STRINGS
+    /* This is the part that doesn't work with WINNT_NATIVE */
+    cbp = 0;
+    for (;;) {
+	while ((num_read = read(SHIN, cbuf + cbp, 1)) == -1) {
+	    if (errno == EINTR)
+		continue;
+	    if (!tried && fixio(SHIN, errno) != -1)
+		tried = 1;
+	    else {
+# ifdef convex
+		/* need to print error message in case the file is migrated */
+		if (errno != EINTR)
+		    stderror(ERR_SYSTEM, progname, strerror(errno));
+# endif  /* convex */
+		*cp = '\0'; /* Loses possible partial character */
+		return -1;
+	    }
+	}
+	cbp++;
+	if (mbtowc(cp, cbuf, cbp) == -1) {
+	    mbtowc(NULL, NULL, 0);
+	    if (cbp < MB_LEN_MAX)
+		continue; /* Maybe a partial character */
+	    /* And drop the following bytes, if any */
+	    *cp = (unsigned char)*cbuf;
+	}
+	break;
+    }
+#else /* not WIDE_STRINGS */
     while ((num_read = read(SHIN, (char *) &tcp, 1)) == -1) {
 	if (errno == EINTR)
 	    continue;
 	if (!tried && fixio(SHIN, errno) != -1)
 	    tried = 1;
 	else {
-#ifdef convex
+# ifdef convex
             /* need to print error message in case the file is migrated */
             if (errno != EINTR)
                 stderror(ERR_SYSTEM, progname, strerror(errno));
-#endif  /* convex */
-#ifdef WINNT
+# endif  /* convex */
+# ifdef WINNT_NATIVE
 	    __nt_want_vcode = 0;
-#endif /* WINNT */
+# endif /* WINNT_NATIVE */
 	    *cp = '\0';
 	    return -1;
 	}
     }
-#ifdef WINNT
+# ifdef WINNT_NATIVE
     if (__nt_want_vcode == 2)
 	*cp = __nt_vcode;
     else
 	*cp = tcp;
     __nt_want_vcode = 0;
-#else
+# else
     *cp = tcp;
-#endif /* WINNT */
+# endif /* WINNT_NATIVE */
+#endif /* not WIDE_STRINGS */
     return num_read;
 }
 
@@ -810,12 +859,12 @@ SpellLine(cmdonly)
 	mismatch[1] = HISTSUB;
 	if (!Strchr(mismatch, *argptr) &&
 	    (!cmdonly || starting_a_command(argptr, InputBuf))) {
-#ifdef WINNT
+#ifdef WINNT_NATIVE
 	    /*
 	     * This hack avoids correcting drive letter changes
 	     */
 	    if((Cursor - InputBuf) != 2 || (char)InputBuf[1] != ':')
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
 	    {
 #ifdef HASH_SPELL_CHECK
 		Char save;
