@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.refresh.c,v 3.28 2000/11/11 23:03:34 christos Exp $ */
+/* $Header: /src/pub/tcsh/ed.refresh.c,v 3.34 2004/11/21 04:38:03 christos Exp $ */
 /*
  * ed.refresh.c: Lower level screen refreshing functions
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.refresh.c,v 3.28 2000/11/11 23:03:34 christos Exp $")
+RCSID("$Id: ed.refresh.c,v 3.34 2004/11/21 04:38:03 christos Exp $")
 
 #include "ed.h"
 /* #define DEBUG_UPDATE */
@@ -45,31 +41,37 @@ RCSID("$Id: ed.refresh.c,v 3.28 2000/11/11 23:03:34 christos Exp $")
 
 /* refresh.c -- refresh the current set of lines on the screen */
 
-Char   *litptr[256];
+Char   *litptr[512];
 static int vcursor_h, vcursor_v;
 static int rprompt_h, rprompt_v;
 
-static	void	Draw 			__P((int));
-static	void	Vdraw 			__P((int));
+static	size_t	char_width		__P((Char));
+static	void	Draw 			__P((Char));
+static	void	Vdraw 			__P((Char));
 static	void	RefreshPromptpart	__P((Char *));
-static	void	update_line 		__P((Char *, Char *, int));
-static	void	str_insert		__P((Char *, int, int, Char *, int));
-static	void	str_delete		__P((Char *, int, int, int));
-static	void	str_cp			__P((Char *, Char *, int));
-static	void	PutPlusOne		__P((int));
-static	void	cpy_pad_spaces		__P((Char *, Char *, int));
+static	void	update_line 		__P((eChar *, eChar *, int));
+static	void	str_insert		__P((eChar *, int, int, eChar *, int));
+static	void	str_delete		__P((eChar *, int, int, int));
+static	void	str_cp			__P((eChar *, eChar *, int));
+#ifndef WINNT_NATIVE
+static
+#else
+extern
+#endif
+	void    PutPlusOne      __P((Char));
+static	void	cpy_pad_spaces		__P((eChar *, eChar *, int));
 #if defined(DSPMBYTE)
-static	Char 	*update_line_fix_mbyte_point __P((Char *, Char *, int));
+static	eChar 	*update_line_fix_mbyte_point __P((eChar *, eChar *, int));
 #endif
 #if defined(DEBUG_UPDATE) || defined(DEBUG_REFRESH) || defined(DEBUG_LITERAL)
 static	void	dprintf			__P((char *, ...));
 #ifdef DEBUG_UPDATE
-static	void	dprintstr		__P((char *, Char *, Char *));
+static	void	dprintstr		__P((char *, const eChar *, const eChar *));
 
 static void
 dprintstr(str, f, t)
 char *str;
-Char *f, *t;
+const eChar *f, *t;
 {
     dprintf("%s:\"", str);
     while (f < t)
@@ -118,11 +120,28 @@ dprintf(va_list)
 }
 #endif  /* DEBUG_UPDATE || DEBUG_REFRESH || DEBUG_LITERAL */
 
+/* Returns 1 for NUL and nonprinting characters */
+static size_t
+char_width(c)
+     Char c;
+{
+#if defined (WIDE_STRINGS) && defined (HAVE_WCWIDTH)
+    int r;
+
+    r = wcwidth (c);
+    if (r <= 0)
+	r = 1;
+    return r;
+#else
+    return 1;
+#endif
+}
+
 static void
 Draw(c)				/* draw c, expand tabs, ctl chars */
-    register int c;
+    Char c;
 {
-    register Char ch = c & CHAR;
+    Char ch = c & CHAR;
 
     if (Isprint(ch)) {
 	Vdraw(c);
@@ -191,7 +210,7 @@ Draw(c)				/* draw c, expand tabs, ctl chars */
 	return;
     }
 #endif
-    else {
+    else { /* If WIDE_STRINGS, truncates characters above U+00FF */
 	Vdraw('\\');
 	Vdraw(((c >> 6) & 7) + '0');
 	Vdraw(((c >> 3) & 7) + '0');
@@ -201,18 +220,26 @@ Draw(c)				/* draw c, expand tabs, ctl chars */
 
 static void
 Vdraw(c)			/* draw char c onto V lines */
-    register int c;
+    Char c;
 {
+    size_t width;
 #ifdef DEBUG_REFRESH
 # ifdef SHORT_STRINGS
-    dprintf("Vdrawing %6.6o '%c'\r\n", c, c & ASCII);
+    dprintf("Vdrawing %6.6o '%c'\r\n", (unsigned)c, (int)(c & ASCII));
 # else
-    dprintf("Vdrawing %3.3o '%c'\r\n", c, c);
+    dprintf("Vdrawing %3.3o '%c'\r\n", (unsigned)c, (int)c);
 # endif /* SHORT_STRNGS */
 #endif  /* DEBUG_REFRESH */
 
+    width = char_width(c);
+    /* Hopefully this is what all the terminals do with multi-column characters
+       that "span line breaks". */
+    while (vcursor_h + width > (size_t)TermH)
+	Vdraw(' ');
     Vdisplay[vcursor_v][vcursor_h] = (Char) c;
     vcursor_h++;		/* advance to next place */
+    while (--width != 0)
+	Vdisplay[vcursor_v][vcursor_h++] = CHAR_ERR;
     if (vcursor_h >= TermH) {
 	Vdisplay[vcursor_v][TermH] = '\0';	/* assure end of line */
 	vcursor_h = 0;		/* reset it. */
@@ -227,6 +254,7 @@ Vdraw(c)			/* draw char c onto V lines */
     }
 }
 
+static unsigned int litnum = 0;
 /*
  *  RefreshPromptpart()
  *	draws a prompt element, expanding literals (we know it's ASCIZ)
@@ -235,8 +263,7 @@ static void
 RefreshPromptpart(buf)
     Char *buf;
 {
-    register Char *cp;
-    static unsigned int litnum = 0;
+    Char *cp;
     if (buf == NULL)
     {
       litnum = 0;
@@ -255,7 +282,7 @@ RefreshPromptpart(buf)
 	    while (*cp & LITERAL)
 		cp++;
 	    if (*cp)
-		Vdraw((int) (litnum++ | LITERAL));
+		Vdraw((Char) (litnum++ | LITERAL));
 	    else {
 		/*
 		 * XXX: This is a bug, we lose the last literal, if it is not
@@ -276,12 +303,16 @@ RefreshPromptpart(buf)
  *	virtual image. The routine to re-draw a line can be replaced
  *	easily in hopes of a smarter one being placed there.
  */
-static int OldvcV = 0;
+#ifndef WINNT_NATIVE
+static
+#endif
+int OldvcV = 0;
+
 void
 Refresh()
 {
-    register int cur_line;
-    register Char *cp;
+    int cur_line;
+    Char *cp;
     int     cur_h, cur_v = 0, new_vcv;
     int     rhdiff;
     Char    oldgetting;
@@ -310,11 +341,38 @@ Refresh()
 
     /* draw the current input buffer */
     for (cp = InputBuf; (cp < LastChar); cp++) {
+#if defined(DSPMBYTE)
+	if (vcursor_h + 1 >= TermH && Ismbyte1(*cp)) {
+	    if (dspmbyte_utf8) {
+		int i;
+		for (i = 1 ; cp + i < LastChar; i++)
+		    if (!Ismbyte2(cp[i]))
+			break;
+		if (i > 1 && !(cp[i - 2] & LITERAL))
+		    Vdraw(' ');
+	    } else {
+		Vdraw(' ');
+	    }
+	}
+# endif
 	if (cp == Cursor) {
 	    cur_h = vcursor_h;	/* save for later */
 	    cur_v = vcursor_v;
 	}
-	Draw(*cp);
+	if (*cp & LITERAL) {
+	    Char *litstart = cp;
+	    int i;
+	    for (i = 1; cp + i < LastChar; i++)
+		if (!(cp[i] & LITERAL))
+		    break;
+	    if (cp + i < LastChar && litnum < sizeof(litptr)/sizeof(*litptr)) {
+		litptr[litnum] = litstart;
+		Vdraw((Char) (litnum++ | LITERAL));
+	    }
+	    cp += i;
+	} else {
+	    Draw(*cp);
+	}
     }
 
     if (cur_h == -1) {		/* if I haven't been set yet, I'm at the end */
@@ -341,7 +399,8 @@ Refresh()
     new_vcv = vcursor_v;	/* must be done BEFORE the NUL is written */
     Vdraw('\0');		/* put NUL on end */
 
-#ifdef DEBUG_REFRESH
+#if 0 && defined (DEBUG_REFRESH)
+    /* Doesn't work because Vdisplay is eChar **, not Char ** */
     dprintf("TermH=%d, vcur_h=%d, vcur_v=%d, Vdisplay[0]=\r\n:%80.80s:\r\n",
 	    TermH, vcursor_h, vcursor_v, short2str(Vdisplay[0]));
 #endif /* DEBUG_REFRESH */
@@ -363,10 +422,6 @@ Refresh()
 	 * screen line, it won't be a NUL or some old leftover stuff.
 	 */
 	cpy_pad_spaces(Display[cur_line], Vdisplay[cur_line], TermH);
-#ifdef notdef
-	(void) Strncpy(Display[cur_line], Vdisplay[cur_line], (size_t) TermH);
-	Display[cur_line][TermH] = '\0';	/* just in case */
-#endif
     }
 #ifdef DEBUG_REFRESH
     dprintf("\r\nvcursor_v = %d, OldvcV = %d, cur_line = %d\r\n",
@@ -374,7 +429,7 @@ Refresh()
 #endif /* DEBUG_REFRESH */
     if (OldvcV > new_vcv) {
 	for (; cur_line <= OldvcV; cur_line++) {
-	    update_line(Display[cur_line], STRNULL, cur_line);
+	    update_line(Display[cur_line], eSTRNULL, cur_line);
 	    *Display[cur_line] = '\0';
 	}
     }
@@ -416,12 +471,12 @@ PastBottom()
    maximum length of d is dlen */
 static void
 str_insert(d, dat, dlen, s, num)
-    register Char *d;
-    register int dat, dlen;
-    register Char *s;
-    register int num;
+    eChar *d;
+    int dat, dlen;
+    eChar *s;
+    int num;
 {
-    register Char *a, *b;
+    eChar *a, *b;
 
     if (num <= 0)
 	return;
@@ -462,10 +517,10 @@ str_insert(d, dat, dlen, s, num)
 /* delete num characters d at dat, maximum length of d is dlen */
 static void
 str_delete(d, dat, dlen, num)
-    register Char *d;
-    register int dat, dlen, num;
+    eChar *d;
+    int dat, dlen, num;
 {
-    register Char *a, *b;
+    eChar *a, *b;
 
     if (num <= 0)
 	return;
@@ -495,8 +550,8 @@ str_delete(d, dat, dlen, num)
 
 static void
 str_cp(a, b, n)
-    register Char *a, *b;
-    register int n;
+    eChar *a, *b;
+    int n;
 {
     while (n-- && *b)
 	*a++ = *b++;
@@ -504,9 +559,9 @@ str_cp(a, b, n)
 
 
 #if defined(DSPMBYTE) /* BY TAGA Nayuta VERY THANKS */
-static Char *
+static eChar *
 update_line_fix_mbyte_point(start, target, d)
-     Char *start, *target;
+     eChar *start, *target;
      int d;
 {
     if (_enable_mbdisp) {
@@ -550,16 +605,16 @@ new:	eddie> Oh, my little buggy says to me, as lurgid as
 
 static void			/* could be changed to make it smarter */
 update_line(old, new, cur_line)
-    register Char *old, *new;
+    eChar *old, *new;
     int     cur_line;
 {
-    register Char *o, *n, *p, c;
-    Char   *ofd, *ols, *oe, *nfd, *nls, *ne;
-    Char   *osb, *ose, *nsb, *nse;
+    eChar *o, *n, *p, c;
+    eChar  *ofd, *ols, *oe, *nfd, *nls, *ne;
+    eChar  *osb, *ose, *nsb, *nse;
     int     fx, sx;
 
     /*
-     * find first diff
+     * find first diff (won't be CHAR_ERR in either line)
      */
     for (o = old, n = new; *o && (*o == *n); o++, n++)
 	continue;
@@ -610,8 +665,16 @@ update_line(old, new, cur_line)
      */
     while ((o > ofd) && (n > nfd) && (*--o == *--n))
 	continue;
-    ols = ++o;
-    nls = ++n;
+    if (*o != *n) {
+	o++;
+	n++;
+    }
+    while (*o == CHAR_ERR) {
+	o++;
+	n++;
+    }
+    ols = o;
+    nls = n;
 
     /*
      * find same begining and same end
@@ -1089,10 +1152,10 @@ update_line(old, new, cur_line)
 
 static void
 cpy_pad_spaces(dst, src, width)
-    register Char *dst, *src;
-    register int width;
+    eChar *dst, *src;
+    int width;
 {
-    register int i;
+    int i;
 
     for (i = 0; i < width; i++) {
 	if (*src == (Char) 0)
@@ -1110,8 +1173,8 @@ cpy_pad_spaces(dst, src, width)
 void
 RefCursor()
 {				/* only move to new cursor pos */
-    register Char *cp, c;
-    register int h, th, v;
+    Char *cp, c;
+    int h, th, v;
 
     /* first we must find where the cursor is... */
     h = 0;
@@ -1119,10 +1182,17 @@ RefCursor()
     th = TermH;			/* optimize for speed */
 
     for (cp = PromptBuf; *cp; cp++) {	/* do prompt */
+	size_t width;
+	
 	if (*cp & LITERAL)
 	    continue;
 	c = *cp & CHAR;		/* extra speed plus strip the inverse */
-	h++;			/* all chars at least this long */
+	width = char_width (c);
+	if (h + width > (size_t)TermH) {
+	    h = 0;
+	    v++;
+	}
+	h += width;
 
 	/* from wolman%crltrx.DEC@decwrl.dec.com (Alec Wolman) */
 	/* lets handle newline as part of the prompt */
@@ -1160,8 +1230,17 @@ RefCursor()
     }
 
     for (cp = InputBuf; cp < Cursor; cp++) {	/* do input buffer to Cursor */
+	size_t width;
+
+	if (*cp & LITERAL)
+	    continue;
 	c = *cp & CHAR;		/* extra speed plus strip the inverse */
-	h++;			/* all chars at least this long */
+	width = char_width (c);
+	if (h + width > (size_t)TermH) {
+	    h = 0;
+	    v++;
+	}
+	h += width;
 
 	if (c == '\n') {	/* handle newline in data part too */
 	    h = 0;
@@ -1189,6 +1268,20 @@ RefCursor()
 	    }
 	}
 
+#if defined(DSPMBYTE)
+        if (h + 1 >= th && cp + 1 < LastChar && Ismbyte1(cp[1])) {
+            if (dspmbyte_utf8) {
+                int i;
+                for (i = 2 ; cp + i < LastChar; i++)
+                    if (!Ismbyte2(cp[i]))
+                        break;
+                if (i > 2 && !(cp[i - 2] & LITERAL))
+		    h++;
+            } else
+		h++;
+        }
+#endif
+
 	if (h >= th) {		/* check, extra long tabs picked up here also */
 	    h = 0;
 	    v++;
@@ -1201,12 +1294,20 @@ RefCursor()
     flush();
 }
 
+#ifndef WINTT_NATIVE
 static void
 PutPlusOne(c)
-    int    c;
+    Char   c;
 {
-    (void) putraw(c);
+    size_t width;
+
+    width = char_width(c);
+    while (CursorH + width > (size_t)TermH)
+	PutPlusOne(' ');
+    (void) putwraw(c);
     Display[CursorV][CursorH++] = (Char) c;
+    while (--width != 0)
+	Display[CursorV][CursorH++] = CHAR_ERR;
     if (CursorH >= TermH) {	/* if we must overflow */
 	CursorH = 0;
 	CursorV++;
@@ -1223,12 +1324,13 @@ PutPlusOne(c)
 	}
     }
 }
+#endif
 
 void
 RefPlusOne()
 {				/* we added just one char, handle it fast.
 				 * assumes that screen cursor == real cursor */
-    register Char c, mc;
+    Char c, mc;
 
     c = Cursor[-1] & CHAR;	/* the char we just added */
 
@@ -1276,7 +1378,7 @@ RefPlusOne()
 	PutPlusOne(c);
     }
 #endif
-    else {
+    else { /* If WIDE_STRINGS, truncates characters above U+00FF */
 	PutPlusOne('\\');
 	PutPlusOne(((c >> 6) & 7) + '0');
 	PutPlusOne(((c >> 3) & 7) + '0');
@@ -1290,7 +1392,7 @@ RefPlusOne()
 void
 ClearDisp()
 {
-    register int i;
+    int i;
 
     CursorV = 0;		/* clear the display buffer */
     CursorH = 0;
@@ -1302,7 +1404,7 @@ ClearDisp()
 void
 ClearLines()
 {				/* Make sure all lines are *really* blank */
-    register int i;
+    int i;
 
     if (T_CanCEOL) {
 	/*
