@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.screen.c,v 3.50 2003/02/08 20:03:25 christos Exp $ */
+/* $Header: /src/pub/tcsh/ed.screen.c,v 3.56 2004/11/21 04:38:03 christos Exp $ */
 /*
  * ed.screen.c: Editor/termcap-curses interface
  */
@@ -32,35 +32,12 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.screen.c,v 3.50 2003/02/08 20:03:25 christos Exp $")
+RCSID("$Id: ed.screen.c,v 3.56 2004/11/21 04:38:03 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
 #include "ed.defns.h"
 
-#ifndef POSIX
-/*
- * We don't prototype these, cause some systems have them wrong!
- */
-extern int   tgetent	__P(());
-extern char *tgetstr	__P(());
-extern int   tgetflag	__P(());
-extern int   tgetnum	__P(());
-extern char *tgoto	__P(());
-# define PUTPURE putpure
-# define PUTRAW putraw
-#else
-extern int   tgetent	__P((char *, char *));
-extern char *tgetstr	__P((char *, char **));
-extern int   tgetflag	__P((char *));
-extern int   tgetnum	__P((char *));
-extern char *tgoto	__P((char *, int, int));
-extern void  tputs	__P((char *, int, void (*)(int)));
-# define PUTPURE ((void (*)__P((int))) putpure)
-# define PUTRAW ((void (*)__P((int))) putraw)
-#endif
-
-
 /* #define DEBUG_LITERAL */
 
 /*
@@ -77,8 +54,8 @@ extern void  tputs	__P((char *, int, void (*)(int)));
 #define Val(a) tval[a].val
 
 static struct {
-    char   *b_name;
-    int     b_rate;
+    const char   *b_name;
+    speed_t b_rate;
 }       baud_rate[] = {
 
 #ifdef B0
@@ -187,7 +164,7 @@ static struct {
 #define T_at7   37
 #define T_str   38
 static struct termcapstr {
-    char   *name;
+    const char   *name;
     char   *long_name;
     char   *str;
 } tstr[T_str + 1];
@@ -201,7 +178,7 @@ static struct termcapstr {
 #define T_xn	5
 #define T_val	6
 static struct termcapval {
-    char   *name;
+    const char   *name;
     char   *long_name;
     int     val;
 } tval[T_val + 1];
@@ -446,7 +423,7 @@ TCalloc(t, cap)
 /*ARGSUSED*/
 void
 TellTC(what)
-    char   *what;
+    const char   *what;
 {
     struct termcapstr *t;
 
@@ -479,9 +456,9 @@ TellTC(what)
 static void
 ReBufferDisplay()
 {
-    register int i;
-    Char  **b;
-    Char  **bufp;
+    int i;
+    eChar **b;
+    eChar **bufp;
 
     b = Display;
     Display = NULL;
@@ -499,14 +476,14 @@ ReBufferDisplay()
     }
     TermH = Val(T_co);
     TermV = (INBUFSIZE * 4) / TermH + 1;
-    b = (Char **) xmalloc((size_t) (sizeof(Char *) * (TermV + 1)));
+    b = (eChar **) xmalloc((size_t) (sizeof(*b) * (TermV + 1)));
     for (i = 0; i < TermV; i++)
-	b[i] = (Char *) xmalloc((size_t) (sizeof(Char) * (TermH + 1)));
+	b[i] = (eChar *) xmalloc((size_t) (sizeof(*b[i]) * (TermH + 1)));
     b[TermV] = NULL;
     Display = b;
-    b = (Char **) xmalloc((size_t) (sizeof(Char *) * (TermV + 1)));
+    b = (eChar **) xmalloc((size_t) (sizeof(*b) * (TermV + 1)));
     for (i = 0; i < TermV; i++)
-	b[i] = (Char *) xmalloc((size_t) (sizeof(Char) * (TermH + 1)));
+	b[i] = (eChar *) xmalloc((size_t) (sizeof(*b[i]) * (TermH + 1)));
     b[TermV] = NULL;
     Vdisplay = b;
 }
@@ -595,7 +572,7 @@ EchoTC(v)
     int     arg_need, arg_cols, arg_rows;
     int     verbose = 0, silent = 0;
     char   *area;
-    static char *fmts = "%s\n", *fmtd = "%d\n";
+    static const char *fmts = "%s\n", *fmtd = "%d\n";
     struct termcapstr *t;
     char    buf[TC_BUFSIZE];
 
@@ -807,17 +784,17 @@ static struct {
     int	    type;
 } arrow[] = {
 #define A_K_DN	0
-    { STRdown,	T_kd },
+    { STRdown,	T_kd, { 0 }, 0 },
 #define A_K_UP	1
-    { STRup,	T_ku },
+    { STRup,	T_ku, { 0 }, 0 },
 #define A_K_LT	2
-    { STRleft,	T_kl },
+    { STRleft,	T_kl, { 0 }, 0 },
 #define A_K_RT	3
-    { STRright, T_kr },
+    { STRright, T_kr, { 0 }, 0 },
 #define A_K_HO  4
-    { STRhome,  T_kh },
+    { STRhome,  T_kh, { 0 }, 0 },
 #define A_K_EN  5
-    { STRend,   T_at7}
+    { STRend,   T_at7, { 0 }, 0}
 };
 #define A_K_NKEYS 6
 
@@ -1115,9 +1092,19 @@ MoveToLine(where)		/* move to line <where> (first line == 0) */
     if (del > 0) {
 	while (del > 0) {
 	    if ((T_Margin & MARGIN_AUTO) && Display[CursorV][0] != '\0') {
+		size_t h;
+
+		for (h = TermH - 1; h > 0 && Display[CursorV][h] == CHAR_ERR;
+		     h--)
+		    ;
 		/* move without newline */
-		MoveToChar(TermH - 1);
-		so_write(&Display[CursorV][CursorH], 1); /* updates CursorH/V*/
+		MoveToChar(h);
+#ifdef DSPMBYTE
+		if (h > 0 && Ismbyte2(Display[CursorV][CursorH]))
+		    MoveToChar(h - 1);
+#else
+		so_write(&Display[CursorV][CursorH], TermH - CursorH); /* updates CursorH/V*/
+#endif
 		del--;
 	    }
 	    else {
@@ -1187,11 +1174,12 @@ mc_again:
 		    && !_enable_mbdisp
 #endif /* DSPMBYTE */
 		) {
-		    if ((CursorH & 0370) != (where & 0370)) {
+		    if ((CursorH & 0370) != (where & ~0x7)
+			&& Display[CursorV][where & ~0x7] != CHAR_ERR) {
 			/* if not within tab stop */
-			for (i = (CursorH & 0370); i < (where & 0370); i += 8)
+			for (i = (CursorH & 0370); i < (where & ~0x7); i += 8)
 			    (void) putraw('\t');	/* then tab over */
-			CursorH = where & 0370;
+			CursorH = where & ~0x7;
 			/* Note: considering that we often want to go to
 			   TermH - 1 for the wrapping, it would be nice to
 			   optimize this case by tabbing to the last column
@@ -1226,8 +1214,8 @@ mc_again:
 
 void
 so_write(cp, n)
-    register Char *cp;
-    register int n;
+    eChar *cp;
+    int n;
 {
     if (n <= 0)
 	return;			/* catch bugs */
@@ -1241,21 +1229,22 @@ so_write(cp, n)
     }
 
     do {
-	if (*cp & LITERAL) {
-	    extern Char *litptr[];
-	    Char   *d;
-
+	if (*cp != CHAR_ERR) {
+	    if (*cp & LITERAL) {
+		Char   *d;
+		
 #ifdef DEBUG_LITERAL
-	    xprintf("so: litnum %d, litptr %x\r\n",
-		    *cp & CHAR, litptr[*cp & CHAR]);
+		xprintf("so: litnum %d, litptr %x\r\n",
+			(int)(*cp & CHAR), litptr[*cp & CHAR]);
 #endif /* DEBUG_LITERAL */
-	    for (d = litptr[*cp++ & CHAR]; *d & LITERAL; d++)
-		(void) putraw(*d & CHAR);
-	    (void) putraw(*d);
-
+		for (d = litptr[*cp & CHAR]; *d & LITERAL; d++)
+		    (void) putwraw(*d & CHAR);
+		(void) putwraw(*d);
+	    }
+	    else
+		(void) putwraw(*cp);
 	}
-	else
-	    (void) putraw(*cp++);
+	cp++;
 	CursorH++;
     } while (--n);
 
@@ -1265,12 +1254,21 @@ so_write(cp, n)
 	    CursorV++;
 	    if (T_Margin & MARGIN_MAGIC) {
 		/* force the wrap to avoid the "magic" situation */
-		Char c;
-		if ((c = Display[CursorV][CursorH]) != '\0')
+		eChar c;
+		if ((c = Display[CursorV][CursorH]) != '\0') {
 		    so_write(&c, 1);
-		else
+#ifdef DSPMBYTE
+		    if (CursorH > 0 && (c = Display[CursorV][CursorH]) != 0)
+			if (Ismbyte2(c))
+			    so_write(&c, 1);
+#endif
+		    while(Display[CursorV][CursorH] == CHAR_ERR)
+			CursorH++;
+		}
+		else {
 		    (void) putraw(' ');
-		CursorH = 1;
+		    CursorH = 1;
+		}
 	    }
 	}
 	else			/* no wrap, but cursor stays on screen */
@@ -1321,8 +1319,8 @@ DeleteChars(num)		/* deletes <num> characters */
 
 void
 Insert_write(cp, num)		/* Puts terminal in insert character mode, */
-    register Char *cp;
-    register int num;		/* or inserts num characters in the line */
+    eChar *cp;
+    int num;		/* or inserts num characters in the line */
 {
     if (num <= 0)
 	return;
@@ -1352,10 +1350,7 @@ Insert_write(cp, num)		/* Puts terminal in insert character mode, */
     if (GoodStr(T_im) && GoodStr(T_ei)) { /* if I have insert mode */
 	(void) tputs(Str(T_im), 1, PUTPURE);
 
-	CursorH += num;
-	do 
-	    (void) putraw(*cp++);
-	while (--num);
+	so_write(cp, num);	/* this updates CursorH/V */
 
 	if (GoodStr(T_ip))	/* have to make num chars insert */
 	    (void) tputs(Str(T_ip), 1, PUTPURE);
@@ -1368,9 +1363,7 @@ Insert_write(cp, num)		/* Puts terminal in insert character mode, */
 	if (GoodStr(T_ic))	/* have to make num chars insert */
 	    (void) tputs(Str(T_ic), 1, PUTPURE);	/* insert a char */
 
-	(void) putraw(*cp++);
-
-	CursorH++;
+	so_write(cp++, 1);	/* this updates CursorH/V */
 
 	if (GoodStr(T_ip))	/* have to make num chars insert */
 	    (void) tputs(Str(T_ip), 1, PUTPURE);/* pad the inserted char */
@@ -1383,7 +1376,7 @@ void
 ClearEOL(num)			/* clear to end of line.  There are num */
     int     num;		/* characters to clear */
 {
-    register int i;
+    int i;
 
     if (num <= 0)
 	return;
@@ -1442,8 +1435,8 @@ ClearToBottom()
 void
 GetTermCaps()
 {				/* read in the needed terminal capabilites */
-    register int i;
-    char   *ptr;
+    int i;
+    const char   *ptr;
     char    buf[TC_BUFSIZE];
     static char bp[TC_BUFSIZE];
     char   *area;
