@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.term.c,v 1.26 2001/08/06 23:51:09 christos Exp $ */
+/* $Header: /src/pub/tcsh/ed.term.c,v 1.30 2004/08/04 17:12:28 christos Exp $ */
 /*
  * ed.term.c: Low level terminal interface
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -37,10 +33,9 @@
 #include "sh.h"
 #ifndef WINNT_NATIVE
 
-RCSID("$Id: ed.term.c,v 1.26 2001/08/06 23:51:09 christos Exp $")
+RCSID("$Id: ed.term.c,v 1.30 2004/08/04 17:12:28 christos Exp $")
 
 #include "ed.h"
-#include "ed.term.h"
 
 int didsetty = 0;
 ttyperm_t ttylist = {   
@@ -90,7 +85,7 @@ ttyperm_t ttylist = {
 };
 
 static struct tcshmodes {
-    char *m_name;
+    const char *m_name;
 #ifdef SOLARIS2
     unsigned long m_value;
 #else /* !SOLARIS2 */
@@ -558,7 +553,7 @@ static struct tcshmodes {
 
 #if defined(EAGAIN) && defined(EWOULDBLOCK) && (EWOULDBLOCK != EAGAIN)
 # define OKERROR(e) (((e) == EAGAIN) || ((e) == EWOULDBLOCK) || ((e) == EINTR))
-#elif defined(EGAIN)
+#elif defined(EAGAIN)
 # define OKERROR(e) (((e) == EAGAIN) || ((e) == EINTR))
 #elif defined(EWOULDBLOCK)
 # define OKERROR(e) (((e) == EWOULDBLOCK) || ((e) == EINTR))
