@@ -13,11 +13,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -63,12 +59,14 @@ static char sccsid[] = "@(#)glob.c	5.12 (Berkeley) 6/24/91";
 #include <ctype.h>
 typedef void * ptr_t;
 #endif
-#ifdef WINNT
+#ifdef WINNT_NATIVE
 	#pragma warning(disable:4244)
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
 
 #define Char __Char
 #include "sh.h"
+#include "glob.h"
+
 #undef Char
 #undef QUOTE
 #undef TILDE
@@ -77,8 +75,6 @@ typedef void * ptr_t;
 #undef ismeta
 #undef Strchr
 
-#include "glob.h"
-
 #ifndef S_ISDIR
 #define S_ISDIR(a)	(((a) & S_IFMT) == S_IFDIR)
 #endif
@@ -150,10 +146,32 @@ static	void	 qprintf	__P((Char *));
 #endif
 
 int
-globcharcoll(c1, c2)
-    int c1, c2;
+globcharcoll(c1, c2, cs)
+    __Char c1, c2;
+    int cs;
 {
 #if defined(NLS) && defined(LC_COLLATE) && !defined(NOSTRCOLL)
+# ifdef WIDE_STRINGS
+    wchar_t s1[2], s2[2];
+
+    if (c1 == c2)
+	return (0);
+    if (cs) {
+	c1 = towlower(c1);
+	c2 = towlower(c2);
+    } else {
+	/* This should not be here, but I'll rather leave it in than engage in
+	   a LC_COLLATE flamewar about a shell I don't use... */
+	if (iswlower(c1) && iswupper(c2))
+	    return (1);
+	if (iswupper(c1) && iswlower(c2))
+	    return (-1);
+    }
+    s1[0] = c1;
+    s2[0] = c2;
+    s1[1] = s2[1] = '\0';
+    return wcscoll(s1, s2);
+# else /* not WIDE_STRINGS */
     char s1[2], s2[2];
 
     if (c1 == c2)
@@ -162,12 +180,20 @@ globcharcoll(c1, c2)
      * From kevin lyda <kevin@suberic.net>:
      * strcoll does not guarantee case sorting, so we pre-process now:
      */
-    if (islower(c1) && isupper(c2))
-	return (1);
+    if (cs) {
+	c1 = islower(c1) ? c1 : tolower(c1);
+	c2 = islower(c2) ? c2 : tolower(c2);
+    } else {
+	if (islower(c1) && isupper(c2))
+	    return (1);
+	if (isupper(c1) && islower(c2))
+	    return (-1);
+    }
     s1[0] = c1;
     s2[0] = c2;
     s1[1] = s2[1] = '\0';
     return strcoll(s1, s2);
+# endif
 #else
     return (c1 - c2);
 #endif
@@ -182,10 +208,10 @@ globcharcoll(c1, c2)
 
 static DIR *
 Opendir(str)
-    register Char *str;
+    Char *str;
 {
     char    buf[GLOBBUFLEN];
-    register char *dc = buf;
+    char *dc = buf;
 #if defined(hpux) || defined(__hpux)
     struct stat st;
 #endif
@@ -207,11 +233,11 @@ Opendir(str)
 #ifdef S_IFLNK
 static int
 Lstat(fn, sb)
-    register Char *fn;
+    Char *fn;
     struct stat *sb;
 {
     char    buf[GLOBBUFLEN];
-    register char *dc = buf;
+    char *dc = buf;
 
     while ((*dc++ = *fn++) != '\0')
 	continue;
@@ -234,11 +260,11 @@ Lstat(fn, sb)
 
 static int
 Stat(fn, sb)
-    register Char *fn;
+    Char *fn;
     struct stat *sb;
 {
     char    buf[GLOBBUFLEN];
-    register char *dc = buf;
+    char *dc = buf;
 
     while ((*dc++ = *fn++) != '\0')
 	continue;
@@ -318,10 +344,15 @@ glob(pattern, flags, errfunc, pglob)
     Char *bufnext, *bufend, *compilebuf, m_not;
     const unsigned char *compilepat, *patnext;
     int     c, not;
-    Char patbuf[GLOBBUFLEN + 1], *qpatnext;
+    Char *qpatnext;
+#ifdef WIDE_STRINGS
+    Char patbuf[GLOBBUFLEN + MB_LEN_MAX + 1];
+#else
+    Char patbuf[GLOBBUFLEN + 1];
+#endif
     int     no_match;
 
-    patnext = (unsigned char *) pattern;
+    patnext = (const unsigned char *) pattern;
     if (!(flags & GLOB_APPEND)) {
 	pglob->gl_pathc = 0;
 	pglob->gl_pathv = NULL;
@@ -353,8 +384,19 @@ glob(pattern, flags, errfunc, pglob)
 
     if (flags & GLOB_QUOTE) {
 	/* Protect the quoted characters */
-	while (bufnext < bufend && (c = *patnext++) != EOS) 
-#ifdef DSPMBYTE
+	while (bufnext < bufend && (c = *patnext++) != EOS) {
+#ifdef WIDE_STRINGS
+	    int len;
+	    
+	    len = mblen(patnext - 1, MB_LEN_MAX);
+	    if (len == -1)
+		mblen(NULL, 0);
+	    if (len > 1) {
+		*bufnext++ = (Char) c;
+		while (--len != 0)
+		    *bufnext++ = (Char) *patnext++;
+	    } else
+#elif defined (DSPMBYTE)
 	    if (Ismbyte1(c) && *patnext != EOS)
 	    {
 	      *bufnext++ = (Char) c;
@@ -371,6 +413,7 @@ glob(pattern, flags, errfunc, pglob)
 	    }
 	    else
 		*bufnext++ = (Char) c;
+	}
     }
     else 
 	while (bufnext < bufend && (c = *patnext++) != EOS) 
@@ -381,7 +424,21 @@ glob(pattern, flags, errfunc, pglob)
     qpatnext = patbuf;
     /* we don't need to check for buffer overflow any more */
     while ((c = *qpatnext++) != EOS) {
-#ifdef DSPMBYTE
+#ifdef WIDE_STRINGS
+	int len;
+
+	/* patbuf contains unmodified characters from pattern except that it 
+	   may contain additional flags, so we can check the original string
+	   here. */
+	len = mblen(pattern + ((qpatnext - 1) - patbuf), MB_LEN_MAX);
+	if (len == -1)
+	    mblen(NULL, 0);
+	if (len > 1) {
+	    *bufnext++ = CHAR(c);
+	    while (--len != 0)
+		*bufnext++ = CHAR(*qpatnext++);
+	} else
+#elif defined (DSPMBYTE)
 	if (Ismbyte1(c) && *qpatnext != EOS)
 	{
 	  *bufnext++ = CHAR(c);
@@ -592,8 +649,8 @@ glob3(pathbuf, pathend, pattern, restpattern, pglob, no_match)
 
     /* search directory for matching names */
     while ((dp = readdir(dirp)) != NULL) {
-	register unsigned char *sc;
-	register Char *dc;
+	unsigned char *sc;
+	Char *dc;
 
 	/* initial DOT must be matched literally */
 	if (dp->d_name[0] == DOT && *pattern != DOT)
@@ -634,8 +691,8 @@ globextend(path, pglob)
     Char *path;
     glob_t *pglob;
 {
-    register char **pathv;
-    register int i;
+    char **pathv;
+    int i;
     unsigned int newsize;
     char   *copy;
     Char *p;
@@ -658,8 +715,8 @@ globextend(path, pglob)
     for (p = path; *p++;)
 	continue;
     if ((copy = (char *) xmalloc((size_t) (p - path))) != NULL) {
-	register char *dc = copy;
-	register Char *sc = path;
+	char *dc = copy;
+	Char *sc = path;
 
 	while ((*dc++ = *sc++) != '\0')
 	    continue;
@@ -670,56 +727,123 @@ globextend(path, pglob)
 }
 
 
+#ifdef WIDE_STRINGS
+static size_t
+One_mbtowc(wchar_t *pwc, const Char *s, size_t n)
+{
+    char buf[MB_LEN_MAX], *p;
+
+    if (n > MB_LEN_MAX)
+	n = MB_LEN_MAX;
+    p = buf;
+    while (p < buf + n && (*p++ = CHAR(*s++)) != 0)
+	;
+    return one_mbtowc(pwc, buf, n);
+}
+#endif
+
 /*
  * pattern matching function for filenames.  Each occurrence of the *
  * pattern causes a recursion level.
  */
 static  int
 match(name, pat, patend, m_not)
-    register Char *name, *pat, *patend;
+    Char *name, *pat, *patend;
     int m_not;
 {
     int ok, negate_range;
     Char c, k;
 
     while (pat < patend) {
+#ifdef WIDE_STRINGS
+	size_t lwk;
+	__Char wc, wk;
+
+	USE(k);
+	c = *pat; /* Only for M_MASK bits */
+	pat += One_mbtowc(&wc, pat, MB_LEN_MAX);
+	lwk = One_mbtowc(&wk, name, MB_LEN_MAX);
+#else /* not WIDE_STRINGS */
 	c = *pat++;
+#endif
 	switch (c & M_MASK) {
 	case M_ALL:
 	    if (pat == patend)
 		return (1);
-	    do 
+	    for (;;) {
 		if (match(name, pat, patend, m_not))
 		    return (1);
-	    while (*name++ != EOS);
+		if (*name == EOS)
+		    break;
+#ifdef WIDE_STRINGS
+		name += lwk;
+		lwk = One_mbtowc(&wk, name, MB_LEN_MAX);
+#else
+		name++;
+#endif
+	    }
 	    return (0);
 	case M_ONE:
-	    if (*name++ == EOS)
+	    if (*name == EOS)
 		return (0);
+#ifdef WIDE_STRINGS
+	    name += lwk;
+#else
+	    name++;
+#endif
 	    break;
 	case M_SET:
 	    ok = 0;
+#ifdef WIDE_STRINGS
+	    if (*name == EOS)
+		return (0);
+	    name += lwk;
+#else
 	    if ((k = *name++) == EOS)
 		return (0);
+#endif
 	    if ((negate_range = ((*pat & M_MASK) == m_not)) != 0)
 		++pat;
+#ifdef WIDE_STRINGS
+	    while ((*pat & M_MASK) != M_END) {
+		pat += One_mbtowc(&wc, pat, MB_LEN_MAX);
+		if ((*pat & M_MASK) == M_RNG) {
+		    __Char wc2;
+		    
+		    pat++;
+		    pat += One_mbtowc(&wc2, pat, MB_LEN_MAX);
+		    if (globcharcoll(wc, wk, 0) <= 0 &&
+			globcharcoll(wk, wc2, 0) <= 0)
+			ok = 1;
+		} else if (wc == wk)
+		    ok = 1;
+	    }
+	    pat += One_mbtowc(&wc, pat, MB_LEN_MAX);
+#else /* WIDE_STRINGS */
 	    while (((c = *pat++) & M_MASK) != M_END) {
 		if ((*pat & M_MASK) == M_RNG) {
-		    if (globcharcoll(CHAR(c), CHAR(k)) <= 0 &&
-			globcharcoll(CHAR(k), CHAR(pat[1])) <= 0)
+		    if (globcharcoll(CHAR(c), CHAR(k), 0) <= 0 &&
+			globcharcoll(CHAR(k), CHAR(pat[1]), 0) <= 0)
 			ok = 1;
 		    pat += 2;
 		}
 		else if (c == k)
 		    ok = 1;
 	    }
+#endif
 	    if (ok == negate_range)
 		return (0);
 	    break;
 	default:
+#ifdef WIDE_STRINGS
+	    name += lwk;
+	    if (samecase(wk) != samecase(wc))
+		return (0);
+#else
 	    k = *name++;
 	    if (samecase(k) != samecase(c))
 		return (0);
+#endif
 	    break;
 	}
     }
@@ -731,8 +855,8 @@ void
 globfree(pglob)
     glob_t *pglob;
 {
-    register int i;
-    register char **pp;
+    int i;
+    char **pp;
 
     if (pglob->gl_pathv != NULL) {
 	pp = pglob->gl_pathv + pglob->gl_offs;
