@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.c,v 3.106 2002/11/21 20:02:00 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.c,v 3.117 2004/11/20 18:25:25 christos Exp $ */
 /*
  * sh.c: Main shell routines
  */
@@ -39,7 +39,7 @@ char    copyright[] =
  All rights reserved.\n";
 #endif /* not lint */
 
-RCSID("$Id: sh.c,v 3.106 2002/11/21 20:02:00 christos Exp $")
+RCSID("$Id: sh.c,v 3.117 2004/11/20 18:25:25 christos Exp $")
 
 #include "tc.h"
 #include "ed.h"
@@ -47,7 +47,6 @@ RCSID("$Id: sh.c,v 3.106 2002/11/21 20:02:00 christos Exp $")
 
 extern bool MapsAreInited;
 extern bool NLSMapsAreInited;
-extern bool NoNLSRebind;
 
 /*
  * C Shell
@@ -114,7 +113,7 @@ bool    fast = 0;
 static bool    batch = 0;
 static bool    mflag = 0;
 static bool    prompt = 1;
-static int     enterhist = 0;
+int     enterhist = 0;
 bool    tellwhat = 0;
 time_t  t_period;
 Char  *ffile = NULL;
@@ -160,9 +159,9 @@ struct saved_state {
 
 static	int		  srccat	__P((Char *, Char *));
 #ifndef WINNT_NATIVE
-static	int		  srcfile	__P((char *, bool, int, Char **));
+static	int		  srcfile	__P((const char *, bool, int, Char **));
 #else
-int		  srcfile	__P((char *, bool, int, Char **));
+int		  srcfile	__P((const char *, bool, int, Char **));
 #endif /*WINNT_NATIVE*/
 static	sigret_t	  phup		__P((int));
 static	void		  srcunit	__P((int, bool, int, Char **));
@@ -182,13 +181,15 @@ main(argc, argv)
     int     argc;
     char  **argv;
 {
-    register Char *cp;
+    Char *cp;
 #ifdef AUTOLOGOUT
-    register Char *cp2;
+    Char *cp2;
 #endif
-    register char *tcp, *ttyn;
-    register int f;
-    register char **tempv;
+    char *tcp, *ttyn;
+    int f;
+    char **tempv;
+    int osetintr;
+    signalfun_t oparintr;
 
 #ifdef BSDSIGS
     sigvec_t osv;
@@ -246,6 +247,10 @@ main(argc, argv)
 	(void) close(f);
     }
 
+#ifdef O_TEXT
+    setmode(0, O_TEXT);
+#endif
+
     osinit();			/* Os dependent initialization */
 
     
@@ -358,7 +363,7 @@ main(argc, argv)
 
 	for (k = 0200; k <= 0377 && !Isprint(k); k++)
 	    continue;
-	AsciiOnly = k > 0377;
+	AsciiOnly = MB_CUR_MAX == 1 && k > 0377;
     }
 #else
     AsciiOnly = getenv("LANG") == NULL && getenv("LC_CTYPE") == NULL;
@@ -669,6 +674,11 @@ main(argc, argv)
      */
     set(STRaddsuffix, Strsave(STRNULL), VAR_READWRITE);
 
+    /*
+     * Compatibility with tcsh >= 6.12 by default
+     */
+    set(STRcsubstnonl, Strsave(STRNULL), VAR_READWRITE);
+    
     /*
      * Random default kill ring size
      */
@@ -850,7 +860,7 @@ main(argc, argv)
 		 * ensure that you don't make * nonportable csh scripts.
 		 */
 		{
-		    register int count;
+		    int count;
 
 		    cp = arginp + Strlen(arginp);
 		    count = 0;
@@ -871,7 +881,7 @@ main(argc, argv)
 #ifdef apollo
 	    case 'D':		/* -D	Define environment variable */
 		{
-		    register Char *dp;
+		    Char *dp;
 
 		    cp = str2short(tcp);
 		    if (dp = Strchr(cp, '=')) {
@@ -976,6 +986,9 @@ main(argc, argv)
 	    /* ... doesn't return */
 	    stderror(ERR_SYSTEM, tempv[0], strerror(errno));
 	}
+#ifdef O_TEXT
+	setmode(nofile, O_TEXT);
+#endif
 	if (ffile != NULL)
 	    xfree((ptr_t) ffile);
 	dolzero = 1;
@@ -1253,15 +1266,14 @@ main(argc, argv)
      * Set an exit here in case of an interrupt or error reading the shell
      * start-up scripts.
      */
+    osetintr = setintr;
+    oparintr = parintr;
     reenter = setexit();	/* PWP */
     exitset++;
     haderr = 0;			/* In case second time through */
     if (!fast && reenter == 0) {
 	/* Will have varval(STRhome) here because set fast if don't */
 	{
-	    int     osetintr = setintr;
-	    signalfun_t oparintr = parintr;
-
 #ifdef BSDSIGS
 	    sigmask_t omask = sigblock(sigmask(SIGINT));
 #else
@@ -1317,6 +1329,10 @@ main(argc, argv)
 	if (!fast && (loginsh || rdirs))
 	    loaddirs(NULL);
     }
+    /* Reset interrupt flag */
+    setintr = osetintr;
+    parintr = oparintr;
+
     /* Initing AFTER .cshrc is the Right Way */
     if (intty && !arginp) {	/* PWP setup stuff */
 	ed_Init();		/* init the new line editor */
@@ -1379,9 +1395,9 @@ void
 importpath(cp)
     Char   *cp;
 {
-    register int i = 0;
-    register Char *dp;
-    register Char **pv;
+    int i = 0;
+    Char *dp;
+    Char **pv;
     int     c;
 
     for (dp = cp; *dp; dp++)
@@ -1426,7 +1442,7 @@ srccat(cp, dp)
     if (cp[0] == '/' && cp[1] == '\0') 
 	return srcfile(short2str(dp), (mflag ? 0 : 1), 0, NULL);
     else {
-	register Char *ep;
+	Char *ep;
 	char   *ptr;
 	int rv;
 
@@ -1456,15 +1472,18 @@ static int
 int
 #endif /*WINNT_NATIVE*/
 srcfile(f, onlyown, flag, av)
-    char   *f;
+    const char   *f;
     bool    onlyown;
     int flag;
     Char **av;
 {
-    register int unit;
+    int unit;
 
     if ((unit = open(f, O_RDONLY|O_LARGEFILE)) == -1) 
 	return 0;
+#ifdef O_TEXT
+    setmode(unit, O_TEXT);
+#endif
     unit = dmove(unit, -1);
 
     (void) close_on_exec(unit, 1);
@@ -1501,12 +1520,21 @@ st_save(st, unit, hflg, al, av)
     st->OLDSTD = st->SHOUT = st->SHDIAG = -1;/* test later to restore these */
     if (didfds) {
 	    struct stat s1, s2;
-	    if (NEED_SAVE_FD(0,OLDSTD))
-		    st->OLDSTD = OLDSTD, OLDSTD = dmove(0, -1);
-	    if (NEED_SAVE_FD(1,SHOUT))
-		    st->SHOUT = SHOUT, SHOUT = dmove(1, -1);
-	    if (NEED_SAVE_FD(2,SHDIAG))
-		    st->SHDIAG = SHDIAG, SHDIAG = dmove(2, -1);
+	    if (NEED_SAVE_FD(0,OLDSTD)) {
+		    st->OLDSTD = OLDSTD;
+		    OLDSTD = dmove(0, -1);
+		    (void)close_on_exec(OLDSTD, 1);
+	    }
+	    if (NEED_SAVE_FD(1,SHOUT)) {
+		    st->SHOUT = SHOUT;
+		    SHOUT = dmove(1, -1);
+		    (void)close_on_exec(SHOUT, 1);
+	    }
+	    if (NEED_SAVE_FD(2,SHDIAG)) {
+		    st->SHDIAG = SHDIAG;
+		    SHDIAG = dmove(2, -1);
+		    (void)close_on_exec(SHDIAG, 1);
+	    }
 	    donefds();
     }
 
@@ -1581,9 +1609,9 @@ st_restore(st, av)
 
     /* Reset input arena */
     {
-	register int i;
-	register Char** nfbuf = fbuf;
-	register int nfblocks = fblocks;
+	int i;
+	Char** nfbuf = fbuf;
+	int nfblocks = fblocks;
 
 	fblocks = 0;
 	fbuf = NULL;
@@ -1629,7 +1657,7 @@ st_restore(st, av)
  */
 static void
 srcunit(unit, onlyown, hflg, av)
-    register int unit;
+    int unit;
     bool    onlyown;
     int hflg;
     Char **av;
@@ -1716,6 +1744,7 @@ goodbye(v, c)
     Char **v;
     struct command *c;
 {
+    USE(v);
     USE(c);
     record();
 
@@ -1854,6 +1883,7 @@ sigret_t
 pintr(snum)
 int snum;
 {
+    USE(snum);
 #ifdef UNRELSIGS
     if (snum)
 	(void) sigset(snum, pintr);
@@ -1869,7 +1899,7 @@ void
 pintr1(wantnl)
     bool    wantnl;
 {
-    register Char **v;
+    Char **v;
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif
@@ -1892,15 +1922,11 @@ pintr1(wantnl)
     }
     /* MH - handle interrupted completions specially */
     {
-	extern int InsideCompletion;
-
 	if (InsideCompletion)
 	    stderror(ERR_SILENT);
     }
     /* JV - Make sure we shut off inputl */
     {
-	extern Char GettingInput;
-
 	(void) Cookedmode();
 	GettingInput = 0;
     }
@@ -1969,7 +1995,6 @@ void
 process(catch)
     bool    catch;
 {
-    extern char Expand;
     jmp_buf_t osetexit;
     /* PWP: This might get nuked my longjmp so don't make it a register var */
     struct command *t = savet;
@@ -2166,7 +2191,6 @@ process(catch)
 	freesyn(savet), savet = NULL;
 #ifdef SIG_WINDOW
 	if (windowchg || (catch && intty && !whyles && !tellwhat)) {
-	    windowchg = 0;
 	    (void) check_window_size(0);	/* for window systems */
 	}
 #endif /* SIG_WINDOW */
@@ -2179,12 +2203,11 @@ process(catch)
 /*ARGSUSED*/
 void
 dosource(t, c)
-    register Char **t;
+    Char **t;
     struct command *c;
 {
-    register Char *f;
+    Char *f;
     bool    hflg = 0;
-    extern int bequiet;
     char    buf[BUFSIZE];
 
     USE(c);
@@ -2203,6 +2226,15 @@ dosource(t, c)
     f = globone(*t++, G_ERROR);
     (void) strcpy(buf, short2str(f));
     xfree((ptr_t) f);
+    gflag = 0, tglob(t);
+    if (gflag) {
+	t = globall(t);
+	if (t == 0)
+	    stderror(ERR_NAME | ERR_NOMATCH);
+    } else {
+	t = saveblk(t);
+	trim(t);
+    }
     if ((!srcfile(buf, 0, hflg, t)) && (!hflg) && (!bequiet))
 	stderror(ERR_SYSTEM, buf, strerror(errno));
 }
@@ -2229,8 +2261,8 @@ dosource(t, c)
 static void
 mailchk()
 {
-    register struct varent *v;
-    register Char **vp;
+    struct varent *v;
+    Char **vp;
     time_t  t;
     int     intvl, cnt;
     struct stat stb;
@@ -2363,7 +2395,7 @@ initdesc()
 {
 #ifdef NLS_BUGS
 #ifdef NLS_CATALOGS
-    (void)catclose(catd);
+    nlsclose();
 #endif /* NLS_CATALOGS */
 #endif /* NLS_BUGS */
 
@@ -2438,7 +2470,7 @@ xexit(i)
      * because messages will stop working on the parent too.
      */
     if (child == 0)
-	(void) catclose(catd);
+	nlsclose();
 #endif /* NLS_CATALOGS */
 #ifdef WINNT_NATIVE
     nt_cleanup();
