@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.decls.h,v 3.38 2002/05/16 13:51:25 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.decls.h,v 3.42 2004/08/08 06:42:28 christos Exp $ */
 /*
  * sh.decls.h	 External declarations from sh*.c
  */
@@ -115,7 +115,7 @@ extern	int		  tenex		__P((Char *, int));
 /*
  * sh.func.c
  */
-extern	void		  tsetenv	__P((Char *, Char *));
+extern	void		  tsetenv	__P((const Char *, const Char *));
 extern	void		  Unsetenv	__P((Char *));
 extern	void		  doalias	__P((Char **, struct command *));
 extern	void		  dobreak	__P((Char **, struct command *));
@@ -172,13 +172,19 @@ extern	int		  Gmatch	__P((Char *, Char *));
 extern	int		  Gnmatch	__P((Char *, Char *, Char **));
 extern	void		  ginit		__P((void));
 extern	Char		**globall	__P((Char **));
-extern	void		  rscan		__P((Char **, void (*)(int)));
+extern	void		  rscan		__P((Char **, void (*)(Char)));
 extern	void		  tglob		__P((Char **));
 extern	void		  trim		__P((Char **));
 #if defined(FILEC) && defined(TIOCSTI)
 extern	int		  sortscmp	__P((Char **, Char **));
 #endif
+
+#ifdef HAVE_ICONV
+extern	char		 *iconv_catgets	__P((nl_catd, int, int, const char *));
+#endif
 extern	void		  nlsinit	__P((void));
+extern	void	          nlsclose	__P((void));
+extern  int	  	  t_pmatch	__P((Char *, Char *, Char **, int));
 
 /*
  * sh.hist.c
@@ -204,19 +210,19 @@ extern	void		  btell		__P((struct Ain *));
 extern	void		  btoeof	__P((void));
 extern	void		  copylex	__P((struct wordent *, 
 					     struct wordent *));
-extern	Char		 *domod		__P((Char *, int));
+extern	Char		 *domod		__P((Char *, Char));
 extern	void		  freelex	__P((struct wordent *));
 extern	int		  lex		__P((struct wordent *));
 extern	void		  prlex		__P((struct wordent *));
-extern	int		  readc		__P((bool));
+extern	eChar		  readc		__P((bool));
 extern	void		  settell	__P((void));
-extern	void		  unreadc	__P((int));
+extern	void		  unreadc	__P((Char));
 
 
 /*
  * sh.misc.c
  */
-extern	int		  any		__P((char *, int));
+extern	int		  any		__P((const char *, Char));
 extern	Char		**blkcpy	__P((Char **, Char **));
 extern	void		  blkfree	__P((Char **));
 extern	int		  blklen	__P((Char **));
@@ -234,7 +240,7 @@ extern	void		  donefds	__P((void));
 extern	Char		  lastchr	__P((Char *));
 extern	void		  lshift	__P((Char **, int));
 extern	int		  number	__P((Char *));
-extern	int		  prefix	__P((Char *, Char *));
+extern	int		  prefix	__P((const Char *, const Char *));
 extern	Char		**saveblk	__P((Char **));
 extern	void		  setzero	__P((char *, int));
 extern	Char		 *strip		__P((Char *));
@@ -246,7 +252,7 @@ extern	void		  udvar		__P((Char *));
 extern  char   	  	 *strstr	__P((const char *, const char *));
 #endif /* !POSIX */
 #ifndef SHORT_STRINGS
-extern	char		 *strspl	__P((char *, char *));
+extern	char		 *strspl	__P((const char *, const char *));
 extern	char		 *strend	__P((char *));
 #endif /* SHORT_STRINGS */
 
@@ -278,6 +284,13 @@ extern	void		  psecs		__P((long));
 extern	int		  putpure	__P((int));
 extern	int		  putraw	__P((int));
 extern	void		  xputchar	__P((int));
+#ifdef WIDE_STRINGS
+extern	void		  putwraw	__P((Char));
+extern	void		  xputwchar	__P((Char));
+#else
+# define putwraw(C) putraw(C)
+# define xputwchar(C) xputchar(C)
+#endif
 
 
 /*
