@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.dol.c,v 3.51 2002/11/21 20:02:00 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.dol.c,v 3.53 2004/08/04 17:12:28 christos Exp $ */
 /*
  * sh.dol.c: Variable substitutions
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.dol.c,v 3.51 2002/11/21 20:02:00 christos Exp $")
+RCSID("$Id: sh.dol.c,v 3.53 2004/08/04 17:12:28 christos Exp $")
 
 /*
  * C shell
@@ -47,10 +47,11 @@ RCSID("$Id: sh.dol.c,v 3.51 2002/11/21 20:02:00 christos Exp $")
  * some QUOTEing may have occurred already, so we dont "trim()" here.
  */
 
-static int Dpeekc, Dpeekrd;	/* Peeks for DgetC and Dreadc */
+static Char Dpeekc;		/* Peek for DgetC */
+static eChar Dpeekrd;		/* Peek for Dreadc */
 static Char *Dcp, **Dvp;	/* Input vector for Dreadc */
 
-#define	DEOF	-1
+#define	DEOF	CHAR_ERR
 
 #define	unDgetC(c)	Dpeekc = c
 
@@ -82,13 +83,13 @@ static	void	 Dfix2		__P((Char **));
 static	Char 	*Dpack		__P((Char *, Char *));
 static	int	 Dword		__P((void));
 static	void	 dolerror	__P((Char *));
-static	int	 DgetC		__P((int));
+static	eChar	 DgetC		__P((int));
 static	void	 Dgetdol	__P((void));
 static	void	 fixDolMod	__P((void));
 static	void	 setDolp	__P((Char *));
-static	void	 unDredc	__P((int));
-static	int	 Dredc		__P((void));
-static	void	 Dtestq		__P((int));
+static	void	 unDredc	__P((eChar));
+static	eChar	 Dredc		__P((void));
+static	void	 Dtestq		__P((Char));
 
 /*
  * Fix up the $ expansions and quotations in the
@@ -96,10 +97,10 @@ static	void	 Dtestq		__P((int));
  */
 void
 Dfix(t)
-    register struct command *t;
+    struct command *t;
 {
-    register Char **pp;
-    register Char *p;
+    Char **pp;
+    Char *p;
 
     if (noexec)
 	return;
@@ -127,7 +128,7 @@ Dfix(t)
  */
 Char   *
 Dfix1(cp)
-    register Char *cp;
+    Char *cp;
 {
     Char   *Dv[2];
 
@@ -170,8 +171,8 @@ static Char *
 Dpack(wbuf, wp)
     Char   *wbuf, *wp;
 {
-    register int c;
-    register int i = MAXWLEN - (int) (wp - wbuf);
+    eChar c;
+    int i = MAXWLEN - (int) (wp - wbuf);
 #if defined(DSPMBYTE)
     int mbytepos = 1;
 #endif /* DSPMBYTE */
@@ -239,11 +240,11 @@ mbyteskip:
 static int
 Dword()
 {
-    register int c, c1;
+    eChar c, c1;
     Char    wbuf[BUFSIZE];
-    register Char *wp = wbuf;
-    register int i = MAXWLEN;
-    register bool dolflg;
+    Char *wp = wbuf;
+    int i = MAXWLEN;
+    bool dolflg;
     bool    sofar = 0, done = 0;
 
     while (!done) {
@@ -285,7 +286,7 @@ Dword()
 		if (c == c1)
 		    break;
 		if (c == '\n' || c == DEOF)
-		    stderror(ERR_UNMATCHED, c1);
+		    stderror(ERR_UNMATCHED, (int)c1);
 		if ((c & (QUOTE | TRIM)) == ('\n' | QUOTE)) {
 		    if ((wp[-1] & TRIM) == '\\')
 			--wp;
@@ -383,11 +384,11 @@ Dword()
  * Any QUOTES character which is returned from a $ expansion is
  * QUOTEd so that it will not be recognized above.
  */
-static int
+static eChar
 DgetC(flag)
-    register int flag;
+    int flag;
 {
-    register int c;
+    Char c;
 
 top:
     if ((c = Dpeekc) != 0) {
@@ -447,13 +448,12 @@ dolerror(s)
 static void
 Dgetdol()
 {
-    register Char *np;
-    register struct varent *vp = NULL;
+    Char *np;
+    struct varent *vp = NULL;
     Char    name[4 * MAXVARLEN + 1];
-    int     c, sc;
+    eChar   c, sc;
     int     subscr = 0, lwb = 1, upb = 0;
     bool    dimen = 0, bitset = 0, length = 0;
-    char    tnp;
     Char    wbuf[BUFSIZE];
     static Char *dolbang = NULL;
 
@@ -492,7 +492,9 @@ Dgetdol()
 #ifdef COHERENT
     /* Coherent compiler doesn't allow case-labels that are not 
        constant-expressions */
-#ifdef SHORT_STRINGS
+#ifdef WIDE_STRINGS
+    case 0x8000003C:		/* Does Coherent have 32-bit int at all? */
+#elif defined (SHORT_STRINGS)
     case 0100074:
 #else /* !SHORT_STRINGS */
     case 0274:
@@ -507,11 +509,52 @@ Dgetdol()
 	if (length)
 	    stderror(ERR_NOTALLOWED, "$%<");
 	{
+#ifdef WIDE_STRINGS
+	    char cbuf[MB_LEN_MAX];
+	    size_t cbp = 0;
+#else
+	    char tnp;
+#endif
+
 #ifdef BSDSIGS
 	    sigmask_t omask = sigsetmask(sigblock(0) & ~sigmask(SIGINT));
 #else /* !BSDSIGS */
 	    (void) sigrelse(SIGINT);
 #endif /* BSDSIGS */
+#ifdef WIDE_STRINGS
+	    np = wbuf;
+	    while (force_read(OLDSTD, cbuf + cbp++, 1) == 1) {
+	        int len;
+
+		len = mbtowc(np, cbuf, cbp);
+		if (len == -1) {
+		    mbtowc(NULL, NULL, 0);
+		    if (cbp < MB_LEN_MAX)
+		        continue; /* Maybe a partial character */
+		    *np = (unsigned char)*cbuf;
+		}
+		if (len <= 0)
+		    len = 1;
+		if (cbp != (size_t)len)
+		    memmove(cbuf, cbuf + len, cbp - len);
+		cbp -= len;
+		if (np >= &wbuf[BUFSIZE - 1])
+		    stderror(ERR_LTOOLONG);
+		if (*np == '\n')
+		    break;
+		np++;
+	    }
+	    while (cbp != 0) {
+	        *np = (unsigned char)*cbuf;
+		if (np >= &wbuf[BUFSIZE - 1])
+		    stderror(ERR_LTOOLONG);
+		if (*np == '\n')
+		    break;
+		np++;
+		cbp--;
+		memmove(cbuf, cbuf + 1, cbp);
+	    }
+#else
 	    for (np = wbuf; force_read(OLDSTD, &tnp, 1) == 1; np++) {
 		*np = (unsigned char) tnp;
 		if (np >= &wbuf[BUFSIZE - 1])
@@ -519,6 +562,7 @@ Dgetdol()
 		if (tnp == '\n')
 		    break;
 	    }
+#endif
 	    *np = 0;
 #ifdef BSDSIGS
 	    (void) sigsetmask(omask);
@@ -692,7 +736,7 @@ Dgetdol()
 	else if (*np != '-')
 	    stderror(ERR_MISSING, '-');
 	else {
-	    register int i = upb;
+	    int i = upb;
 
 	    np++;
 	    if (Isdigit(*np)) {
@@ -776,7 +820,7 @@ eatbrac:
 static void
 fixDolMod()
 {
-    register int c;
+    eChar c;
 
     c = DgetC(0);
     if (c == ':') {
@@ -802,7 +846,7 @@ fixDolMod()
 
 	    if (c == 's') {	/* [eichin:19910926.0755EST] */
 		int delimcnt = 2;
-		int delim = DgetC(0);
+		eChar delim = DgetC(0);
 		dolmod[dolnmod++] = (Char) c;
 		dolmod[dolnmod++] = (Char) delim;
 		
@@ -811,7 +855,7 @@ fixDolMod()
 		    seterror(ERR_BADSUBST);
 		    break;
 		}	
-		while ((c = DgetC(0)) != (-1)) {
+		while ((c = DgetC(0)) != DEOF) {
 		    dolmod[dolnmod++] = (Char) c;
 		    if(c == delim) delimcnt--;
 		    if(!delimcnt) break;
@@ -823,7 +867,7 @@ fixDolMod()
 		continue;
 	    }
 	    if (!any("luhtrqxes", c))
-		stderror(ERR_BADMOD, c);
+		stderror(ERR_BADMOD, (int)c);
 #ifndef COMPAT
 	    dolmod[dolnmod++] = (Char) c;
 #else
@@ -843,9 +887,9 @@ fixDolMod()
 
 static void
 setDolp(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register Char *dp;
+    Char *dp;
 #ifndef COMPAT
     int i;
 #endif /* COMPAT */
@@ -865,7 +909,7 @@ setDolp(cp)
     for (i = 0; i < dolnmod; i++) {
 	/* handle s// [eichin:19910926.0510EST] */
 	if(dolmod[i] == 's') {
-	    int delim;
+	    Char delim;
 	    Char *lhsub, *rhsub, *np;
 	    size_t lhlen = 0, rhlen = 0;
 	    int didmod = 0;
@@ -968,16 +1012,16 @@ setDolp(cp)
 
 static void
 unDredc(c)
-    int     c;
+    eChar   c;
 {
 
     Dpeekrd = c;
 }
 
-static int
+static eChar
 Dredc()
 {
-    register int c;
+    Char c;
 
     if ((c = Dpeekrd) != 0) {
 	Dpeekrd = 0;
@@ -995,7 +1039,7 @@ Dredc()
 
 static void
 Dtestq(c)
-    register int c;
+    Char c;
 {
 
     if (cmap(c, QUOTES))
@@ -1011,11 +1055,11 @@ void
 heredoc(term)
     Char   *term;
 {
-    int c;
+    eChar  c;
     Char   *Dv[2];
-    Char    obuf[BUFSIZE], lbuf[BUFSIZE], mbuf[BUFSIZE];
+    Char    obuf[BUFSIZE + 1], lbuf[BUFSIZE], mbuf[BUFSIZE];
     int     ocnt, lcnt, mcnt;
-    register Char *lbp, *obp, *mbp;
+    Char *lbp, *obp, *mbp;
     Char  **vp;
     bool    quoted;
     char   *tmp;
@@ -1062,10 +1106,17 @@ again:
     quoted = gflag;
     ocnt = BUFSIZE;
     obp = obuf;
+    obuf[BUFSIZE] = 0;
     inheredoc = 1;
 #ifdef WINNT_NATIVE
     __dup_stdin = 1;
 #endif /* WINNT_NATIVE */
+#ifdef O_TEXT
+    setmode(1, O_TEXT);
+#endif
+#ifdef O_BINARY
+    setmode(0, O_BINARY);
+#endif
     for (;;) {
 	/*
 	 * Read up a line
@@ -1074,7 +1125,7 @@ again:
 	lcnt = BUFSIZE - 4;
 	for (;;) {
 	    c = readc(1);	/* 1 -> Want EOF returns */
-	    if (c < 0 || c == '\n')
+	    if (c == CHAR_ERR || c == '\n')
 		break;
 	    if ((c &= TRIM) != 0) {
 		*lbp++ = (Char) c;
@@ -1089,8 +1140,10 @@ again:
 	/*
 	 * Check for EOF or compare to terminator -- before expansion
 	 */
-	if (c < 0 || eq(lbuf, term)) {
-	    (void) write(0, short2str(obuf), (size_t) (BUFSIZE - ocnt));
+	if (c == CHAR_ERR || eq(lbuf, term)) {
+	    *obp = 0;
+	    tmp = short2str(obuf);
+	    (void) write(0, tmp, strlen (tmp));
 	    (void) lseek(0, (off_t) 0, L_SET);
 	    inheredoc = 0;
 	    return;
@@ -1105,7 +1158,8 @@ again:
 	    for (lbp = lbuf; (c = *lbp++) != 0;) {
 		*obp++ = (Char) c;
 		if (--ocnt == 0) {
-		    (void) write(0, short2str(obuf), BUFSIZE);
+		    tmp = short2str(obuf);
+		    (void) write(0, tmp, strlen (tmp));
 		    obp = obuf;
 		    ocnt = BUFSIZE;
 		}
@@ -1169,14 +1223,16 @@ again:
 	    for (mbp = *vp; *mbp; mbp++) {
 		*obp++ = *mbp & TRIM;
 		if (--ocnt == 0) {
-		    (void) write(0, short2str(obuf), BUFSIZE);
+		    tmp = short2str(obuf);
+		    (void) write(0, tmp, strlen (tmp));
 		    obp = obuf;
 		    ocnt = BUFSIZE;
 		}
 	    }
 	    *obp++ = '\n';
 	    if (--ocnt == 0) {
-		(void) write(0, short2str(obuf), BUFSIZE);
+	        tmp = short2str(obuf);
+		(void) write(0, tmp, strlen (tmp));
 		obp = obuf;
 		ocnt = BUFSIZE;
 	    }
