@@ -1,4 +1,4 @@
-/* $Header: /u/christos/cvsroot/tcsh/sh.exp.c,v 3.37 1998/10/25 15:10:09 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.exp.c,v 3.42 2004/08/04 17:12:29 christos Exp $ */
 /*
  * sh.exp.c: Expression evaluations
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,9 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.exp.c,v 3.37 1998/10/25 15:10:09 christos Exp $")
+RCSID("$Id: sh.exp.c,v 3.42 2004/08/04 17:12:29 christos Exp $")
+
+#include "tw.h"
 
 /*
  * C shell
@@ -61,7 +59,7 @@ RCSID("$Id: sh.exp.c,v 3.37 1998/10/25 15:10:09 christos Exp $")
 
 static	int	 sh_access	__P((Char *, int));
 static	int	 exp1		__P((Char ***, bool));
-static	int	 exp2		__P((Char ***, bool));
+static	int	 exp2x		__P((Char ***, bool));
 static	int	 exp2a		__P((Char ***, bool));
 static	int	 exp2b		__P((Char ***, bool));
 static	int	 exp2c		__P((Char ***, bool));
@@ -202,23 +200,23 @@ sh_access(fname, mode)
 
 int
 expr(vp)
-    register Char ***vp;
+    Char ***vp;
 {
     return (exp0(vp, 0));
 }
 
 int
 exp0(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register int p1 = exp1(vp, ignore);
+    int p1 = exp1(vp, ignore);
 
 #ifdef EDEBUG
     etraci("exp0 p1", p1, vp);
 #endif /* EDEBUG */
     if (**vp && eq(**vp, STRor2)) {
-	register int p2;
+	int p2;
 
 	(*vp)++;
 	p2 = exp0(vp, (ignore & TEXP_IGNORE) || p1);
@@ -232,16 +230,16 @@ exp0(vp, ignore)
 
 static int
 exp1(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register int p1 = exp2(vp, ignore);
+    int p1 = exp2x(vp, ignore);
 
 #ifdef EDEBUG
     etraci("exp1 p1", p1, vp);
 #endif /* EDEBUG */
     if (**vp && eq(**vp, STRand2)) {
-	register int p2;
+	int p2;
 
 	(*vp)++;
 	p2 = exp1(vp, (ignore & TEXP_IGNORE) || !p1);
@@ -254,20 +252,20 @@ exp1(vp, ignore)
 }
 
 static int
-exp2(vp, ignore)
-    register Char ***vp;
+exp2x(vp, ignore)
+    Char ***vp;
     bool    ignore;
 {
-    register int p1 = exp2a(vp, ignore);
+    int p1 = exp2a(vp, ignore);
 
 #ifdef EDEBUG
     etraci("exp3 p1", p1, vp);
 #endif /* EDEBUG */
     if (**vp && eq(**vp, STRor)) {
-	register int p2;
+	int p2;
 
 	(*vp)++;
-	p2 = exp2(vp, ignore);
+	p2 = exp2x(vp, ignore);
 #ifdef EDEBUG
 	etraci("exp3 p2", p2, vp);
 #endif /* EDEBUG */
@@ -278,16 +276,16 @@ exp2(vp, ignore)
 
 static int
 exp2a(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register int p1 = exp2b(vp, ignore);
+    int p1 = exp2b(vp, ignore);
 
 #ifdef EDEBUG
     etraci("exp2a p1", p1, vp);
 #endif /* EDEBUG */
     if (**vp && eq(**vp, STRcaret)) {
-	register int p2;
+	int p2;
 
 	(*vp)++;
 	p2 = exp2a(vp, ignore);
@@ -301,16 +299,16 @@ exp2a(vp, ignore)
 
 static int
 exp2b(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register int p1 = exp2c(vp, ignore);
+    int p1 = exp2c(vp, ignore);
 
 #ifdef EDEBUG
     etraci("exp2b p1", p1, vp);
 #endif /* EDEBUG */
     if (**vp && eq(**vp, STRand)) {
-	register int p2;
+	int p2;
 
 	(*vp)++;
 	p2 = exp2b(vp, ignore);
@@ -324,12 +322,12 @@ exp2b(vp, ignore)
 
 static int
 exp2c(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register Char *p1 = exp3(vp, ignore);
-    register Char *p2;
-    register int i;
+    Char *p1 = exp3(vp, ignore);
+    Char *p2;
+    int i;
 
 #ifdef EDEBUG
     etracc("exp2c p1", p1, vp);
@@ -372,11 +370,11 @@ exp2c(vp, ignore)
 
 static Char *
 exp3(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register Char *p1, *p2;
-    register int i;
+    Char *p1, *p2;
+    int i;
 
     p1 = exp3a(vp, ignore);
 #ifdef EDEBUG
@@ -418,11 +416,11 @@ exp3(vp, ignore)
 
 static Char *
 exp3a(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register Char *p1, *p2, *op;
-    register int i;
+    Char *p1, *p2, *op;
+    int i;
 
     p1 = exp4(vp, ignore);
 #ifdef EDEBUG
@@ -448,18 +446,18 @@ exp3a(vp, ignore)
 
 static Char *
 exp4(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register Char *p1, *p2;
-    register int i = 0;
+    Char *p1, *p2;
+    int i = 0;
 
     p1 = exp5(vp, ignore);
 #ifdef EDEBUG
     etracc("exp4 p1", p1, vp);
 #endif /* EDEBUG */
     if (isa(**vp, ADDOP)) {
-	register Char *op = *(*vp)++;
+	Char *op = *(*vp)++;
 
 	p2 = exp4(vp, ignore);
 #ifdef EDEBUG
@@ -485,11 +483,11 @@ exp4(vp, ignore)
 
 static Char *
 exp5(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
-    register Char *p1, *p2;
-    register int i = 0;
+    Char *p1, *p2;
+    int i = 0;
 
     p1 = exp6(vp, ignore);
 #ifdef EDEBUG
@@ -497,7 +495,7 @@ exp5(vp, ignore)
 #endif /* EDEBUG */
 
     if (isa(**vp, MULOP)) {
-	register Char *op = *(*vp)++;
+	Char *op = *(*vp)++;
 	if ((ignore & TEXP_NOGLOB) != 0) 
 	    /* 
 	     * We are just trying to get the right side of
@@ -539,11 +537,11 @@ exp5(vp, ignore)
 
 static Char *
 exp6(vp, ignore)
-    register Char ***vp;
+    Char ***vp;
     bool    ignore;
 {
     int     ccode, i = 0;
-    register Char *cp;
+    Char *cp;
 
     if (**vp == 0)
 	stderror(ERR_NAME | ERR_EXPRESSION);
@@ -579,7 +577,7 @@ exp6(vp, ignore)
 	return (putn(ccode));
     }
     if (eq(**vp, STRLbrace)) {
-	register Char **v;
+	Char **v;
 	struct command faket;
 	Char   *fakecom[2];
 
@@ -742,18 +740,30 @@ filetest(cp, vp, ignore)
 
 #ifdef S_IFLNK
 	    if (tolower(*ft) == 'l') {
-		if (!lst && TCSH_LSTAT(short2str(ep), lst = &lstb) == -1) {
-		    xfree((ptr_t) ep);
-		    return (Strsave(errval));
+		/* 
+		 * avoid convex compiler bug.
+		 */
+		if (!lst) {
+		    lst = &lstb;
+		    if (TCSH_LSTAT(short2str(ep), lst) == -1) {
+			xfree((ptr_t) ep);
+			return (Strsave(errval));
+		    }
 		}
 		if (*ft == 'L')
 		    st = lst;
 	    }
 	    else 
 #endif /* S_IFLNK */
-		if (!st && TCSH_STAT(short2str(ep), st = &stb) == -1) {
-		    xfree((ptr_t) ep);
-		    return (Strsave(errval));
+		/* 
+		 * avoid convex compiler bug.
+		 */
+		if (!st) {
+		    st = &stb;
+		    if (TCSH_STAT(short2str(ep), st) == -1) {
+			xfree((ptr_t) ep);
+			return (Strsave(errval));
+		    }
 		}
 
 	    switch (*ft) {
@@ -980,18 +990,18 @@ filetest(cp, vp, ignore)
 
 static void
 evalav(v)
-    register Char **v;
+    Char **v;
 {
     struct wordent paraml1;
-    register struct wordent *hp = &paraml1;
+    struct wordent *hp = &paraml1;
     struct command *t;
-    register struct wordent *wdp = hp;
+    struct wordent *wdp = hp;
 
     set(STRstatus, Strsave(STR0), VAR_READWRITE);
     hp->prev = hp->next = hp;
     hp->word = STRNULL;
     while (*v) {
-	register struct wordent *new =
+	struct wordent *new =
 	(struct wordent *) xcalloc(1, sizeof *wdp);
 
 	new->prev = wdp;
@@ -1005,14 +1015,14 @@ evalav(v)
     t = syntax(paraml1.next, &paraml1, 0);
     if (seterr)
 	stderror(ERR_OLD);
-    execute(t, -1, NULL, NULL);
+    execute(t, -1, NULL, NULL, TRUE);
     freelex(&paraml1), freesyn(t);
 }
 
 static int
 isa(cp, what)
-    register Char *cp;
-    register int what;
+    Char *cp;
+    int what;
 {
     if (cp == 0)
 	return ((what & RESTOP) != 0);
@@ -1062,7 +1072,7 @@ isa(cp, what)
 
 static int
 egetn(cp)
-    register Char *cp;
+    Char *cp;
 {
     if (*cp && *cp != '-' && !Isdigit(*cp))
 	stderror(ERR_NAME | ERR_EXPRESSION);
