@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.func.c,v 3.93 2001/03/13 12:53:50 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.func.c,v 3.118 2004/11/20 18:25:26 christos Exp $ */
 /*
  * sh.func.c: csh builtin functions
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.func.c,v 3.93 2001/03/13 12:53:50 christos Exp $")
+RCSID("$Id: sh.func.c,v 3.118 2004/11/20 18:25:26 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -45,6 +41,11 @@ RCSID("$Id: sh.func.c,v 3.93 2001/03/13 12:53:50 christos Exp $")
 #include "nt.const.h"
 #endif /* WINNT_NATIVE */
 
+#ifdef HAVE_ICONV
+#include <langinfo.h>
+static iconv_t catgets_iconv; /* Or (iconv_t)-1 */
+#endif
+
 /*
  * C shell
  */
@@ -53,7 +54,6 @@ extern char **environ;
 
 extern bool MapsAreInited;
 extern bool NLSMapsAreInited;
-extern bool NoNLSRebind;
 extern bool GotTermCaps;
 
 static int zlast = -1;
@@ -61,19 +61,20 @@ static int zlast = -1;
 static	void	islogin		__P((void));
 static	void	preread		__P((void));
 static	void	doagain		__P((void));
-static  char   *isrchx		__P((int));
+static  const char *isrchx	__P((int));
 static	void	search		__P((int, int, Char *));
 static	int	getword		__P((Char *));
 static	void	toend		__P((void));
 static	void	xecho		__P((int, Char **));
 static	bool	islocale_var	__P((Char *));
+static	void	wpfree		__P((struct whyle *));
 
 struct biltins *
 isbfunc(t)
     struct command *t;
 {
-    register Char *cp = t->t_dcom[0];
-    register struct biltins *bp, *bp1, *bp2;
+    Char *cp = t->t_dcom[0];
+    struct biltins *bp, *bp1, *bp2;
     static struct biltins label = {"", dozip, 0, 0};
     static struct biltins foregnd = {"%job", dofg1, 0, 0};
     static struct biltins backgnd = {"%job &", dobg1, 0, 0};
@@ -134,8 +135,8 @@ isbfunc(t)
 
 void
 func(t, bp)
-    register struct command *t;
-    register struct biltins *bp;
+    struct command *t;
+    struct biltins *bp;
 {
     int     i;
 
@@ -155,8 +156,8 @@ doonintr(v, c)
     Char  **v;
     struct command *c;
 {
-    register Char *cp;
-    register Char *vv = v[1];
+    Char *cp;
+    Char *vv = v[1];
 
     USE(c);
     if (parintr == SIG_IGN)
@@ -253,6 +254,7 @@ dofiletest(v, c)
 {
     Char **fileptr, *ftest, *res;
 
+    USE(c);
     if (*(ftest = *++v) != '-')
 	stderror(ERR_NAME | ERR_FILEINQ);
     ++v;
@@ -291,11 +293,11 @@ prvars()
 /*ARGSUSED*/
 void
 doalias(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register struct varent *vp;
-    register Char *p;
+    struct varent *vp;
+    Char *p;
 
     USE(c);
     v++;
@@ -304,7 +306,7 @@ doalias(v, c)
 	plist(&aliases, VAR_ALL);
     else if (*v == 0) {
 	vp = adrof1(strip(p), &aliases);
-	if (vp)
+	if (vp && vp->vec)
 	    blkpr(vp->vec), xputchar('\n');
     }
     else {
@@ -346,15 +348,18 @@ dologin(v, c)
     Char  **v;
     struct command *c;
 {
-    USE(c);
 #ifdef WINNT_NATIVE
+    USE(c);
     USE(v);
 #else /* !WINNT_NATIVE */
+    char **p = short2blk(v);
+    USE(c);
     islogin();
     rechist(NULL, adrof(STRsavehist) != NULL);
     (void) signal(SIGTERM, parterm);
-    (void) execl(_PATH_BIN_LOGIN, "login", short2str(v[1]), NULL);
-    (void) execl(_PATH_USRBIN_LOGIN, "login", short2str(v[1]), NULL);
+    (void) execv(_PATH_BIN_LOGIN, p);
+    (void) execv(_PATH_USRBIN_LOGIN, p);
+    blkfree((Char **) p);
     untty();
     xexit(1);
 #endif /* !WINNT_NATIVE */
@@ -400,8 +405,8 @@ doif(v, kp)
     Char  **v;
     struct command *kp;
 {
-    register int i;
-    register Char **vv;
+    int i;
+    Char **vv;
 
     v++;
     i = expr(&v);
@@ -437,7 +442,7 @@ doif(v, kp)
  */
 void
 reexecute(kp)
-    register struct command *kp;
+    struct command *kp;
 {
     kp->t_dflg &= F_SAVE;
     kp->t_dflg |= F_REPEAT;
@@ -446,7 +451,7 @@ reexecute(kp)
      * pgrp's as the jobs would then have no way to get the tty (we can't give
      * it to them, and our parent wouldn't know their pgrp, etc.
      */
-    execute(kp, (tpgrp > 0 ? tpgrp : -1), NULL, NULL);
+    execute(kp, (tpgrp > 0 ? tpgrp : -1), NULL, NULL, TRUE);
 }
 
 /*ARGSUSED*/
@@ -477,7 +482,7 @@ void
 gotolab(lab)
     Char *lab;
 {
-    register struct whyle *wp;
+    struct whyle *wp;
     /*
      * While we still can, locate any unknown ends of existing loops. This
      * obscure code is the WORST result of the fact that we don't really parse.
@@ -501,10 +506,10 @@ gotolab(lab)
 /*ARGSUSED*/
 void
 doswitch(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register Char *cp, *lp;
+    Char *cp, *lp;
 
     USE(c);
     v++;
@@ -563,11 +568,11 @@ doexit(v, c)
 /*ARGSUSED*/
 void
 doforeach(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register Char *cp, *sp;
-    register struct whyle *nwp;
+    Char *cp, *sp;
+    struct whyle *nwp;
 
     USE(c);
     v++;
@@ -617,8 +622,8 @@ dowhile(v, c)
     Char  **v;
     struct command *c;
 {
-    register int status;
-    register bool again = whyles != 0 && 
+    int status;
+    bool again = whyles != 0 && 
 			  SEEKEQ(&whyles->w_start, &lineloc) &&
 			  whyles->w_fename == 0;
 
@@ -635,7 +640,7 @@ dowhile(v, c)
     if (*v)
 	stderror(ERR_NAME | ERR_EXPRESSION);
     if (!again) {
-	register struct whyle *nwp =
+	struct whyle *nwp =
 	(struct whyle *) xcalloc(1, sizeof(*nwp));
 
 	nwp->w_start = lineloc;
@@ -731,21 +736,23 @@ dorepeat(v, kp)
     Char  **v;
     struct command *kp;
 {
-    register int i;
+    int i = 1;
 
 #ifdef BSDSIGS
-    register sigmask_t omask = 0;
-
+    sigmask_t omask = 0;
 #endif /* BSDSIGS */
 
-    i = getn(v[1]);
+    do {
+	i *= getn(v[1]);
+	lshift(v, 2);
+    } while (v[0] != NULL && Strcmp(v[0], STRrepeat) == 0);
+
     if (setintr)
 #ifdef BSDSIGS
 	omask = sigblock(sigmask(SIGINT)) & ~sigmask(SIGINT);
 #else /* !BSDSIGS */
 	(void) sighold(SIGINT);
 #endif /* BSDSIGS */
-    lshift(v, 2);
     while (i > 0) {
 	if (setintr)
 #ifdef BSDSIGS
@@ -806,11 +813,11 @@ srchx(cp)
     return (-1);
 }
 
-static char *
+static const char *
 isrchx(n)
-    register int n;
+    int n;
 {
-    register struct srch *sp, *sp2;
+    struct srch *sp, *sp2;
 
     for (sp = srchn, sp2 = srchn + nsrchn; sp < sp2; sp++)
 	if (sp->s_value == n)
@@ -825,12 +832,14 @@ static Char *Sgoal;
 static void
 search(type, level, goal)
     int     type;
-    register int level;
+    int level;
     Char   *goal;
 {
     Char    wordbuf[BUFSIZE];
-    register Char *aword = wordbuf;
-    register Char *cp;
+    Char *aword = wordbuf;
+    Char *cp;
+    struct whyle *wp;
+    int wlevel = 0;
 
     Stype = (Char) type;
     Sgoal = goal;
@@ -868,13 +877,24 @@ search(type, level, goal)
 
 	case TC_FOREACH:
 	case TC_WHILE:
+	    wlevel++;
 	    if (type == TC_BREAK)
 		level++;
 	    break;
 
 	case TC_END:
+	    if (type == TC_BRKSW) {
+		if (wlevel == 0) {
+		    wp = whyles;
+		    if (wp) {
+			    whyles = wp->w_next;
+			    wpfree(wp);
+		    }
+		}
+	    }
 	    if (type == TC_BREAK)
 		level--;
+	    wlevel--;
 	    break;
 
 	case TC_SWITCH:
@@ -926,10 +946,10 @@ search(type, level, goal)
 
 static int
 getword(wp)
-    register Char *wp;
+    Char *wp;
 {
     int found = 0, first;
-    int c, d;
+    eChar c, d;
 
     c = readc(1);
     d = 0;
@@ -939,8 +959,8 @@ getword(wp)
 	if (c == '#')
 	    do
 		c = readc(1);
-	    while (c >= 0 && c != '\n');
-	if (c < 0)
+	    while (c != CHAR_ERR && c != '\n');
+	if (c == CHAR_ERR)
 	    goto past;
 	if (c == '\n') {
 	    if (wp)
@@ -960,7 +980,7 @@ getword(wp)
 		else if (d == c)
 		    d = 0;
 	    }
-	    if (c < 0)
+	    if (c == CHAR_ERR)
 		goto past;
 	    if (wp) {
 		*wp++ = (Char) c;
@@ -1031,6 +1051,17 @@ toend()
     wfree();
 }
 
+static void
+wpfree(wp)
+    struct whyle *wp;
+{
+	if (wp->w_fe0)
+	    blkfree(wp->w_fe0);
+	if (wp->w_fename)
+	    xfree((ptr_t) wp->w_fename);
+	xfree((ptr_t) wp);
+}
+
 void
 wfree()
 {
@@ -1052,7 +1083,7 @@ wfree()
 #endif /* FDEBUG */
 
     for (; whyles; whyles = nwp) {
-	register struct whyle *wp = whyles;
+	struct whyle *wp = whyles;
 	nwp = wp->w_next;
 
 #ifdef FDEBUG
@@ -1080,11 +1111,7 @@ wfree()
 	    }
 	}
 
-	if (wp->w_fe0)
-	    blkfree(wp->w_fe0);
-	if (wp->w_fename)
-	    xfree((ptr_t) wp->w_fename);
-	xfree((ptr_t) wp);
+	wpfree(wp);
     }
 }
 
@@ -1112,9 +1139,9 @@ doglob(v, c)
 static void
 xecho(sep, v)
     int    sep;
-    register Char **v;
+    Char **v;
 {
-    register Char *cp;
+    Char *cp;
     int     nonl = 0;
 #ifdef ECHO_STYLE
     int	    echo_style = ECHO_STYLE;
@@ -1163,7 +1190,7 @@ xecho(sep, v)
 	nonl++, v++;
 
     while ((cp = *v++) != 0) {
-	register int c;
+	Char c;
 
 	while ((c = *cp++) != 0) {
 	    if ((echo_style & SYSV_ECHO) != 0 && c == '\\') {
@@ -1220,7 +1247,7 @@ xecho(sep, v)
 		    break;
 		}
 	    }
-	    xputchar(c | QUOTE);
+	    xputwchar(c | QUOTE);
 
 	}
 	if (*v)
@@ -1247,10 +1274,10 @@ islocale_var(var)
     Char *var;
 {
     static Char *locale_vars[] = {
-	STRLANG,	STRLC_CTYPE,	STRLC_NUMERIC,	STRLC_TIME,
-	STRLC_COLLATE,	STRLC_MESSAGES,	STRLC_MONETARY, 0
+	STRLANG,	STRLC_ALL, 	STRLC_CTYPE,	STRLC_NUMERIC,
+	STRLC_TIME,	STRLC_COLLATE,	STRLC_MESSAGES,	STRLC_MONETARY, 0
     };
-    register Char **v;
+    Char **v;
 
     for (v = locale_vars; *v; ++v)
 	if (eq(var, *v))
@@ -1261,12 +1288,10 @@ islocale_var(var)
 /*ARGSUSED*/
 void
 doprintenv(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     Char   *e;
-    extern bool output_raw;
-    extern bool xlate_cr;
 
     USE(c);
     if (setintr)
@@ -1278,7 +1303,7 @@ doprintenv(v, c)
 
     v++;
     if (*v == 0) {
-	register Char **ep;
+	Char **ep;
 
 	xlate_cr = 1;
 	for (ep = STR_environ; *ep; ep++)
@@ -1300,7 +1325,7 @@ doprintenv(v, c)
 /*ARGSUSED*/
 void
 dosetenv(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     Char   *vp, *lp;
@@ -1313,6 +1338,12 @@ dosetenv(v, c)
 
     vp = *v++;
 
+    lp = vp;
+ 
+    for (; *lp != '\0' ; lp++) {
+	if (*lp == '=')
+	    stderror(ERR_NAME | ERR_SYNTAX);
+    }
     if ((lp = *v++) == 0)
 	lp = STRNULL;
 
@@ -1351,16 +1382,16 @@ dosetenv(v, c)
 # ifdef LC_COLLATE
 	(void) setlocale(LC_COLLATE, "");
 # endif
+# ifdef LC_CTYPE
+	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
+# endif /* LC_CTYPE */
 # ifdef NLS_CATALOGS
 #  ifdef LC_MESSAGES
 	(void) setlocale(LC_MESSAGES, "");
 #  endif /* LC_MESSAGES */
-	(void) catclose(catd);
+	nlsclose();
 	nlsinit();
 # endif /* NLS_CATALOGS */
-# ifdef LC_CTYPE
-	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
-# endif /* LC_CTYPE */
 # ifdef SETLOCALEBUG
 	dont_free = 0;
 # endif /* SETLOCALEBUG */
@@ -1370,7 +1401,7 @@ dosetenv(v, c)
 	tw_cmd_free();	/* since the collation sequence has changed */
 	for (k = 0200; k <= 0377 && !Isprint(k); k++)
 	    continue;
-	AsciiOnly = k > 0377;
+	AsciiOnly = MB_CUR_MAX == 1 && k > 0377;
 #else /* !NLS */
 	AsciiOnly = 0;
 #endif /* NLS */
@@ -1382,6 +1413,13 @@ dosetenv(v, c)
 	return;
     }
 
+#ifdef NLS_CATALOGS
+    if (eq(vp, STRNLSPATH)) {
+	nlsclose();
+	nlsinit();
+    }
+#endif
+
     if (eq(vp, STRNOREBIND)) {
 	NoNLSRebind = 1;
 	MapsAreInited = 0;
@@ -1475,7 +1513,7 @@ dosetenv(v, c)
 /*ARGSUSED*/
 void
 dounsetenv(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     Char  **ep, *p, *n;
@@ -1536,16 +1574,16 @@ dounsetenv(v, c)
 # ifdef LC_COLLATE
 		    (void) setlocale(LC_COLLATE, "");
 # endif
+# ifdef LC_CTYPE
+	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
+# endif /* LC_CTYPE */
 # ifdef NLS_CATALOGS
 #  ifdef LC_MESSAGES
 		    (void) setlocale(LC_MESSAGES, "");
 #  endif /* LC_MESSAGES */
-		    (void) catclose(catd);
+		    nlsclose();
 		    nlsinit();
 # endif /* NLS_CATALOGS */
-# ifdef LC_CTYPE
-	(void) setlocale(LC_CTYPE, ""); /* for iscntrl */
-# endif /* LC_CTYPE */
 # ifdef SETLOCALEBUG
 		    dont_free = 0;
 # endif /* SETLOCALEBUG */
@@ -1555,7 +1593,7 @@ dounsetenv(v, c)
 		    tw_cmd_free();/* since the collation sequence has changed */
 		    for (k = 0200; k <= 0377 && !Isprint(k); k++)
 			continue;
-		    AsciiOnly = k > 0377;
+		    AsciiOnly = MB_CUR_MAX == 1 && k > 0377;
 #else /* !NLS */
 		    AsciiOnly = getenv("LANG") == NULL &&
 			getenv("LC_CTYPE") == NULL;
@@ -1576,6 +1614,12 @@ dounsetenv(v, c)
 		else if (eq(name, STRLS_COLORS))
 		    parseLS_COLORS(n);
 #endif /* COLOR_LS_F */
+#ifdef NLS_CATALOGS
+		else if (eq(name, STRNLSPATH)) {
+		    nlsclose();
+		    nlsinit();
+		}
+#endif
 		/*
 		 * start again cause the environment changes
 		 */
@@ -1586,7 +1630,7 @@ dounsetenv(v, c)
 
 void
 tsetenv(name, val)
-    Char   *name, *val;
+    const Char *name, *val;
 {
 #ifdef SETENV_IN_LIB
 /*
@@ -1604,8 +1648,9 @@ tsetenv(name, val)
     (void) strcpy(nameBuf, cname);
     setenv(nameBuf, short2str(val), 1);
 #else /* !SETENV_IN_LIB */
-    register Char **ep = STR_environ;
-    register Char *cp, *dp;
+    Char **ep = STR_environ;
+    const Char *ccp;
+    Char *cp, *dp;
     Char   *blk[2];
     Char  **oep = ep;
 
@@ -1614,13 +1659,13 @@ tsetenv(name, val)
 #endif /* WINNT_NATIVE */
     for (; *ep; ep++) {
 #ifdef WINNT_NATIVE
-	for (cp = name, dp = *ep; *cp && Tolower(*cp & TRIM) == Tolower(*dp);
-				cp++, dp++)
+	for (ccp = name, dp = *ep; *ccp && Tolower(*ccp & TRIM) == Tolower(*dp);
+				ccp++, dp++)
 #else
-	for (cp = name, dp = *ep; *cp && (*cp & TRIM) == *dp; cp++, dp++)
+	for (ccp = name, dp = *ep; *ccp && (*ccp & TRIM) == *dp; ccp++, dp++)
 #endif /* WINNT_NATIVE */
 	    continue;
-	if (*cp != 0 || *dp != '=')
+	if (*ccp != 0 || *dp != '=')
 	    continue;
 	cp = Strspl(STRequal, val);
 	xfree((ptr_t) * ep);
@@ -1645,8 +1690,8 @@ void
 Unsetenv(name)
     Char   *name;
 {
-    register Char **ep = STR_environ;
-    register Char *cp, *dp;
+    Char **ep = STR_environ;
+    Char *cp, *dp;
     Char **oep = ep;
 
 #ifdef WINNT_NATIVE
@@ -1672,11 +1717,11 @@ Unsetenv(name)
 /*ARGSUSED*/
 void
 doumask(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register Char *cp = v[1];
-    register int i;
+    Char *cp = v[1];
+    int i;
 
     USE(c);
     if (cp == 0) {
@@ -1711,22 +1756,22 @@ doumask(v, c)
 #   define toset(a) ((a) + 1)
 #  endif /* aiws */
 # else /* BSDLIMIT */
-#  if defined(BSD4_4) && !defined(__386BSD__)
-    typedef quad_t RLIM_TYPE;
+#  if (defined(BSD4_4) || defined(__linux__) || (HPUXVERSION >= 1100)) && !defined(__386BSD__)
+    typedef rlim_t RLIM_TYPE;
 #  else
 #   if defined(SOLARIS2) || (defined(sgi) && SYSVREL > 3)
      typedef rlim_t RLIM_TYPE;
 #   else
 #    if defined(_SX)
       typedef long long RLIM_TYPE;
-#    else /* _SX */
+#    else /* !_SX */
       typedef unsigned long RLIM_TYPE;
 #    endif /* _SX */
 #   endif /* SOLARIS2 || (sgi && SYSVREL > 3) */
 #  endif /* BSD4_4 && !__386BSD__  */
 # endif /* BSDLIMIT */
 
-# if (HPUXVERSION > 700) && defined(BSDLIMIT)
+# if (HPUXVERSION > 700) && (HPUXVERSION < 1100) && defined(BSDLIMIT)
 /* Yes hpux8.0 has limits but <sys/resource.h> does not make them public */
 /* Yes, we could have defined _KERNEL, and -I/etc/conf/h, but is that better? */
 #  ifndef RLIMIT_CPU
@@ -1747,7 +1792,7 @@ doumask(v, c)
 #  ifndef SIGRTMIN
 #   define FILESIZE512
 #  endif /* SIGRTMIN */
-# endif /* (HPUXVERSION > 700) && BSDLIMIT */
+# endif /* (HPUXVERSION > 700) && (HPUXVERSION < 1100) && BSDLIMIT */
 
 # if SYSVREL > 3 && defined(BSDLIMIT) && !defined(_SX)
 /* In order to use rusage, we included "/usr/ucbinclude/sys/resource.h" in */
@@ -1825,16 +1870,20 @@ struct limits limits[] =
     { RLIMIT_NPROC,	"maxproc",	1,	""		},
 # endif /* RLIMIT_NPROC */
 
-# ifdef RLIMIT_OFILE
+# if defined(RLIMIT_OFILE) && !defined(RLIMIT_NOFILE)
     { RLIMIT_OFILE,	"openfiles",	1,	""		},
-# endif /* RLIMIT_OFILE */
+# endif /* RLIMIT_OFILE && !defined(RLIMIT_NOFILE) */
+
+# ifdef RLIMIT_SBSIZE
+    { RLIMIT_SBSIZE,	"sbsize",	1,	""		},
+# endif /* RLIMIT_SBSIZE */
 
     { -1, 		NULL, 		0, 	NULL		}
 };
 
 static struct limits *findlim	__P((Char *));
 static RLIM_TYPE getval		__P((struct limits *, Char **));
-static void limtail		__P((Char *, char*));
+static void limtail		__P((Char *, const char *));
 static void plim		__P((struct limits *, int));
 static int setlim		__P((struct limits *, int, RLIM_TYPE));
 
@@ -1862,7 +1911,7 @@ static struct limits *
 findlim(cp)
     Char   *cp;
 {
-    register struct limits *lp, *res;
+    struct limits *lp, *res;
 
     res = (struct limits *) NULL;
     for (lp = limits; lp->limconst >= 0; lp++)
@@ -1881,11 +1930,11 @@ findlim(cp)
 /*ARGSUSED*/
 void
 dolimit(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register struct limits *lp;
-    register RLIM_TYPE limit;
+    struct limits *lp;
+    RLIM_TYPE limit;
     int    hard = 0;
 
     USE(c);
@@ -1911,13 +1960,10 @@ dolimit(v, c)
 
 static  RLIM_TYPE
 getval(lp, v)
-    register struct limits *lp;
+    struct limits *lp;
     Char  **v;
 {
-    register float f;
-#ifndef atof	/* This can be a macro on linux */
-    extern double  atof __P((const char *));
-#endif /* atof */
+    float f;
     Char   *cp = *v++;
 
     f = atof(short2str(cp));
@@ -2015,9 +2061,9 @@ badscal:
 static void
 limtail(cp, str)
     Char   *cp;
-    char   *str;
+    const char   *str;
 {
-    char *sp;
+    const char *sp;
 
     sp = str;
     while (*cp && *cp == *str)
@@ -2030,16 +2076,16 @@ limtail(cp, str)
 /*ARGSUSED*/
 static void
 plim(lp, hard)
-    register struct limits *lp;
+    struct limits *lp;
     int hard;
 {
 # ifdef BSDLIMIT
     struct rlimit rlim;
 # endif /* BSDLIMIT */
     RLIM_TYPE limit;
-    int     div = lp->limdiv;
+    int     xdiv = lp->limdiv;
 
-    xprintf("%s \t", lp->limname);
+    xprintf("%-13.13s", lp->limname);
 
 # ifndef BSDLIMIT
     limit = ulimit(lp->limconst, 0);
@@ -2061,7 +2107,7 @@ plim(lp, hard)
 	if (limit >= (RLIM_INFINITY / 512))
 	    limit = RLIM_INFINITY;
 	else
-	    div = (div == 1024 ? 2 : 1);
+	    xdiv = (xdiv == 1024 ? 2 : 1);
     }
 # endif /* !BSDLIMIT || FILESIZE512 */
 
@@ -2073,17 +2119,17 @@ plim(lp, hard)
 	psecs((long) limit);
     else
 # endif /* RLIMIT_CPU */
-	xprintf("%ld %s", (long) (limit / div), lp->limscale);
+	xprintf("%ld %s", (long) (limit / xdiv), lp->limscale);
     xputchar('\n');
 }
 
 /*ARGSUSED*/
 void
 dounlimit(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register struct limits *lp;
+    struct limits *lp;
     int    lerr = 0;
     int    hard = 0;
     int	   force = 0;
@@ -2122,7 +2168,7 @@ dounlimit(v, c)
 
 static int
 setlim(lp, hard, limit)
-    register struct limits *lp;
+    struct limits *lp;
     int    hard;
     RLIM_TYPE limit;
 {
@@ -2143,6 +2189,9 @@ setlim(lp, hard, limit)
     else
 	rlim.rlim_cur = limit;
 
+    if (rlim.rlim_cur > rlim.rlim_max)
+	rlim.rlim_max = rlim.rlim_cur;
+
     if (setrlimit(lp->limconst, &rlim) < 0) {
 # else /* BSDLIMIT */
     if (limit != RLIM_INFINITY && lp->limconst == RLIMIT_FSIZE)
@@ -2153,10 +2202,10 @@ setlim(lp, hard, limit)
 # endif /* aiws */
     if (ulimit(toset(lp->limconst), limit) < 0) {
 # endif /* BSDLIMIT */
-	xprintf(CGETS(15, 1, "%s: %s: Can't %s%s limit\n"), bname, lp->limname,
-		limit == RLIM_INFINITY ? CGETS(15, 2, "remove") :
-		CGETS(15, 3, "set"),
-		hard ? CGETS(14, 4, " hard") : "");
+	xprintf(CGETS(15, 1, "%s: %s: Can't %s%s limit (%s)\n"), bname,
+	    lp->limname, limit == RLIM_INFINITY ? CGETS(15, 2, "remove") :
+	    CGETS(15, 3, "set"), hard ? CGETS(14, 4, " hard") : "",
+	    strerror(errno));
 	return (-1);
     }
     return (0);
@@ -2196,6 +2245,8 @@ dosuspend(v, c)
     if (tpgrp != -1) {
 retry:
 	ctpgrp = tcgetpgrp(FSHTTY);
+	if (ctpgrp == -1)
+	    stderror(ERR_SYSTEM, "tcgetpgrp", strerror(errno));
 	if (ctpgrp != opgrp) {
 	    old = signal(SIGTTIN, SIG_DFL);
 	    (void) kill(0, SIGTTIN);
@@ -2273,9 +2324,9 @@ doeval(v, c)
 	trim(gav);
     }
 
-    saveIN = dcopy(SHIN, -1);
-    saveOUT = dcopy(SHOUT, -1);
-    saveDIAG = dcopy(SHDIAG, -1);
+    (void)close_on_exec(saveIN = dcopy(SHIN, -1), 1);
+    (void)close_on_exec(saveOUT = dcopy(SHOUT, -1), 1);
+    (void)close_on_exec(saveDIAG = dcopy(SHDIAG, -1), 1);
 
     getexit(osetexit);
 
@@ -2289,9 +2340,9 @@ doeval(v, c)
 #endif /* cray */
 	evalvec = gav;
 	evalp = 0;
-	SHIN = dcopy(0, -1);
-	SHOUT = dcopy(1, -1);
-	SHDIAG = dcopy(2, -1);
+	(void)close_on_exec(SHIN = dcopy(0, -1), 1);
+	(void)close_on_exec(SHOUT = dcopy(1, -1), 1);
+	(void)close_on_exec(SHDIAG = dcopy(2, -1), 1);
 #ifndef CLOSE_ON_EXEC
 	didcch = 0;
 #endif /* CLOSE_ON_EXEC */
@@ -2309,9 +2360,9 @@ doeval(v, c)
     (void) close(SHIN);
     (void) close(SHOUT);
     (void) close(SHDIAG);
-    SHIN = dmove(saveIN, oSHIN);
-    SHOUT = dmove(saveOUT, oSHOUT);
-    SHDIAG = dmove(saveDIAG, oSHDIAG);
+    (void)close_on_exec (SHIN = dmove(saveIN, oSHIN), 1);
+    (void)close_on_exec (SHOUT = dmove(saveOUT, oSHOUT), 1);
+    (void)close_on_exec (SHDIAG = dmove(saveDIAG, oSHDIAG), 1);
 
     if (gv)
 	blkfree(gv);
@@ -2334,12 +2385,8 @@ struct command *c;
     /* would use print_by_column() in tw.parse.c but that assumes
      * we have an array of Char * to pass.. (sg)
      */
-    extern int Tty_raw_mode;
-    extern int TermH;		/* from the editor routines */
-    extern int lbuffed;		/* from sh.print.c */
-
-    register struct biltins *b;
-    register int row, col, columns, rows;
+    struct biltins *b;
+    int row, col, columns, rows;
     unsigned int w, maxwidth;
 
     USE(c);
@@ -2385,6 +2432,54 @@ struct command *c;
     flush();
 }
 
+#ifdef HAVE_ICONV
+char *
+iconv_catgets(ctd, set_id, msg_id, s)
+nl_catd ctd;
+int set_id, msg_id;
+const char *s;
+{
+    static char *buf = NULL;
+    static size_t buf_size = 0;
+  
+    char *orig, *dest, *p;
+#ifdef __NetBSD__
+    const char *src;
+#else
+    char *src;
+#endif
+    size_t src_size, dest_size;
+  
+    orig = catgets(ctd, set_id, msg_id, s);
+    if (catgets_iconv == (iconv_t)-1 || orig == s)
+        return orig;
+    src = orig;
+    src_size = strlen(src) + 1;
+    if (buf == NULL && (buf = xmalloc(buf_size = src_size + 32)) == NULL)
+	return orig;
+    dest = buf;
+    while (src_size != 0) {
+        dest_size = buf + buf_size - dest;
+	if (iconv(catgets_iconv, &src, &src_size, &dest, &dest_size)
+	    == (size_t)-1) {
+	    switch (errno) {
+	    case E2BIG:
+		if ((p = xrealloc(buf, buf_size * 2)) == NULL)
+		    return orig;
+		buf_size *= 2;
+		dest = p + (dest - buf);
+		buf = p;
+		break;
+		
+	    case EILSEQ: case EINVAL: default:
+		return orig;
+	    }
+	}
+    }
+    return buf;
+}
+#endif
+
 void
 nlsinit()
 {
@@ -2395,6 +2490,11 @@ nlsinit()
         xsnprintf((char *)catalog, sizeof(catalog), "tcsh.%s",
 		  short2str(varval(STRcatalog)));
     catd = catopen(catalog, MCLoadBySet);
+#ifdef HAVE_ICONV
+    /* catgets (), not CGETS, the charset name should be in ASCII anyway. */
+    catgets_iconv = iconv_open (nl_langinfo (CODESET),
+				catgets(catd, 255, 1, "ASCII"));
+#endif /* HAVE_ICONV */
 #endif /* NLS_CATALOGS */
 #ifdef WINNT_NATIVE
     nls_dll_init();
@@ -2405,3 +2505,17 @@ nlsinit()
     editinit();		/* init the editor messages */
     terminit();		/* init the termcap messages */
 }
+
+void
+nlsclose()
+{
+#ifdef NLS_CATALOGS
+#ifdef HAVE_ICONV
+    if (catgets_iconv != (iconv_t)-1) {
+	iconv_close(catgets_iconv);
+	catgets_iconv = (iconv_t)-1;
+    }
+#endif /* HAVE_ICONV */
+    catclose(catd);
+#endif /* NLS_CATALOGS */
+}
