@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.glob.c,v 3.55 2004/03/21 16:48:14 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.glob.c,v 3.59 2004/11/21 04:29:39 christos Exp $ */
 /*
  * sh.glob.c: Regular expression expansion
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.glob.c,v 3.55 2004/03/21 16:48:14 christos Exp $")
+RCSID("$Id: sh.glob.c,v 3.59 2004/11/21 04:29:39 christos Exp $")
 
 #include "tc.h"
 #include "tw.h"
@@ -80,7 +80,7 @@ static	Char	**globexpand	__P((Char **));
 static	int	  globbrace	__P((Char *, Char *, Char ***));
 static  void	  expbrace	__P((Char ***, Char ***, int));
 static	void	  pword		__P((int));
-static	void	  psave		__P((int));
+static	void	  psave		__P((Char));
 static	void	  backeval	__P((Char *, bool));
 
 static Char *
@@ -652,10 +652,10 @@ ginit()
 
 void
 rscan(t, f)
-    register Char **t;
-    void    (*f) __P((int));
+    Char **t;
+    void    (*f) __P((Char));
 {
-    register Char *p;
+    Char *p;
 
     while ((p = *t++) != '\0')
 	while (*p)
@@ -664,9 +664,9 @@ rscan(t, f)
 
 void
 trim(t)
-    register Char **t;
+    Char **t;
 {
-    register Char *p;
+    Char *p;
 
     while ((p = *t++) != '\0')
 	while (*p)
@@ -675,9 +675,9 @@ trim(t)
 
 void
 tglob(t)
-    register Char **t;
+    Char **t;
 {
-    register Char *p, *c;
+    Char *p, *c;
 
     while ((p = *t++) != '\0') {
 	if (*p == '~' || *p == '=')
@@ -734,7 +734,7 @@ dobackp(cp, literal)
     Char   *cp;
     bool    literal;
 {
-    register Char *lp, *rp;
+    Char *lp, *rp;
     Char   *ep, word[LONGBSIZE];
 
     if (pargv) {
@@ -788,8 +788,8 @@ backeval(cp, literal)
     Char   *cp;
     bool    literal;
 {
-    register int icnt, c;
-    register Char *ip;
+    int icnt;
+    Char c, *ip;
     struct command faket;
     bool    hadnl;
     int     pvec[2], quoted;
@@ -824,7 +824,8 @@ backeval(cp, literal)
      */
     mypipe(pvec);
     if (pfork(&faket, -1) == 0) {
-	struct command *t;
+	jmp_buf_t osetexit;
+	struct command *volatile t;
 
 	(void) close(pvec[0]);
 	(void) dmove(pvec[1], 1);
@@ -841,7 +842,7 @@ backeval(cp, literal)
 	arginp = cp;
 	for (arginp = cp; *cp; cp++) {
 	    *cp &= TRIM;
-	    if (*cp == '\n' || *cp == '\r')
+	    if (is_set(STRcsubstnonl) && (*cp == '\n' || *cp == '\r'))
 		*cp = ' ';
 	}
 
@@ -853,26 +854,58 @@ backeval(cp, literal)
 	evalvec = NULL;
 	alvecp = NULL;
 	evalp = NULL;
-	(void) lex(&paraml);
-	if (seterr)
-	    stderror(ERR_OLD);
-	alias(&paraml);
-	t = syntax(paraml.next, &paraml, 0);
-	if (seterr)
-	    stderror(ERR_OLD);
-	if (t)
-	    t->t_dflg |= F_NOFORK;
+
+	t = NULL;
+	getexit(osetexit);
+	for (;;) {
+
+	    if (paraml.next && paraml.next != &paraml)
+		freelex(&paraml);
+	    
+	    paraml.next = paraml.prev = &paraml;
+	    paraml.word = STRNULL;
+	    (void) setexit();
+	    justpr = 0;
+	    
+	    /*
+	     * For the sake of reset()
+	     */
+	    freelex(&paraml);
+	    if (t)
+		freesyn(t), t = NULL;
+
+	    if (haderr) {
+		/* unwind */
+		doneinp = 0;
+		resexit(osetexit);
+		reset();
+	    }
+	    if (seterr) {
+		xfree((ptr_t) seterr);
+		seterr = NULL;
+	    }
+
+	    (void) lex(&paraml);
+	    if (seterr)
+		stderror(ERR_OLD);
+	    alias(&paraml);
+	    t = syntax(paraml.next, &paraml, 0);
+	    if (seterr)
+		stderror(ERR_OLD);
 #ifdef SIGTSTP
-	(void) sigignore(SIGTSTP);
+	    (void) sigignore(SIGTSTP);
 #endif
 #ifdef SIGTTIN
-	(void) sigignore(SIGTTIN);
+	    (void) sigignore(SIGTTIN);
 #endif
 #ifdef SIGTTOU
-	(void) sigignore(SIGTTOU);
+	    (void) sigignore(SIGTTOU);
 #endif
-	execute(t, -1, NULL, NULL, TRUE);
-	exitstat();
+	    execute(t, -1, NULL, NULL, TRUE);
+
+	    freelex(&paraml);
+	    freesyn(t), t = NULL;
+	}
     }
     xfree((ptr_t) cp);
     (void) close(pvec[1]);
@@ -880,21 +913,55 @@ backeval(cp, literal)
     ip = NULL;
     do {
 	int     cnt = 0;
+	char   *tmp;
 
+	tmp = tibuf;
 	for (;;) {
 	    if (icnt == 0) {
-		int     i;
+		int     i, eof;
 
 		ip = ibuf;
 		do
-		    icnt = read(pvec[0], tibuf, BUFSIZE);
+		    icnt = read(pvec[0], tmp, tibuf + BUFSIZE - tmp);
 		while (icnt == -1 && errno == EINTR);
+		eof = 0;
 		if (icnt <= 0) {
-		    c = -1;
-		    break;
+		    if (tmp == tibuf) {
+		        c = -1;
+			break;
+		    }
+		    icnt = 0;
+		    eof = 1;
 		}
+		icnt += tmp - tibuf;
+#ifdef WIDE_STRINGS
+		i = 0;
+		tmp = tibuf;
+		while (tmp < tibuf + icnt) {
+		    int len;
+
+		    len = mbtowc(&ip[i], tmp, tibuf + icnt - tmp);
+		    if (len == -1) {
+		        mbtowc(NULL, NULL, 0);
+		        if (!eof && (size_t)(tibuf + icnt - tmp) < MB_CUR_MAX) {
+			    break; /* Maybe a partial character */
+			}
+			ip[i] = (unsigned char) *tmp; /* Error */
+		    }
+		    if (len <= 0)
+		        len = 1;
+		    i++;
+		    tmp += len;
+		}
+		if (tmp != tibuf) {
+		    memmove (tibuf, tmp, tibuf + icnt - tmp);
+		    tmp = tibuf + (tibuf + icnt - tmp);
+		}
+		icnt = i;
+#else /* not WIDE_STRINGS */
 		for (i = 0; i < icnt; i++)
 		    ip[i] = (unsigned char) tibuf[i];
+#endif
 	    }
 	    if (hadnl)
 		break;
@@ -936,7 +1003,7 @@ backeval(cp, literal)
 
 static void
 psave(c)
-    int    c;
+    Char   c;
 {
     if (--pnleft <= 0)
 	stderror(ERR_WTOOLONG);
@@ -987,12 +1054,12 @@ Gnmatch(string, pattern, endstr)
     if (endstr == NULL)
 	/* Exact matches only */
 	for (p = blk; *p; p++) 
-	    gres |= t_pmatch(string, *p, &tstring, 0) == 2 ? 1 : 0;
+	    gres |= t_pmatch(string, *p, &tstring, 1) == 2 ? 1 : 0;
     else {
 	/* partial matches */
 	int minc = 0x7fffffff;
 	for (p = blk; *p; p++) 
-	    if (t_pmatch(string, *p, &tstring, 0) != 0) {
+	    if (t_pmatch(string, *p, &tstring, 1) != 0) {
 		int t = (int) (tstring - string);
 		gres |= 1;
 		if (minc == -1 || minc > t)
@@ -1027,9 +1094,9 @@ t_pmatch(string, pattern, estr, cs)
 	 */
 	patternc = *pattern++;
 	switch (patternc) {
-	case 0:
+	case '\0':
 	    *estr = string;
-	    return (stringc == 0 ? 2 : 1);
+	    return (stringc == '\0' ? 2 : 1);
 	case '?':
 	    if (stringc == 0)
 		return (0);
@@ -1079,21 +1146,24 @@ t_pmatch(string, pattern, estr, cs)
 		if (match)
 		    continue;
 		if (rangec == '-' && *(pattern-2) != '[' && *pattern  != ']') {
-		    match = (globcharcoll(stringc, *pattern & TRIM, cs) <= 0 &&
-		    globcharcoll(*(pattern-2) & TRIM, stringc, cs) <= 0);
+		    match = (globcharcoll(stringc, *pattern & TRIM, 0) <= 0 &&
+		    globcharcoll(*(pattern-2) & TRIM, stringc, 0) <= 0);
 		    pattern++;
 		}
 		else 
 		    match = (stringc == (rangec & TRIM));
 	    }
-	    if (rangec == 0)
+	    if (rangec == '\0')
 		stderror(ERR_NAME | ERR_MISSING, ']');
+	    if ((!match) && (stringc == '\0'))
+		return (0);
 	    if (match == negate_range)
 		return (0);
 	    *estr = string;
 	    break;
 	default:
-	    if ((patternc & TRIM) != stringc)
+	    if (cs ? (patternc & TRIM) != stringc
+		: Tolower(patternc & TRIM) != Tolower(stringc))
 		return (0);
 	    *estr = string;
 	    break;
@@ -1105,7 +1175,7 @@ void
 Gcat(s1, s2)
     Char   *s1, *s2;
 {
-    register Char *p, *q;
+    Char *p, *q;
     int     n;
 
     for (p = s1; *p++;)
@@ -1129,7 +1199,7 @@ Gcat(s1, s2)
 #if defined(FILEC) && defined(TIOCSTI)
 int
 sortscmp(a, b)
-    register Char **a, **b;
+    Char **a, **b;
 {
     if (!a)			/* check for NULL */
 	return (b ? 1 : 0);
