@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.h,v 3.114 2004/02/21 20:34:25 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.h,v 3.120 2004/11/20 18:23:03 christos Exp $ */
 /*
  * sh.h: Catch it all globals and includes file!
  */
@@ -35,6 +35,10 @@
 
 #include "config.h"
 
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
 #ifndef HAVE_QUAD
 #ifdef __GNUC__
 #define HAVE_QUAD	1
@@ -78,12 +82,24 @@
 #endif
 
 #ifdef SHORT_STRINGS
+# ifdef WIDE_STRINGS
+#include <wchar.h>
+typedef wchar_t Char;
+typedef unsigned long uChar;
+typedef wint_t eChar; /* Can contain any Char value or CHAR_ERR */
+#define CHAR_ERR WEOF /* Pretty please, use bit 31... */
+# else
 typedef short Char;
 typedef unsigned short uChar;
+typedef int eChar;
+#define CHAR_ERR (-1)
+# endif
 # define SAVE(a) (Strsave(str2short(a)))
 #else
 typedef char Char;
 typedef unsigned char uChar;
+typedef int eChar;
+#define CHAR_ERR (-1)
 # define SAVE(a) (strsave(a))
 #endif 
 
@@ -479,7 +495,6 @@ extern pid_t getpgrp __P((int));
 extern pid_t getpgrp __P((void));
 #  endif	/* BSD || SUNOS4 || IRISD || DGUX */
 # endif /* POSIX */
-extern pid_t setpgrp __P((pid_t, pid_t));
 #endif /* !WINNT_NATIVE */
 
 typedef sigret_t (*signalfun_t) __P((int));
@@ -609,6 +624,9 @@ EXTERN bool    arun IZERO;	/* Currently running multi-line-aliases */
 EXTERN int     implicit_cd IZERO;/* implicit cd enabled?(1=enabled,2=verbose) */
 EXTERN bool    inheredoc IZERO;	/* Currently parsing a heredoc */
 EXTERN bool    windowchg IZERO;	/* We received a window change event */
+#if defined(KANJI) && defined(SHORT_STRINGS) && defined(DSPMBYTE)
+EXTERN bool    dspmbyte_ls;
+#endif
 
 /*
  * Global i/o info
@@ -660,9 +678,9 @@ EXTERN int     backpid;		/* pid of the last background job */
  * uid_t and gid_t are not defined in all the systems so I would have to
  * make special cases for them. In the future...
  */
-EXTERN int     uid, euid, 	/* Invokers real and effective */
-	       gid, egid;	/* User and group ids */
-EXTERN int     opgrp,		/* Initial pgrp and tty pgrp */
+EXTERN uid_t   uid, euid; 	/* Invokers real and effective */
+EXTERN gid_t   gid, egid;	/* User and group ids */
+EXTERN pid_t   opgrp,		/* Initial pgrp and tty pgrp */
                shpgrp,		/* Pgrp of shell */
                tpgrp;		/* Terminal process group */
 				/* If tpgrp is -1, leave tty alone! */
@@ -757,9 +775,19 @@ extern signalfun_t parterm;	/* Parents terminate catch */
  */
 #define		META		0200
 #define		ASCII		0177
-#ifdef SHORT_STRINGS
+#ifdef WIDE_STRINGS		/* Implies SHORT_STRINGS */
+/* 31st char bit used for 'ing (not 32nd, we want all values nonnegative) */
+# define	QUOTE		0x40000000
+# define	TRIM		0x3FFFFFFF /* Mask to strip quote bit */
+# define	UNDER		0x20000000 /* Underline flag */
+# define	BOLD		0x10000000 /* Bold flag */
+# define	STANDOUT	0x08000000 /* Standout flag */
+# define	LITERAL		0x04000000 /* Literal character flag */
+# define	ATTRIBUTES	0x3C000000 /* The bits used for attributes */
+# define	CHAR		0x001FFFFF /* Mask to mask out the character */
+#elif defined (SHORT_STRINGS)
 # define	QUOTE 	((Char)	0100000)/* 16nth char bit used for 'ing */
-# define	TRIM		0077777	/* Mask to strip quote bit */
+# define	TRIM		0073777	/* Mask to strip quote/lit bit */
 # define	UNDER		0040000	/* Underline flag */
 # define	BOLD		0020000	/* Bold flag */
 # define	STANDOUT	0010000	/* Standout flag */
@@ -787,11 +815,15 @@ EXTERN int     AsciiOnly;	/* If set only 7 bits expected in characters */
  * in the buffer.
  */
 EXTERN struct Bin {
-    off_t   Bfseekp;		/* Seek pointer */
+    off_t   Bfseekp;		/* Seek pointer, generally != lseek() value */
     off_t   Bfbobp;		/* Seekp of beginning of buffers */
     off_t   Bfeobp;		/* Seekp of end of buffers */
     int     Bfblocks;		/* Number of buffer blocks */
     Char  **Bfbuf;		/* The array of buffer blocks */
+#ifdef WIDE_STRINGS
+    /* Number of bytes in each character if (cantell) */
+    unsigned char Bfclens[BUFSIZE + 1];
+#endif
 }       B;
 
 /*
@@ -822,6 +854,7 @@ extern int aret;		/* Type of last char returned */
 #define	feobp	B.Bfeobp
 #define	fblocks	B.Bfblocks
 #define	fbuf	B.Bfbuf
+#define fclens  B.Bfclens
 
 /*
  * The shell finds commands in loops by reseeking the input
@@ -962,7 +995,7 @@ typedef void (*bfunc_t) __P((Char **, struct command *));
 #endif /* hpux && __STDC__ && !__GNUC__ */
 
 extern struct biltins {
-    char   *bname;
+    const char   *bname;
     bfunc_t bfunct;
     int     minargs, maxargs;
 } bfunc[];
@@ -971,11 +1004,12 @@ extern int nbfunc;
 extern struct biltins  nt_bfunc[];
 extern int nt_nbfunc;
 #endif /* WINNT_NATIVE*/
+extern int bequiet;
 
 extern struct srch {
-    char   *s_name;
-    int     s_value;
-}       srchn[];
+    const char *s_name;
+    int  s_value;
+} srchn[];
 extern int nsrchn;
 
 /*
@@ -1031,7 +1065,7 @@ EXTERN Char  **alvec IZERO_STRUCT,
  */
 EXTERN int   gflag;		/* After tglob -> is globbing needed? */
 
-#define MAXVARLEN 30		/* Maximum number of char in a variable name */
+#define MAXVARLEN 256		/* Maximum number of char in a variable name */
 
 #ifdef __CYGWIN__
 # undef MAXPATHLEN
@@ -1050,10 +1084,10 @@ EXTERN int   gflag;		/* After tglob -> is globbing needed? */
  * resource limits
  */
 extern struct limits {
-    int     limconst;
-    char   *limname;
-    int     limdiv;
-    char   *limscale;
+    int         limconst;
+    const char *limname;
+    int         limdiv;
+    const char *limscale;
 } limits[];
 #endif /* !HAVENOLIMIT */
 
@@ -1104,8 +1138,12 @@ EXTERN Char    PRCHROOT;	/* Prompt symbol for root */
  * For operating systems with single case filenames (OS/2)
  */
 #ifdef CASE_INSENSITIVE
-# define samecase(x) (isupper((unsigned char)(x)) ? \
-		      tolower((unsigned char)(x)) : (x))
+# ifdef WIDE_STRINGS
+#  define samecase(x) (towlower(x))
+# else
+#  define samecase(x) (isupper((unsigned char)(x)) ? \
+		       tolower((unsigned char)(x)) : (x))
+# endif
 #else
 # define samecase(x) (x)
 #endif /* CASE_INSENSITIVE */
@@ -1135,6 +1173,17 @@ EXTERN Char    PRCHROOT;	/* Prompt symbol for root */
 #define short2blk(a) 		saveblk(a)
 #define short2str(a) 		strip(a)
 #else
+#ifdef WIDE_STRINGS
+#define Strchr(a, b)		wcschr(a, b)
+#define Strrchr(a, b)		wcsrchr(a, b)
+#define Strcat(a, b)  		wcscat(a, b)
+#define Strncat(a, b, c) 	wcsncat(a, b, c)
+#define Strcpy(a, b)  		wcscpy(a, b)
+#define Strncpy(a, b, c)	wcsncpy(a, b, c)
+#define Strlen(a)		wcslen(a)
+#define Strcmp(a, b)		wcscmp(a, b)
+#define Strncmp(a, b, c)	wcsncmp(a, b, c)
+#else
 #define Strchr(a, b)		s_strchr(a, b)
 #define Strrchr(a, b) 		s_strrchr(a, b)
 #define Strcat(a, b)  		s_strcat(a, b)
@@ -1144,6 +1193,7 @@ EXTERN Char    PRCHROOT;	/* Prompt symbol for root */
 #define Strlen(a)		s_strlen(a)
 #define Strcmp(a, b)		s_strcmp(a, b)
 #define Strncmp(a, b, c)	s_strncmp(a, b, c)
+#endif
 #define Strcasecmp(a, b)	s_strcasecmp(a, b)
 
 #define Strspl(a, b)		s_strspl(a, b)
@@ -1155,7 +1205,7 @@ EXTERN Char    PRCHROOT;	/* Prompt symbol for root */
 /*
  * setname is a macro to save space (see sh.err.c)
  */
-EXTERN char   *bname;
+EXTERN const char   *bname;
 
 #define	setname(a)	(bname = (a))
 
@@ -1170,9 +1220,9 @@ EXTERN Char  **evalvec;
 EXTERN Char   *evalp;
 
 extern struct mesg {
-    char   *iname;		/* name from /usr/include */
-    char   *pname;		/* print name */
-}       mesg[];
+    const char   *iname;	/* name from /usr/include */
+    char *pname;		/* print name */
+} mesg[];
 
 /* word_chars is set by default to WORD_CHARS but can be overridden by
    the worchars variable--if unset, reverts to WORD_CHARS */
@@ -1196,9 +1246,22 @@ extern Char    **INVPPTR;
 
 extern char    *progname;
 extern int	tcsh;
+extern bool	xlate_cr;
+extern bool	output_raw;
+extern int	lbuffed;
+extern time_t	Htime;
+extern int	numeof;
+extern int 	insource;
+extern char	linbuf[];
+extern char 	*linp;
+extern int	nsig;
+#ifdef VFORK
+extern bool	use_fork;
+#endif
+extern bool	tellwhat;
+extern bool	NoNLSRebind;
 
 #include "tc.h"
-#include "sh.decls.h"
 
 /*
  * To print system call errors...
@@ -1245,7 +1308,11 @@ extern int errno, sys_nerr;
 #   define MCLoadBySet 0
 #  endif
 EXTERN nl_catd catd;
-#  define CGETS(b, c, d)	catgets(catd, b, c, d)
+#  ifdef HAVE_ICONV
+#   define CGETS(b, c, d)	iconv_catgets(catd, b, c, d)
+#  else
+#   define CGETS(b, c, d)	catgets(catd, b, c, d)
+#  endif
 #  define CSAVS(b, c, d)	strsave(CGETS(b, c, d))
 # else
 #  define CGETS(b, c, d)	d
@@ -1260,6 +1327,7 @@ EXTERN nl_catd catd;
 extern bool    filec;
 #endif /* FILEC */
 
+#include "sh.decls.h"
 /*
  * Since on some machines characters are unsigned, and the signed
  * keyword is not universally implemented, we treat all characters
