@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.hist.c,v 3.27 2000/06/10 22:07:56 kim Exp $ */
+/* $Header: /src/pub/tcsh/sh.hist.c,v 3.30 2004/08/04 17:12:29 christos Exp $ */
 /*
  * sh.hist.c: Shell history expansions and substitutions
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.hist.c,v 3.27 2000/06/10 22:07:56 kim Exp $")
+RCSID("$Id: sh.hist.c,v 3.30 2004/08/04 17:12:29 christos Exp $")
 
 #include "tc.h"
 
@@ -66,8 +62,8 @@ savehist(sp, mflg)
     struct wordent *sp;
     bool mflg;
 {
-    register struct Hist *hp, *np;
-    register int histlen = 0;
+    struct Hist *hp, *np;
+    int histlen = 0;
     Char   *cp;
 
     /* throw away null lines */
@@ -75,7 +71,7 @@ savehist(sp, mflg)
 	return;
     cp = varval(STRhistory);
     if (*cp) {
-	register Char *p = cp;
+	Char *p = cp;
 
 	while (*p) {
 	    if (!Isdigit(*p)) {
@@ -116,14 +112,13 @@ heq(a0, b0)
 struct Hist *
 enthist(event, lp, docopy, mflg)
     int     event;
-    register struct wordent *lp;
+    struct wordent *lp;
     bool    docopy;
     bool    mflg;
 {
-    extern time_t Htime;
     struct Hist *p = NULL, *pp = &Histlist;
     int n, r;
-    register struct Hist *np;
+    struct Hist *np;
     Char *dp;
     
     if ((dp = varval(STRhistdup)) != STRNULL) {
@@ -214,7 +209,7 @@ enthist(event, lp, docopy, mflg)
 
 static void
 hfree(hp)
-    register struct Hist *hp;
+    struct Hist *hp;
 {
 
     freelex(&hp->Hlex);
@@ -317,10 +312,9 @@ dohist1(hp, np, hflg)
 
 static void
 phist(hp, hflg)
-    register struct Hist *hp;
+    struct Hist *hp;
     int     hflg;
 {
-    extern bool output_raw;
     if (hflg & HIST_ONLY) {
        /*
         * Control characters have to be written as is (output_raw).
@@ -348,12 +342,12 @@ phist(hp, hflg)
 	Char buf[INBUFSIZE];
 	struct varent *vp = adrof(STRhistory);
 
-	if (vp && vp->vec[0] && vp->vec[1])
+	if (vp && vp->vec != NULL && vp->vec[0] && vp->vec[1])
 	    cp = vp->vec[1];
 
 	tprintf(FMT_HISTORY, buf, cp, INBUFSIZE, NULL, hp->Htime, (ptr_t) hp);
 	for (cp = buf; *cp;)
-	    xputchar(*cp++);
+	    xputwchar(*cp++);
     }
 }
 
@@ -377,8 +371,25 @@ fmthist(fmt, ptr, buf, bufsiz)
 	    Char ibuf[INBUFSIZE], *ip;
 	    char *p;
 	    (void) sprlex(ibuf, sizeof(ibuf) / sizeof(Char), &hp->Hlex);
-	    for (p = buf, ip = ibuf; (*p++ = (CHAR & *ip++)) != '\0'; )
+	    p = buf;
+	    ip = ibuf;
+#ifdef WIDE_STRINGS
+	    do {
+	        char xbuf[MB_LEN_MAX];
+		size_t len;
+
+		len = one_wctomb(xbuf, CHAR & *ip);
+		if ((size_t)((p - xbuf) + len) >= bufsiz)
+		    break;
+		memcpy(p, xbuf, len);
+		p += len;
+	    } while ((CHAR & *ip++) != 0);
+#else	    
+	    while (p < buf + bufsiz - 1 && (*p++ = (CHAR & *ip++)) != '\0')
 		continue;
+#endif
+	    if (p <= buf + bufsiz - 1)
+	        *p = '\0';
 	}
 	break;
     default:
@@ -439,7 +450,7 @@ rechist(fname, ref)
      */
     oldidfds = didfds;
     didfds = 0;
-    if ((shist = adrof(STRsavehist)) != NULL)
+    if ((shist = adrof(STRsavehist)) != NULL && shist->vec != NULL)
 	if (shist->vec[1] && eq(shist->vec[1], STRmerge))
 	    loadhist(fname, 1);
     fp = creat(short2str(fname), 0600);
