@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.lex.c,v 3.51 2000/07/04 19:42:47 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.lex.c,v 3.60 2004/08/04 17:12:30 christos Exp $ */
 /*
  * sh.lex.c: Lexical analysis into tokens
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,9 +32,11 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.lex.c,v 3.51 2000/07/04 19:42:47 christos Exp $")
+RCSID("$Id: sh.lex.c,v 3.60 2004/08/04 17:12:30 christos Exp $")
 
 #include "ed.h"
+
+#include <assert.h>
 /* #define DEBUG_INP */
 /* #define DEBUG_SEEK */
 
@@ -51,21 +49,22 @@ RCSID("$Id: sh.lex.c,v 3.51 2000/07/04 19:42:47 christos Exp $")
  * There is some involved processing here, because of the complications
  * of input buffering, and especially because of history substitution.
  */
-static	Char		*word		__P((void));
-static	int	 	 getC1		__P((int));
+static	Char		*word		__P((int));
+static	eChar	 	 getC1		__P((int));
 static	void	 	 getdol		__P((void));
-static	void	 	 getexcl	__P((int));
+static	void	 	 getexcl	__P((Char));
 static	struct Hist 	*findev		__P((Char *, bool));
 static	void	 	 setexclp	__P((Char *));
-static	int	 	 bgetc		__P((void));
+static	eChar	 	 bgetc		__P((void));
 static	void		 balloc		__P((int));
 static	void	 	 bfree		__P((void));
-static	struct wordent	*gethent	__P((int));
+static	struct wordent	*gethent	__P((Char));
 static	int	 	 matchs		__P((Char *, Char *));
 static	int	 	 getsel		__P((int *, int *, int));
 static	struct wordent	*getsub		__P((struct wordent *));
-static	Char 		*subword	__P((Char *, int, bool *));
-static	struct wordent	*dosub		__P((int, struct wordent *, bool));
+static	Char 		*subword	__P((Char *, Char, bool *));
+static	struct wordent	*dosub		__P((Char, struct wordent *, bool));
+static	ssize_t		 wide_read	__P((int, Char *, size_t, int));
 
 /*
  * Peekc is a peek character for getC, peekread for readc.
@@ -130,7 +129,7 @@ static Char *histlinep = NULL;	/* current pointer into histline */
 
 static Char getCtmp;
 
-#define getC(f)		(((getCtmp = peekc) != '\0') ? (peekc = 0, getCtmp) : getC1(f))
+#define getC(f)		(((getCtmp = peekc) != '\0') ? (peekc = 0, (eChar)getCtmp) : getC1(f))
 #define	ungetC(c)	peekc = (Char) c
 #define	ungetD(c)	peekd = (Char) c
 
@@ -140,6 +139,11 @@ static Char getCtmp;
 time_t Htime = (time_t)0;
 static time_t a2time_t __P((Char *));
 
+/*
+ * special parsing rules apply for source -h
+ */
+extern int enterhist;
+
 /*
  * for history event processing
  * in the command 'echo !?foo?:1 !$' we want the !$ to expand from the line
@@ -152,7 +156,8 @@ lex(hp)
     struct wordent *hp;
 {
     struct wordent *wdp;
-    int     c;
+    eChar    c;
+    int     parsehtime = enterhist;
 
 
     uselastevent = 1;
@@ -167,7 +172,7 @@ lex(hp)
     do
 	c = readc(0);
     while (c == ' ' || c == '\t');
-    if (c == HISTSUB && intty)
+    if (c == (eChar)HISTSUB && intty)
 	/* ^lef^rit	from tty is short !:s^lef^rit */
 	getexcl(c);
     else
@@ -187,7 +192,8 @@ lex(hp)
 	wdp->next = new;
 	hp->prev = new;
 	wdp = new;
-	wdp->word = word();
+	wdp->word = word(parsehtime);
+	parsehtime = 0;
     } while (wdp->word[0] != '\n');
     if (histlinep < histline + BUFSIZE) {
 	*histlinep = '\0';
@@ -203,8 +209,8 @@ lex(hp)
 }
 
 static time_t
-a2time_t(word)
-    Char *word;
+a2time_t(wordx)
+    Char *wordx;
 {
     /* Attempt to distinguish timestamps from other possible entries.
      * Format: "+NNNNNNNNNN" (10 digits, left padded with ascii '0') */
@@ -213,7 +219,7 @@ a2time_t(word)
     Char *s;
     int ct;
 
-    if (!word || *(s = word) != '+')
+    if (!wordx || *(s = wordx) != '+')
 	return (time_t)0;
 
     for (++s, ret = 0, ct = 0; *s; ++s, ++ct)
@@ -286,9 +292,10 @@ freelex(vp)
 }
 
 static Char *
-word()
+word(parsehtime)
+    int parsehtime;
 {
-    Char c, c1;
+    eChar c, c1;
     Char *wp;
     Char    wbuf[BUFSIZE];
     Char    hbuf[12];
@@ -319,18 +326,20 @@ loop:
 	    goto ret;
 
 	case '#':
-	    if (intty)
+	    if (intty || (enterhist && !parsehtime))
 		break;
 	    c = 0;
 	    h = 0;
 	    do {
 		c1 = c;
 		c = getC(0);
-		if (h < 12)
+		if (h < 11 && parsehtime)
 		    hbuf[h++] = c;
 	    } while (c != '\n');
-	    hbuf[11] = '\0';
-	    Htime = a2time_t(hbuf); 
+	    if (parsehtime) {
+		hbuf[11] = '\0';
+		Htime = a2time_t(hbuf); 
+	    }
 	    if (c1 == '\\')
 		goto loop;
 	    /*FALLTHROUGH*/
@@ -349,7 +358,7 @@ loop:
 		    onelflg = 2;
 		goto loop;
 	    }
-	    if (c != HIST)
+	    if (c != (eChar)HIST)
 		*wp++ = '\\', --i;
 	    c |= QUOTE;
 	default:
@@ -379,7 +388,7 @@ loop:
  *
  * Actually, all I really want to be able to say is 'foo\'bar' --> foo'bar
  */
-		if (c == HIST)
+		if (c == (eChar)HIST)
 		    c |= QUOTE;
 		else {
 		    if (bslash_quote &&
@@ -412,7 +421,7 @@ loop:
 			onelflg = 2;
 		    break;
 		}
-		if (c != HIST)
+		if (c != (eChar)HIST)
 		    *wp++ = '\\', --i;
 		c |= QUOTE;
 	    }
@@ -420,7 +429,7 @@ loop:
 		c1 = c;
 		dolflg = c == '"' ? DOALL : DOEXCL;
 	    }
-	    else if (c != '#' || !intty) {
+	    else if (c != '#' || (!intty && !enterhist)) {
 		ungetC(c);
 		break;
 	    }
@@ -440,11 +449,11 @@ ret:
     return (Strsave(wbuf));
 }
 
-static int
+static eChar
 getC1(flag)
     int flag;
 {
-    Char c;
+    eChar c;
 
     for (;;) {
 	if ((c = peekc) != 0) {
@@ -491,7 +500,7 @@ getC1(flag)
 	    getdol();
 	    continue;
 	}
-	if (c == HIST && (flag & DOEXCL)) {
+	if (c == (eChar)HIST && (flag & DOEXCL)) {
 	    getexcl(0);
 	    continue;
 	}
@@ -505,8 +514,8 @@ getdol()
 {
     Char *np, *ep;
     Char    name[4 * MAXVARLEN + 1];
-    int c;
-    int     sc;
+    eChar c;
+    eChar   sc;
     bool    special = 0, toolong;
 
     np = name, *np++ = '$';
@@ -668,7 +677,7 @@ getdol()
 	    /* scan s// [eichin:19910926.0512EST] */
 	    if (c == 's') {
 		int delimcnt = 2;
-		int delim = getC(0);
+		eChar delim = getC(0);
 		*np++ = (Char) delim;
 		
 		if (!delim || letter(delim)
@@ -676,7 +685,7 @@ getdol()
 		    seterror(ERR_BADSUBST);
 		    break;
 		}	
-		while ((c = getC(0)) != (-1)) {
+		while ((c = getC(0)) != CHAR_ERR) {
 		    *np++ = (Char) c;
 		    if(c == delim) delimcnt--;
 		    if(!delimcnt) break;
@@ -745,11 +754,11 @@ static int quesarg;
 
 static void
 getexcl(sc)
-    int    sc;
+    Char    sc;
 {
     struct wordent *hp, *ip;
     int     left, right, dol;
-    int c;
+    eChar c;
 
     if (sc == 0) {
 	sc = getC(0);
@@ -831,9 +840,9 @@ getsub(en)
     struct wordent *en;
 {
     Char *cp;
-    int     delim;
-    int c;
-    int     sc;
+    eChar   delim;
+    eChar   c;
+    eChar   sc;
     bool global;
     Char    orhsb[sizeof(rhsb) / sizeof(Char)];
 
@@ -957,7 +966,7 @@ getsub(en)
 	default:
 	    if (c == '\n')
 		unreadc(c);
-	    seterror(ERR_BADBANGMOD, c);
+	    seterror(ERR_BADBANGMOD, (int)c);
 	    return (en);
 	}
 	(void) Strcpy(slhs, lhsb);
@@ -984,7 +993,7 @@ getsub(en)
 #define HIST_PURGE -50000000
 static struct wordent *
 dosub(sc, en, global)
-    int     sc;
+    Char   sc;
     struct wordent *en;
     bool global;
 {
@@ -1045,7 +1054,7 @@ dosub(sc, en, global)
 static Char *
 subword(cp, type, adid)
     Char   *cp;
-    int     type;
+    Char    type;
     bool   *adid;
 {
     Char    wbuf[BUFSIZE];
@@ -1121,7 +1130,7 @@ subword(cp, type, adid)
 Char   *
 domod(cp, type)
     Char   *cp;
-    int     type;
+    Char    type;
 {
     Char *wp, *xp;
     int c;
@@ -1199,7 +1208,7 @@ getsel(al, ar, dol)
     int *al, *ar;
     int     dol;
 {
-    int c = getC(0);
+    eChar c = getC(0);
     int i;
     bool    first = *al < 0;
 
@@ -1282,16 +1291,16 @@ getsel(al, ar, dol)
 
 static struct wordent *
 gethent(sc)
-    int     sc;
+    Char   sc;
 {
     struct Hist *hp;
     Char *np;
-    int c;
+    eChar c;
     int     event;
     bool    back = 0;
 
-    c = sc == HISTSUB ? HIST : getC(0);
-    if (c == HIST) {
+    c = sc == HISTSUB ? (eChar)HIST : getC(0);
+    if (c == (eChar)HIST) {
 	if (alhistp)
 	    return (alhistp);
 	event = eventno;
@@ -1469,19 +1478,17 @@ setexclp(cp)
 
 void
 unreadc(c)
-    int    c;
+    Char    c;
 {
     peekread = (Char) c;
 }
 
-int
+eChar
 readc(wanteof)
     bool    wanteof;
 {
-    int c;
+    eChar c;
     static  int sincereal;	/* Number of real EOFs we've seen */
-    Char *ptr;			/* For STRignoreeof */
-    int numeof = 0;		/* Value of STRignoreeof */
 
 #ifdef DEBUG_INP
     xprintf("readc\n");
@@ -1491,18 +1498,6 @@ readc(wanteof)
 	return (c);
     }
 
-    /* Compute the value of EOFs */
-    if ((ptr = varval(STRignoreeof)) != STRNULL) {
-	while (*ptr) {
-	    if (!Isdigit(*ptr)) {
-		numeof = 0;
-		break;
-	    }
-	    numeof = numeof * 10 + *ptr++ - '0';
-	}
-    } 
-    if (numeof < 1) numeof = 26;	/* Sanity check */
-
 top:
     aret = TCSH_F_SEEK;
     if (alvecp) {
@@ -1559,7 +1554,7 @@ top:
     do {
 	if (arginp == INVPTR || onelflg == 1) {
 	    if (wanteof)
-		return (-1);
+		return CHAR_ERR;
 	    exitstat();
 	}
 	if (arginp) {
@@ -1573,8 +1568,8 @@ top:
 reread:
 #endif /* BSDJOBS */
 	c = bgetc();
-	if (c < 0) {
-#ifndef WINNT
+	if (c == CHAR_ERR) {
+#ifndef WINNT_NATIVE
 # ifndef POSIX
 #  ifdef TERMIO
 	    struct termio tty;
@@ -1584,11 +1579,11 @@ reread:
 # else /* POSIX */
 	    struct termios tty;
 # endif /* POSIX */
-#endif /* !WINNT */
+#endif /* !WINNT_NATIVE */
 	    if (wanteof)
-		return (-1);
+		return CHAR_ERR;
 	    /* was isatty but raw with ignoreeof yields problems */
-#ifndef WINNT
+#ifndef WINNT_NATIVE
 # ifndef POSIX
 #  ifdef TERMIO
 	    if (ioctl(SHIN, TCGETA, (ioctl_t) & tty) == 0 &&
@@ -1601,16 +1596,15 @@ reread:
 	    if (tcgetattr(SHIN, &tty) == 0 &&
 		(tty.c_lflag & ICANON))
 # endif /* POSIX */
-#else /* WINNT */
+#else /* WINNT_NATIVE */
 	    if (isatty(SHIN))
-#endif /* !WINNT */
+#endif /* !WINNT_NATIVE */
 	    {
-		/* was 'short' for FILEC */
 #ifdef BSDJOBS
 		int     ctpgrp;
 #endif /* BSDJOBS */
 
-		if (++sincereal >= numeof)	/* Too many EOFs?  Bye! */
+		if (numeof != 0 && ++sincereal >= numeof)	/* Too many EOFs?  Bye! */
 		    goto oops;
 #ifdef BSDJOBS
 		if (tpgrp != -1 &&
@@ -1698,12 +1692,79 @@ balloc(buf)
     }
 }
 
-static int
+static ssize_t
+wide_read(fildes, buf, nchars, use_fclens)
+    int fildes;
+    Char *buf;
+    size_t nchars;
+    int use_fclens;
+{
+    char cbuf[BUFSIZE + 1];
+    ssize_t res;
+#ifdef WIDE_STRINGS
+    ssize_t r;
+    size_t partial;
+#endif
+    
+    assert (nchars <= sizeof(cbuf)/sizeof(*cbuf));
+#ifdef WIDE_STRINGS
+    res = 0;
+    partial = 0;
+    do {
+	size_t i;
+	
+	do
+	    r = read(fildes, cbuf + partial,
+		     nchars > partial ? nchars - partial : 1);
+	while (partial != 0 && r < 0 && errno == EINTR);
+	if (partial == 0 && r <= 0)
+	    break;
+	partial += r;
+	i = 0;
+	while (i < partial) {
+	    int len;
+
+	    len = mbtowc(buf + res, cbuf + i, partial - i);
+	    if (len == -1) {
+		if (partial < MB_LEN_MAX && r > 0)
+		    /* Maybe a partial character and there is still a chance
+		       to read more */
+		    break;
+		buf[res] = (unsigned char)cbuf[i];
+	    }
+	    if (len <= 0)
+		len = 1;
+	    if (use_fclens)
+		fclens[res] = len;
+	    i += len;
+	    res++;
+	    nchars--;
+	}
+	if (i != partial)
+	    memmove(cbuf, cbuf + i, partial - i);
+	partial -= i;
+    } while (partial != 0);
+    /* Throwing away possible partial multibyte characters on error */
+    return res != 0 ? res : r;
+#else /* not WIDE_STRINGS */
+    USE(use_fclens);
+    res = read(fildes, cbuf, nchars);
+    if (res > 0) {
+	size_t i;
+	
+	for (i = 0; i < (size_t)res; i++)
+	    buf[i] = (unsigned char) cbuf[i];
+    }
+    return res;
+#endif
+}
+
+static eChar
 bgetc()
 {
+    Char ch;
     int c, off, buf;
     int numleft = 0, roomleft;
-    char    tbuf[BUFSIZE + 1];
 
     if (cantell) {
 	if (fseekp < fbobp || fseekp > feobp) {
@@ -1711,37 +1772,42 @@ bgetc()
 	    (void) lseek(SHIN, fseekp, L_SET);
 	}
 	if (fseekp == feobp) {
-	    int     i;
-
 	    fbobp = feobp;
 	    do
-		c = read(SHIN, tbuf, BUFSIZE);
+		c = wide_read(SHIN, fbuf[0], BUFSIZE, 1);
 	    while (c < 0 && errno == EINTR);
 #ifdef convex
 	    if (c < 0)
 		stderror(ERR_SYSTEM, progname, strerror(errno));
 #endif /* convex */
 	    if (c <= 0)
-		return (-1);
-	    for (i = 0; i < c; i++)
-		fbuf[0][i] = (unsigned char) tbuf[i];
+		return CHAR_ERR;
 	    feobp += c;
 	}
-#ifndef WINNT
-	c = fbuf[0][fseekp - fbobp];
+#ifndef WINNT_NATIVE
+	ch = fbuf[0][fseekp - fbobp];
 	fseekp++;
 #else
 	do {
-	    c = fbuf[0][fseekp - fbobp];
+	    ch = fbuf[0][fseekp - fbobp];
 	    fseekp++;
-	} while(c == '\r');
-#endif /* !WINNT */
-	return (c);
+	} while(ch == '\r');
+#endif /* !WINNT_NATIVE */
+	return (ch);
     }
 
     while (fseekp >= feobp) {
-	if (editing && intty) {		/* then use twenex routine */
+	if ((editing
+#if defined(FILEC) && defined(TIOCSTI)
+	    || filec
+#endif /* FILEC && TIOCSTI */
+	    ) && intty) {		/* then use twenex routine */
 	    fseekp = feobp;		/* where else? */
+#if defined(FILEC) && defined(TIOCSTI)
+	    if (!editing)
+		c = numleft = tenex(InputBuf, BUFSIZE);
+	    else
+#endif /* FILEC && TIOCSTI */
 	    c = numleft = Inputl();	/* PWP: get a line */
 	    while (numleft > 0) {
 		off = (int) feobp % BUFSIZE;
@@ -1750,39 +1816,38 @@ bgetc()
 		roomleft = BUFSIZE - off;
 		if (roomleft > numleft)
 		    roomleft = numleft;
-		(void) memmove((ptr_t) (fbuf[buf] + off), (ptr_t) (InputBuf + c - numleft), (size_t) (roomleft * sizeof(Char)));
+		(void) memmove((ptr_t) (fbuf[buf] + off),
+		    (ptr_t) (InputBuf + c - numleft),
+		    (size_t) (roomleft * sizeof(Char)));
 		numleft -= roomleft;
 		feobp += roomleft;
 	    }
-	}
-	else {
+	} else {
 	    off = (int) feobp % BUFSIZE;
 	    buf = (int) feobp / BUFSIZE;
 	    balloc(buf);
 	    roomleft = BUFSIZE - off;
-	    c = read(SHIN, tbuf, (size_t) roomleft);
-	    if (c > 0) {
-		int     i;
-		Char   *ptr = fbuf[buf] + off;
-
-		for (i = 0; i < c; i++)
-		    ptr[i] = (unsigned char) tbuf[i];
+	    c = wide_read(SHIN, fbuf[buf] + off, (size_t) roomleft, 0);
+	    if (c > 0)
 		feobp += c;
-	    }
 	}
 	if (c == 0 || (c < 0 && fixio(SHIN, errno) == -1))
-	    return (-1);
+	    return CHAR_ERR;
     }
-#ifndef WINNT
-    c = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
+#ifdef SIG_WINDOW
+    if (windowchg)
+	(void) check_window_size(0);	/* for window systems */
+#endif /* SIG_WINDOW */
+#ifndef WINNT_NATIVE
+    ch = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
     fseekp++;
 #else
     do {
-	c = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
+	ch = fbuf[(int) fseekp / BUFSIZE][(int) fseekp % BUFSIZE];
 	fseekp++;
-    } while(c == '\r');
-#endif /* !WINNT */
-    return (c);
+    } while(ch == '\r');
+#endif /* !WINNT_NATIVE */
+    return (ch);
 }
 
 static void
@@ -1829,6 +1894,28 @@ bseek(l)
 	xprintf(CGETS(16, 6, "seek to file %x\n"), fseekp);
 #endif
 	fseekp = l->f_seek;
+#ifdef WIDE_CHARS
+	if (cantell) {
+	    if (fseekp >= fbobp) {
+		size_t i;
+		off_t o;
+
+		o = fbobp;
+		for (i = 0; i < feobp - fbobp; i++) {
+		    if (fseekp == o) {
+			fseekp = fbobp + i;
+			return;
+		    }
+		    o += fclens[i];
+		}
+		if (fseekp == o) {
+		    fseekp = feobp;
+		    return;
+		}
+	    }
+	    fbobp = feobp = fseekp + 1; /* To force lseek() */
+	}
+#endif
 	return;
     default:
 	xprintf(CGETS(16, 7, "Bad seek type %d\n"), aret);
@@ -1857,8 +1944,17 @@ struct Ain *l;
 #endif
 	return;
     case TCSH_F_SEEK:
-	/*SUPPRESS 112*/
-	l->f_seek = fseekp;
+#ifdef WIDE_STRINGS
+	if (cantell && fseekp >= fbobp && fseekp < feobp) {
+	    size_t i;
+	    
+	    l->f_seek = fbobp;
+	    for (i = 0; i < fseekp - fbobp; i++)
+		l->f_seek += fclens[i];
+	} else
+#endif
+	    /*SUPPRESS 112*/
+	    l->f_seek = fseekp;
 	l->a_seek = NULL;
 #ifdef DEBUG_SEEK
 	xprintf(CGETS(16, 10, "tell file %x\n"), fseekp);
