@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.misc.c,v 3.25 2002/11/21 20:02:01 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.misc.c,v 3.30 2004/08/08 06:42:28 christos Exp $ */
 /*
  * sh.misc.c: Miscelaneous functions
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.misc.c,v 3.25 2002/11/21 20:02:01 christos Exp $")
+RCSID("$Id: sh.misc.c,v 3.30 2004/08/08 06:42:28 christos Exp $")
 
 static	int	renum	__P((int, int));
 static  Char  **blkend	__P((Char **));
@@ -44,8 +44,8 @@ static  Char  **blkcat	__P((Char **, Char **));
 
 int
 any(s, c)
-    register char *s;
-    register int c;
+    const char *s;
+    Char c;
 {
     if (!s)
 	return (0);		/* Check for nil pointer */
@@ -68,25 +68,24 @@ setzero(cp, i)
 
 char   *
 strsave(s)
-    register const char *s;
+    const char *s;
 {
-    char   *n;
-    register char *p;
+    char   *n, *r;
+    const char *p;
 
     if (s == NULL)
-	s = (const char *) "";
-    for (p = (char *) s; *p++ != '\0';)
+	s = "";
+    for (p = s; *p++ != '\0';)
 	continue;
-    n = p = (char *) xmalloc((size_t)
-			     ((((const char *) p) - s) * sizeof(char)));
-    while ((*p++ = *s++) != '\0')
+    r = n = (char *) xmalloc((size_t)((((const char *) p) - s) * sizeof(char)));
+    while ((*n++ = *s++) != '\0')
 	continue;
-    return (n);
+    return (r);
 }
 
 static Char  **
 blkend(up)
-    register Char **up;
+    Char **up;
 {
 
     while (*up)
@@ -97,7 +96,7 @@ blkend(up)
 
 void
 blkpr(av)
-    register Char **av;
+    Char **av;
 {
 
     for (; *av; av++) {
@@ -109,7 +108,7 @@ blkpr(av)
 
 void
 blkexpand(av, str)
-    register Char **av;
+    Char **av;
     Char *str;
 {
     *str = '\0';
@@ -122,9 +121,9 @@ blkexpand(av, str)
 
 int
 blklen(av)
-    register Char **av;
+    Char **av;
 {
-    register int i = 0;
+    int i = 0;
 
     while (*av++)
 	i++;
@@ -134,9 +133,9 @@ blklen(av)
 Char  **
 blkcpy(oav, bv)
     Char  **oav;
-    register Char **bv;
+    Char **bv;
 {
-    register Char **av = oav;
+    Char **av = oav;
 
     while ((*av++ = *bv++) != NULL)
 	continue;
@@ -156,7 +155,7 @@ void
 blkfree(av0)
     Char  **av0;
 {
-    register Char **av = av0;
+    Char **av = av0;
 
     if (!av0)
 	return;
@@ -167,9 +166,9 @@ blkfree(av0)
 
 Char  **
 saveblk(v)
-    register Char **v;
+    Char **v;
 {
-    register Char **newv =
+    Char **newv =
     (Char **) xcalloc((size_t) (blklen(v) + 1), sizeof(Char **));
     Char  **onewv = newv;
 
@@ -181,11 +180,11 @@ saveblk(v)
 #if !defined(SHORT_STRINGS) && !defined(POSIX)
 char   *
 strstr(s, t)
-    register const char *s, *t;
+    const char *s, *t;
 {
     do {
-	register const char *ss = s;
-	register const char *tt = t;
+	const char *ss = s;
+	const char *tt = t;
 
 	do
 	    if (*tt == '\0')
@@ -200,24 +199,20 @@ strstr(s, t)
 #ifndef SHORT_STRINGS
 char   *
 strspl(cp, dp)
-    char   *cp, *dp;
+    const char *cp, *dp;
 {
     char   *ep;
-    register char *p, *q;
+    size_t cl, dl;
 
     if (!cp)
 	cp = "";
     if (!dp)
 	dp = "";
-    for (p = cp; *p++ != '\0';)
-	continue;
-    for (q = dp; *q++ != '\0';)
-	continue;
-    ep = (char *) xmalloc((size_t) (((p - cp) + (q - dp) - 1) * sizeof(char)));
-    for (p = ep, q = cp; (*p++ = *q++) != '\0';)
-	continue;
-    for (p--, q = dp; (*p++ = *q++) != '\0';)
-	continue;
+    cl = strlen(cp);
+    dl = strlen(dp);
+    ep = (char *) xmalloc((cl + dl + 1) * sizeof(char));
+    memcpy(ep, cp, cl);
+    memcpy(ep + cl, dp, dl + 1);
     return (ep);
 }
 
@@ -225,9 +220,9 @@ strspl(cp, dp)
 
 Char  **
 blkspl(up, vp)
-    register Char **up, **vp;
+    Char **up, **vp;
 {
-    register Char **wp =
+    Char **wp =
     (Char **) xcalloc((size_t) (blklen(up) + blklen(vp) + 1),
 		      sizeof(Char **));
 
@@ -237,7 +232,7 @@ blkspl(up, vp)
 
 Char
 lastchr(cp)
-    register Char *cp;
+    Char *cp;
 {
 
     if (!cp)
@@ -256,8 +251,13 @@ lastchr(cp)
 void
 closem()
 {
-    register int f;
+    int f;
 
+#ifdef NLS_BUGS
+#ifdef NLS_CATALOGS
+    nlsclose();
+#endif /* NLS_CATALOGS */
+#endif /* NLS_BUGS */
 #ifdef YPBUGS
     /* suggested by Justin Bur; thanks to Karl Kleinpaste */
     fix_yp_bugs();
@@ -276,6 +276,11 @@ closem()
 		(void) open(_PATH_DEVNULL, O_RDONLY|O_LARGEFILE);
 #endif /* NISPLUS */
 	  }
+#ifdef NLS_BUGS
+#ifdef NLS_CATALOGS
+    nlsinit();
+#endif /* NLS_CATALOGS */
+#endif /* NLS_BUGS */
 }
 
 #ifndef CLOSE_ON_EXEC
@@ -288,7 +293,7 @@ closem()
 void
 closech()
 {
-    register int f;
+    int f;
 
     if (didcch)
 	return;
@@ -333,7 +338,7 @@ donefds()
  */
 int
 dmove(i, j)
-    register int i, j;
+    int i, j;
 {
 
     if (i == j || i < 0)
@@ -354,7 +359,7 @@ dmove(i, j)
 
 int
 dcopy(i, j)
-    register int i, j;
+    int i, j;
 {
 
     if (i == j || i < 0 || (j < 0 && i > 2))
@@ -372,9 +377,9 @@ dcopy(i, j)
 
 static int
 renum(i, j)
-    register int i, j;
+    int i, j;
 {
-    register int k = dup(i);
+    int k = dup(i);
 
     if (k < 0)
 	return (-1);
@@ -395,10 +400,10 @@ renum(i, j)
  */
 void
 lshift(v, c)
-    register Char **v;
-    register int c;
+    Char **v;
+    int c;
 {
-    register Char **u;
+    Char **u;
 
     for (u = v; *u && --c >= 0; u++)
 	xfree((ptr_t) *u);
@@ -424,9 +429,9 @@ number(cp)
 
 Char  **
 copyblk(v)
-    register Char **v;
+    Char **v;
 {
-    register Char **nv =
+    Char **nv =
     (Char **) xcalloc((size_t) (blklen(v) + 1), sizeof(Char **));
 
     return (blkcpy(nv, v));
@@ -435,7 +440,7 @@ copyblk(v)
 #ifndef SHORT_STRINGS
 char   *
 strend(cp)
-    register char *cp;
+    char *cp;
 {
     if (!cp)
 	return (cp);
@@ -450,7 +455,7 @@ Char   *
 strip(cp)
     Char   *cp;
 {
-    register Char *dp = cp;
+    Char *dp = cp;
 
     if (!cp)
 	return (cp);
@@ -463,7 +468,7 @@ Char   *
 quote(cp)
     Char   *cp;
 {
-    register Char *dp = cp;
+    Char *dp = cp;
 
     if (!cp)
 	return (cp);
@@ -498,7 +503,7 @@ udvar(name)
 
 int
 prefix(sub, str)
-    register Char *sub, *str;
+    const Char *sub, *str;
 {
 
     for (;;) {
