@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.proc.c,v 3.77 2003/02/08 20:03:26 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.proc.c,v 3.83 2004/08/04 17:12:30 christos Exp $ */
 /*
  * sh.proc.c: Job manipulations
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.proc.c,v 3.77 2003/02/08 20:03:26 christos Exp $")
+RCSID("$Id: sh.proc.c,v 3.83 2004/08/04 17:12:30 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
@@ -151,12 +151,9 @@ sigret_t
 pchild(snum)
 int snum;
 {
-    register struct process *pp;
-    register struct process *fp;
-    register int pid;
-#if defined(BSDJOBS) || (!defined(BSDTIMES) && (defined(ODT) || defined(aiws) || defined(uts)))
-    extern int insource;
-#endif /* BSDJOBS || (!BSDTIMES && (ODT || aiws || uts)) */
+    struct process *pp;
+    struct process *fp;
+    int pid;
 #ifdef BSDWAIT
     union wait w;
 #else /* !BSDWAIT */
@@ -173,7 +170,6 @@ int snum;
 # else /* !_SEQUENT_ */
     struct tms proctimes;
 
-    USE(snum);
     if (!timesdone) {
 	timesdone++;
 	(void) times(&shtimes);
@@ -181,6 +177,7 @@ int snum;
 # endif	/* !_SEQUENT_ */
 #endif /* !BSDTIMES */
 
+    USE(snum);
 #ifdef JOBDEBUG
     xprintf("pchild()\n");
 #endif	/* JOBDEBUG */
@@ -445,8 +442,6 @@ found:
 		if ((jobflags & PSTOPPED) == 0)
 		    pflush(pp);
 		{
-		    extern Char GettingInput;
-
 		    if (GettingInput) {
 			errno = 0;
 			(void) Rawmode();
@@ -477,7 +472,7 @@ found:
 void
 pnote()
 {
-    register struct process *pp;
+    struct process *pp;
     int     flags;
 #ifdef BSDSIGS
     sigmask_t omask;
@@ -524,7 +519,7 @@ pfree(pp)
 void
 pwait()
 {
-    register struct process *fp, *pp;
+    struct process *fp, *pp;
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
@@ -562,9 +557,9 @@ pwait()
  */
 void
 pjwait(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *fp;
+    struct process *fp;
     int     jobflags, reason;
 #ifdef BSDSIGS
     sigmask_t omask;
@@ -603,7 +598,7 @@ pjwait(pp)
 	if ((jobflags & PRUNNING) == 0)
 	    break;
 #ifdef JOBDEBUG
-	xprintf("%d starting to sigpause for  SIGCHLD on %d\n",
+	xprintf("%d starting to sigpause for SIGCHLD on %d\n",
 		getpid(), fp->p_procid);
 #endif /* JOBDEBUG */
 #ifdef BSDSIGS
@@ -689,7 +684,7 @@ dowait(v, c)
     Char **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
@@ -730,7 +725,7 @@ loop:
 static void
 pflushall()
 {
-    register struct process *pp;
+    struct process *pp;
 
     for (pp = proclist.p_next; pp != NULL; pp = pp->p_next)
 	if (pp->p_procid)
@@ -744,10 +739,10 @@ pflushall()
  */
 static void
 pflush(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *np;
-    register int idx;
+    struct process *np;
+    int idx;
 
     if (pp->p_procid == 0) {
 	xprintf(CGETS(17, 3, "BUG: process flushed twice"));
@@ -778,7 +773,7 @@ pflush(pp)
  */
 static void
 pclrcurr(pp)
-    register struct process *pp;
+    struct process *pp;
 {
     if (pp == pcurrent) {
 	if (pprevious != NULL) {
@@ -804,7 +799,7 @@ static Char *cmdp;
  */
 Char *
 unparse(t)
-    register struct command *t;
+    struct command *t;
 {
     cmdp = command;
     cmdlen = 0;
@@ -821,9 +816,9 @@ unparse(t)
 void
 palloc(pid, t)
     int     pid;
-    register struct command *t;
+    struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
     int     i;
 
     pp = (struct process *) xcalloc(1, (size_t) sizeof(struct process));
@@ -908,7 +903,7 @@ palloc(pid, t)
 
 static void
 padd(t)
-    register struct command *t;
+    struct command *t;
 {
     Char  **argp;
 
@@ -974,7 +969,7 @@ static void
 pads(cp)
     Char   *cp;
 {
-    register int i;
+    int i;
 
     /*
      * Avoid the Quoted Space alias hack! Reported by:
@@ -1028,7 +1023,7 @@ prestjob()
 void
 pendjob()
 {
-    register struct process *pp, *tp;
+    struct process *pp, *tp;
 
     if (pcurrjob && (pcurrjob->p_flags & (PFOREGND | PSTOPPED)) == 0) {
 	pp = pcurrjob;
@@ -1058,14 +1053,13 @@ pendjob()
 
 static int
 pprint(pp, flag)
-    register struct process *pp;
+    struct process *pp;
     bool    flag;
 {
     int status, reason;
     struct process *tp;
-    extern char *linp, linbuf[];
     int     jobflags, pstatus, pcond;
-    char   *format;
+    const char *format;
 
 #ifdef BACKPIPE
     struct process *pipehead = NULL, *pipetail = NULL, *pmarker = NULL;
@@ -1302,13 +1296,13 @@ prcomd:
 
 static void
 ptprint(tp)
-    register struct process *tp;
+    struct process *tp;
 {
 #ifdef BSDTIMES
     struct timeval tetime, diff;
     static struct timeval ztime;
     struct sysrusage ru;
-    register struct process *pp = tp;
+    struct process *pp = tp;
 
     ru = zru;
     tetime = ztime;
@@ -1324,7 +1318,7 @@ ptprint(tp)
     timeval_t tetime, diff;
     static timeval_t ztime;
     struct process_stats ru;
-    register struct process *pp = tp;
+    struct process *pp = tp;
 
     ru = zru;
     tetime = ztime;
@@ -1352,7 +1346,7 @@ ptprint(tp)
 
 #  endif /* POSIX */
     struct tms zts, rts;
-    register struct process *pp = tp;
+    struct process *pp = tp;
 
     u_time = zu_time;
     s_time = zs_time;
@@ -1386,8 +1380,8 @@ dojobs(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
-    register int flag = NUMBER | NAME | REASON;
+    struct process *pp;
+    int flag = NUMBER | NAME | REASON;
     int     i;
 
     USE(c);
@@ -1417,7 +1411,7 @@ dofg(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 
     USE(c);
     okpcntl();
@@ -1448,7 +1442,7 @@ dofg1(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 
     USE(c);
     okpcntl();
@@ -1476,7 +1470,7 @@ dobg(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 
     USE(c);
     okpcntl();
@@ -1499,7 +1493,7 @@ dobg1(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 
     USE(c);
     pp = pfind(v[0]);
@@ -1533,12 +1527,9 @@ dokill(v, c)
     Char  **v;
     struct command *c;
 {
-    register int signum, len = 0;
-    register char *name;
+    int signum, len = 0;
+    const char *name;
     Char *sigptr;
-    char *ep;
-    extern int T_Cols;
-    extern int nsig;
 
     USE(c);
     v++;
@@ -1567,8 +1558,9 @@ dokill(v, c)
  	    }
  	}
  	if (Isdigit(*sigptr)) {
- 	    signum = strtol(short2str(sigptr), &ep, 10);
-	    if (signum < 0 || signum > (MAXSIG-1) || *ep)
+	    char *ep;
+ 	    signum = strtoul(short2str(sigptr), &ep, 0);
+	    if (*ep || signum < 0 || signum > (MAXSIG-1))
 		stderror(ERR_NAME | ERR_BADSIG);
 	}
 	else {
@@ -1592,14 +1584,13 @@ pkill(v, signum)
     Char  **v;
     int     signum;
 {
-    register struct process *pp, *np;
+    struct process *pp, *np;
     int jobflags = 0, err1 = 0;
     pid_t     pid;
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
     Char   *cp, **vp;
-    char   *ep;
 
 #ifdef BSDSIGS
     omask = sigmask(SIGCHLD);
@@ -1679,16 +1670,14 @@ pkill(v, signum)
 	else if (!(Isdigit(*cp) || *cp == '-'))
 	    stderror(ERR_NAME | ERR_JOBARGS);
 	else {
+	    char *ep;
 #ifndef WINNT_NATIVE
 	    pid = strtol(short2str(cp), &ep, 10);
 #else
-		pid = strtoul(short2str(cp),&ep,0);
+	    pid = strtoul(short2str(cp), &ep, 0);
 #endif /* WINNT_NATIVE */
-	    if (*ep) {
-		xprintf(CGETS(17, 14, "%S: Badly formed number\n"), cp);
-		err1++;
-		goto cont;
-	    }
+	    if (*ep)
+		stderror(ERR_NAME | ERR_JOBARGS);
 	    else if (kill(pid, signum) < 0) {
 		xprintf("%d: %s\n", pid, strerror(errno));
 		err1++;
@@ -1720,11 +1709,11 @@ cont:
  */
 int
 pstart(pp, foregnd)
-    register struct process *pp;
+    struct process *pp;
     int     foregnd;
 {
     int rv = 0;
-    register struct process *np;
+    struct process *np;
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
@@ -1797,7 +1786,7 @@ void
 panystop(neednl)
     bool    neednl;
 {
-    register struct process *pp;
+    struct process *pp;
 
     chkstop = 2;
     for (pp = proclist.p_next; pp; pp = pp->p_next)
@@ -1809,7 +1798,7 @@ struct process *
 pfind(cp)
     Char   *cp;
 {
-    register struct process *pp, *np;
+    struct process *pp, *np;
 
     if (cp == 0 || cp[1] == 0 || eq(cp, STRcent2) || eq(cp, STRcentplus)) {
 	if (pcurrent == NULL)
@@ -1833,7 +1822,7 @@ pfind(cp)
     for (pp = proclist.p_next; pp; pp = pp->p_next)
 	if (pp->p_procid == pp->p_jobid) {
 	    if (cp[1] == '?') {
-		register Char *dp;
+		Char *dp;
 
 		for (dp = pp->p_command; *dp; dp++) {
 		    if (*dp != cp[2])
@@ -1862,10 +1851,10 @@ pfind(cp)
  */
 static struct process *
 pgetcurr(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *np;
-    register struct process *xp = NULL;
+    struct process *np;
+    struct process *xp = NULL;
 
     for (np = proclist.p_next; np; np = np->p_next)
 	if (np != pcurrent && np != pp && np->p_procid &&
@@ -1887,7 +1876,7 @@ donotify(v, c)
     Char  **v;
     struct command *c;
 {
-    register struct process *pp;
+    struct process *pp;
 
     USE(c);
     pp = pfind(*++v);
@@ -1911,7 +1900,7 @@ pfork(t, wanttty)
     struct command *t;		/* command we are forking for */
     int     wanttty;
 {
-    register int pid;
+    int pid;
     bool    ignint = 0;
     int     pgrp;
 #ifdef BSDSIGS
