@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.sem.c,v 3.57 2002/11/21 20:02:01 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.sem.c,v 3.65 2004/11/21 04:35:22 christos Exp $ */
 /*
  * sh.sem.c: I/O redirections and job forking. A touchy issue!
  *	     Most stuff with builtins is incorrect
@@ -33,7 +33,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.sem.c,v 3.57 2002/11/21 20:02:01 christos Exp $")
+RCSID("$Id: sh.sem.c,v 3.65 2004/11/21 04:35:22 christos Exp $")
 
 #include "tc.h"
 #include "tw.h"
@@ -85,15 +85,11 @@ static	void		 chkclob	__P((char *));
 /*VARARGS 1*/
 void
 execute(t, wanttty, pipein, pipeout, do_glob)
-    register struct command *t;
+    struct command *t;
     int     wanttty;
     int *pipein, *pipeout;
     bool do_glob;
 {
-#ifdef VFORK
-    extern bool use_fork;	/* use fork() instead of vfork()? */
-#endif
-
     bool    forked = 0;
     struct biltins *bifunc;
     int     pid = 0;
@@ -654,6 +650,13 @@ execute(t, wanttty, pipein, pipeout, do_glob)
 	    func(t, bifunc);
 	    if (forked)
 		exitstat();
+	    else {
+		if (adrof(STRprintexitvalue)) {
+		    int rv = getn(varval(STRstatus));
+		    if (rv != 0)
+			xprintf(CGETS(17, 2, "Exit %d\n"), rv);
+		}
+	    }
 	    break;
 	}
 	if (t->t_dtyp != NODE_PAREN) {
@@ -663,15 +666,19 @@ execute(t, wanttty, pipein, pipeout, do_glob)
 	/*
 	 * For () commands must put new 0,1,2 in FSH* and recurse
 	 */
-	OLDSTD = dcopy(0, FOLDSTD);
-	SHOUT = dcopy(1, FSHOUT);
+	(void)close_on_exec(OLDSTD = dcopy(0, FOLDSTD), 1);
+	(void)close_on_exec(SHOUT = dcopy(1, FSHOUT), 1);
 	isoutatty = isatty(SHOUT);
-	SHDIAG = dcopy(2, FSHDIAG);
+	(void)close_on_exec(SHDIAG = dcopy(2, FSHDIAG), 1);
 	isdiagatty = isatty(SHDIAG);
 	(void) close(SHIN);
 	SHIN = -1;
 #ifndef CLOSE_ON_EXEC
 	didcch = 0;
+#else
+	(void) close_on_exec(FSHOUT, 1);
+	(void) close_on_exec(FSHDIAG, 1);
+	(void) close_on_exec(FOLDSTD, 1);
 #endif /* !CLOSE_ON_EXEC */
 	didfds = 0;
 	wanttty = -1;
@@ -752,7 +759,7 @@ static sigret_t
 vffree(snum)
 int snum;
 {
-    register Char **v;
+    Char **v;
 
     USE(snum);
     if ((v = gargv) != 0) {
@@ -790,7 +797,7 @@ int snum;
  */
 static Char *
 splicepipe(t, cp)
-    register struct command *t;
+    struct command *t;
     Char *cp;	/* word after < or > */
 {
     Char *blk[2];
@@ -836,12 +843,12 @@ splicepipe(t, cp)
  */
 static void
 doio(t, pipein, pipeout)
-    register struct command *t;
+    struct command *t;
     int    *pipein, *pipeout;
 {
-    register int fd;
-    register Char *cp;
-    register unsigned long flags = t->t_dflg;
+    int fd;
+    Char *cp;
+    unsigned long flags = t->t_dflg;
 
     if (didfds || (flags & F_REPEAT))
 	return;
@@ -861,10 +868,10 @@ doio(t, pipein, pipeout)
 	    xfree((ptr_t) cp);
 	    if ((fd = open(tmp, O_RDONLY|O_LARGEFILE)) < 0)
 		stderror(ERR_SYSTEM, tmp, strerror(errno));
-#ifdef O_LARGEFILE
 	    /* allow input files larger than 2Gb  */
-	    (void) fcntl(fd, O_LARGEFILE, 0);
-#endif /* O_LARGEFILE */
+#ifndef WINNT_NATIVE
+	    (void) fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_LARGEFILE);
+#endif /*!WINNT_NATIVE*/
 	    (void) dmove(fd, 0);
 	}
 	else if (flags & F_PIPEIN) {
@@ -921,10 +928,10 @@ doio(t, pipein, pipeout)
 	    }
 	    if ((fd = creat(tmp, 0666)) < 0)
 		stderror(ERR_SYSTEM, tmp, strerror(errno));
-#ifdef O_LARGEFILE
 	    /* allow input files larger than 2Gb  */
-	    (void) fcntl(fd, O_LARGEFILE, 0);
-#endif /* O_LARGEFILE */
+#ifndef WINNT_NATIVE
+	    (void) fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_LARGEFILE);
+#endif /*!WINNT_NATIVE*/
 	}
 	(void) dmove(fd, 1);
 	is1atty = isatty(1);
@@ -960,13 +967,13 @@ doio(t, pipein, pipeout)
 
 void
 mypipe(pv)
-    register int *pv;
+    int *pv;
 {
 
     if (pipe(pv) < 0)
 	goto oops;
-    pv[0] = dmove(pv[0], -1);
-    pv[1] = dmove(pv[1], -1);
+    (void)close_on_exec(pv[0] = dmove(pv[0], -1), 1);
+    (void)close_on_exec(pv[1] = dmove(pv[1], -1), 1);
     if (pv[0] >= 0 && pv[1] >= 0)
 	return;
 oops:
@@ -975,7 +982,7 @@ oops:
 
 static void
 chkclob(cp)
-    register char *cp;
+    char *cp;
 {
     struct stat stb;
 
