@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.time.c,v 3.23 2001/03/18 19:06:31 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.time.c,v 3.26 2004/08/04 17:12:30 christos Exp $ */
 /*
  * sh.time.c: Shell time keeping and printing.
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary	form must reproduce the	above copyright
  *    notice, this list	of conditions and the following	disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials	mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the	University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or	promote	products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.time.c,v 3.23 2001/03/18 19:06:31 christos Exp $")
+RCSID("$Id: sh.time.c,v 3.26 2004/08/04 17:12:30 christos Exp $")
 
 #ifdef SUNOS4
 # include <machine/param.h>
@@ -164,10 +160,10 @@ dotime(v, c)
 /*ARGSUSED*/
 void
 donice(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register Char *cp;
+    Char *cp;
     int	    nval = 0;
 
     USE(c);
@@ -187,7 +183,7 @@ donice(v, c)
 #ifdef BSDTIMES
 void
 ruadd(ru, ru2)
-    register struct sysrusage *ru,	*ru2;
+    struct sysrusage *ru,	*ru2;
 {
     tvadd(&ru->ru_utime, &ru2->ru_utime);
     tvadd(&ru->ru_stime, &ru2->ru_stime);
@@ -221,7 +217,7 @@ ruadd(ru, ru2)
 # ifdef	_SEQUENT_
 void
 ruadd(ru, ru2)
-    register struct process_stats *ru, *ru2;
+    struct process_stats *ru, *ru2;
 {
     tvadd(&ru->ps_utime, &ru2->ps_utime);
     tvadd(&ru->ps_stime, &ru2->ps_stime);
@@ -293,14 +289,14 @@ ruadd(ru, ru2)
 
 void
 prusage(r0, r1,	e, b)
-    register struct sysrusage *r0,	*r1;
+    struct sysrusage *r0,	*r1;
     timeval_t *e, *b;
 
 #else /* BSDTIMES */
 # ifdef	_SEQUENT_
 void
 prusage(r0, r1,	e, b)
-    register struct process_stats *r0, *r1;
+    struct process_stats *r0, *r1;
     timeval_t *e, *b;
 
 # else /* _SEQUENT_ */
@@ -319,7 +315,7 @@ prusage(bs, es,	e, b)
 #endif /* BSDTIMES */
 {
 #ifdef BSDTIMES
-    register time_t t =
+    time_t t =
     (r1->ru_utime.tv_sec - r0->ru_utime.tv_sec)	* 100 +
     (r1->ru_utime.tv_usec - r0->ru_utime.tv_usec) / 10000 +
     (r1->ru_stime.tv_sec - r0->ru_stime.tv_sec)	* 100 +
@@ -327,7 +323,7 @@ prusage(bs, es,	e, b)
 
 #else
 # ifdef	_SEQUENT_
-    register time_t t =
+    time_t t =
     (r1->ps_utime.tv_sec - r0->ps_utime.tv_sec)	* 100 +
     (r1->ps_utime.tv_usec - r0->ps_utime.tv_usec) / 10000 +
     (r1->ps_stime.tv_sec - r0->ps_stime.tv_sec)	* 100 +
@@ -335,20 +331,20 @@ prusage(bs, es,	e, b)
 
 # else /* _SEQUENT_ */
 #  ifndef POSIX
-    register time_t t =	(es->tms_utime - bs->tms_utime +
+    time_t t =	(es->tms_utime - bs->tms_utime +
 			 es->tms_stime - bs->tms_stime)	* 100 /	HZ;
 
 #  else	/* POSIX */
-    register clock_t t = (es->tms_utime	- bs->tms_utime	+
+    clock_t t = (es->tms_utime	- bs->tms_utime	+
 			  es->tms_stime	- bs->tms_stime) * 100 / clk_tck;
 
 #  endif /* POSIX */
 # endif	/* _SEQUENT_ */
 #endif /* BSDTIMES */
 
-    register char *cp;
-    register long i;
-    register struct varent *vp = adrof(STRtime);
+    const char *cp;
+    long i;
+    struct varent *vp = adrof(STRtime);
 
 #ifdef BSDTIMES
 # ifdef	convex
@@ -398,7 +394,7 @@ prusage(bs, es,	e, b)
     xprintf("t %lu\n", t);
 #endif /* TDEBUG */
 
-    if (vp && vp->vec[0] && vp->vec[1])
+    if (vp && vp->vec && vp->vec[0] && vp->vec[1])
 	cp = short2str(vp->vec[1]);
     for	(; *cp;	cp++)
 	if (*cp	!= '%')
