@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.func.c,v 3.106 2002/11/21 20:02:01 christos Exp $ */
+/* $Header: /src/pub/tcsh/tc.func.c,v 3.113 2004/11/21 04:38:03 christos Exp $ */
 /*
  * tc.func.c: New tcsh builtins.
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.func.c,v 3.106 2002/11/21 20:02:01 christos Exp $")
+RCSID("$Id: tc.func.c,v 3.113 2004/11/21 04:38:03 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -67,7 +67,7 @@ static bool beepcmd_active = 0;
 static signalfun_t alm_fun = NULL;
 
 static	void	 auto_logout	__P((int));
-static	char	*xgetpass	__P((char *));
+static	char	*xgetpass	__P((const char *));
 static	void	 auto_lock	__P((int));
 #ifdef BSDJOBS
 static	void	 insert		__P((struct wordent *, bool));
@@ -109,10 +109,10 @@ expand_lex(buf, bufsiz, sp0, from, to)
     struct  wordent *sp0;
     int     from, to;
 {
-    register struct wordent *sp;
-    register Char *s, *d, *e;
-    register Char prev_c;
-    register int i;
+    struct wordent *sp;
+    Char *s, *d, *e;
+    Char prev_c;
+    int i;
 
     /*
      * Make sure we have enough space to expand into.  E.g. we may have
@@ -201,7 +201,7 @@ Itoa(n, s, min_digits, attributes)
     unsigned int un;	/* handle most negative # too */
     int pad = (min_digits != 0);
 
-    if (sizeof(buf) - 1 < min_digits)
+    if ((int)(sizeof(buf) - 1) < min_digits)
 	min_digits = sizeof(buf) - 1;
 
     un = n;
@@ -227,17 +227,11 @@ Itoa(n, s, min_digits, attributes)
 /*ARGSUSED*/
 void
 dolist(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     int     i, k;
     struct stat st;
-#if defined(KANJI) && defined(SHORT_STRINGS) && defined(DSPMBYTE)
-    extern bool dspmbyte_ls;
-#endif
-#ifdef COLOR_LS_F
-    extern bool color_context_ls;
-#endif /* COLOR_LS_F */
 
     USE(c);
     if (*++v == NULL) {
@@ -260,8 +254,6 @@ dolist(v, c)
 	/*
 	 * We cannot process a flag therefore we let ls do it right.
 	 */
-	static Char STRls[] = {'l', 's', '\0'};
-	static Char STRmCF[] = {'-', 'C', 'F', '\0', '\0' };
 	Char *lspath;
 	struct command *t;
 	struct wordent cmd, *nextword, *lastword;
@@ -423,13 +415,13 @@ dolist(v, c)
     }
 }
 
-static char *defaulttell = "ALL";
+static const char *defaulttell = "ALL";
 extern bool GotTermCaps;
 
 /*ARGSUSED*/
 void
 dotelltc(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     USE(c);
@@ -449,9 +441,10 @@ dotelltc(v, c)
 /*ARGSUSED*/
 void
 doechotc(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
+    USE(c);
     if (!GotTermCaps)
 	GetTermCaps();
     EchoTC(++v);
@@ -465,6 +458,7 @@ dosettc(v, c)
 {
     char    tv[2][BUFSIZE];
 
+    USE(c);
     if (!GotTermCaps)
 	GetTermCaps();
 
@@ -521,7 +515,7 @@ cmd_expand(cmd, str)
 /*ARGSUSED*/
 void
 dowhich(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     int rv = TRUE;
@@ -564,8 +558,9 @@ dowhich(v, c)
 struct process *
 find_stop_ed()
 {
-    register struct process *pp, *retp;
-    register char *ep, *vp, *cp, *p;
+    struct process *pp, *retp;
+    const char *ep, *vp;
+    char *cp, *p;
     int     epl, vpl, pstatus;
 
     if ((ep = getenv("EDITOR")) != NULL) {	/* if we have a value */
@@ -582,9 +577,9 @@ find_stop_ed()
     else 
 	vp = "vi";
 
-    for (vpl = 0; vp[vpl] && !Isspace(vp[vpl]); vpl++)
+    for (vpl = 0; vp[vpl] && !isspace((unsigned char)vp[vpl]); vpl++)
 	continue;
-    for (epl = 0; ep[epl] && !Isspace(ep[epl]); epl++)
+    for (epl = 0; ep[epl] && !isspace((unsigned char)ep[epl]); epl++)
 	continue;
 
     if (pcurrent == NULL)	/* see if we have any jobs */
@@ -636,7 +631,7 @@ find_stop_ed()
 
 void
 fg_proc_entry(pp)
-    register struct process *pp;
+    struct process *pp;
 {
 #ifdef BSDSIGS
     sigmask_t omask;
@@ -681,7 +676,7 @@ fg_proc_entry(pp)
 
 static char *
 xgetpass(prm)
-    char *prm;
+    const char *prm;
 {
     static char pass[PASSMAX + 1];
     int fd, i;
@@ -709,6 +704,15 @@ xgetpass(prm)
     return(pass);
 }
 	
+#ifndef NO_CRYPT
+#ifndef __STDC__
+    extern char *crypt __P(());
+#endif
+#ifdef __linux__
+#include <crypt.h>
+#endif
+#endif
+
 /*
  * Ask the user for his login password to continue working
  * On systems that have a shadow password, this will only 
@@ -727,11 +731,6 @@ auto_lock(n)
     int i;
     char *srpp = NULL;
     struct passwd *pw;
-#ifdef POSIX
-    extern char *crypt __P((const char *, const char *));
-#else
-    extern char *crypt __P(());
-#endif
 
 #undef XCRYPT
 
@@ -855,6 +854,7 @@ sigret_t
 alrmcatch(snum)
 int snum;
 {
+    USE(snum);
 #ifdef UNRELSIGS
     if (snum)
 	(void) sigset(SIGALRM, alrmcatch);
@@ -1004,7 +1004,7 @@ beep_cmd()
 void
 period_cmd()
 {
-    register Char *vp;
+    Char *vp;
     time_t  t, interval;
 #ifdef BSDSIGS
     sigmask_t omask;
@@ -1225,7 +1225,7 @@ rmstar(cp)
     struct wordent *cp;
 {
     struct wordent *we, *args;
-    register struct wordent *tmp, *del;
+    struct wordent *tmp, *del;
 
 #ifdef RMDEBUG
     static Char STRrmdebug[] = {'r', 'm', 'd', 'e', 'b', 'u', 'g', '\0'};
@@ -1332,7 +1332,7 @@ continue_jobs(cp)
     struct wordent *cp;
 {
     struct wordent *we;
-    register struct process *pp, *np;
+    struct process *pp, *np;
     Char   *cmd, *continue_list, *continue_args_list;
 
 #ifdef CNDEBUG
@@ -1407,8 +1407,8 @@ insert(pl, file_args)
     struct wordent *now, *last;
     Char   *cmd, *bcmd, *cp1, *cp2;
     int     cmd_len;
-    Char   *pause = STRunderpause;
-    int     p_len = (int) Strlen(pause);
+    Char   *upause = STRunderpause;
+    int     p_len = (int) Strlen(upause);
 
     cmd_len = (int) Strlen(pl->word);
     cmd = (Char *) xcalloc(1, (size_t) ((cmd_len + 1) * sizeof(Char)));
@@ -1450,7 +1450,7 @@ insert(pl, file_args)
 	while ((*cp1++ = *cp2++) != '\0')
 	    continue;
 	cp1--;
-	cp2 = pause;
+	cp2 = upause;
 	while ((*cp1++ = *cp2++) != '\0')
 	    continue;
 	insert_we(now, last->prev);
@@ -1508,7 +1508,7 @@ static int
 inlist(list, name)
     Char   *list, *name;
 {
-    register Char *l, *n;
+    Char *l, *n;
 
     l = list;
     n = name;
@@ -1564,7 +1564,7 @@ static Char *
 gethomedir(us)
     Char   *us;
 {
-    register struct passwd *pp;
+    struct passwd *pp;
 #ifdef HESIOD
     char **res, **res1, *cp;
     Char *rp;
@@ -1643,7 +1643,7 @@ gettilde(us)
      * Binary search
      */
     for (bp1 = tcache, bp2 = tcache + tlength; bp1 < bp2;) {
-	register int i;
+	int i;
 
 	bp = bp1 + ((bp2 - bp1) >> 1);
 	if ((i = *us - *bp->user) == 0 && (i = Strcmp(us, bp->user)) == 0)
@@ -2163,7 +2163,7 @@ getremotehost()
 	     * have not caught up yet.
 	     */
 	    addr.s_addr = inet_addr(name);
-	    if (addr.s_addr != (unsigned long)~0)
+	    if (addr.s_addr != (unsigned int)~0)
 		host = name;
 	    else {
 		if (sptr != name) {
@@ -2266,3 +2266,49 @@ remotehost()
 
 }
 #endif /* REMOTEHOST */
+
+#ifndef WINNT_NATIVE
+/*
+ * indicate if a terminal type is defined in terminfo/termcap
+ * (by default the current term type). This allows ppl to look
+ * for a working term type automatically in their login scripts
+ * when using a terminal known as different things on different
+ * platforms
+ */
+void
+dotermname(v, c)
+    Char **v;
+    struct command *c;
+{
+    char *termtype;
+    /*
+     * Maximum size of a termcap record. We make it twice as large.
+     */
+    char termcap_buffer[2048];
+
+    USE(c);
+    /* try to find which entry we should be looking for */
+    termtype = (v[1] == NULL ? getenv("TERM") : short2str(v[1]));
+    if (termtype == NULL) {
+	/* no luck - the user didn't provide one and none is 
+	 * specified in the environment
+	 */
+	set(STRstatus, Strsave(STR1), VAR_READWRITE);
+	return;
+    }
+
+    /*
+     * we use the termcap function - if we are using terminfo we 
+     * will end up with it's compatibility function
+     * terminfo/termcap will be initialized with the new
+     * type but we don't care because tcsh has cached all the things
+     * it needs.
+     */
+    if (tgetent(termcap_buffer, termtype) == 1) {
+	xprintf("%s\n", termtype);
+	set(STRstatus, Strsave(STR0), VAR_READWRITE);
+    } else {
+	set(STRstatus, Strsave(STR1), VAR_READWRITE);
+    }
+}
+#endif /* WINNT_NATIVE */
