@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.os.h,v 3.85 2000/07/15 19:58:51 christos Exp $ */
+/* $Header: /src/pub/tcsh/tc.os.h,v 3.91 2004/09/28 15:37:29 christos Exp $ */
 /*
  * tc.os.h: Shell os dependent defines
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -37,9 +33,11 @@
 #ifndef _h_tc_os
 #define _h_tc_os
 
-#ifndef WINNT
+#ifndef __STDC__
+#ifndef WINNT_NATIVE
 #define NEEDstrerror		/* Too hard to find which systems have it */
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
+#endif
 
 
 #ifdef notdef 
@@ -52,6 +50,8 @@
 #endif /* notdef */
 
 #ifdef __CYGWIN__
+#  undef NOFILE
+#  define NOFILE sysconf(_SC_OPEN_MAX)
 #  undef NEEDstrerror
 #endif
 
@@ -167,6 +167,9 @@ struct ucred {
 #   define setpgrp(a, b) setpgrp2(a, b)
 #  endif /* BSDJOBS */
 # endif	/* POSIX */
+# ifndef TIOCSTI
+#  include <sys/strtio.h>
+# endif
 #endif /* hpux */
 
 /*
@@ -237,9 +240,12 @@ struct ucred {
  *
  * From: scott@craycos.com (Scott Bolte)
  */
-#ifndef WINNT
+#ifndef WINNT_NATIVE
 # ifdef F_SETFD
-#  define close_on_exec(fd, v) fcntl((fd), F_SETFD, v)
+#  ifndef FD_CLOEXEC
+#   define FD_CLOEXEC 1
+#  endif
+#  define close_on_exec(fd, v) fcntl((fd), F_SETFD, ((v) ? FD_CLOEXEC : 0))
 # else /* !F_SETFD */
 #  ifdef FIOCLEX
 #   define close_on_exec(fd, v) ioctl((fd), ((v) ? FIOCLEX : FIONCLEX), NULL)
@@ -247,9 +253,9 @@ struct ucred {
 #   define close_on_exec(fd, v)	/* Nothing */
 #  endif /* FIOCLEX */
 # endif /* F_SETFD */
-#else /* WINNT */
+#else /* WINNT_NATIVE */
 # define close_on_exec(fd, v) nt_close_on_exec((fd),(v))
-#endif /* !WINNT */
+#endif /* !WINNT_NATIVE */
 
 /*
  * Stat
@@ -506,7 +512,7 @@ struct ucred {
 #endif /* POSIX */
 
 
-#if !defined(SOLARIS2) && !defined(sinix) && !defined(BSD4_4) && !defined(WINNT)
+#if !defined(SOLARIS2) && !defined(sinix) && !defined(BSD4_4) && !defined(WINNT_NATIVE)
 # if (SYSVREL > 0 && !defined(OREO) && !defined(sgi) && !defined(linux) && !defined(sinix) && !defined(_AIX) &&!defined(_UWIN)) || defined(NeXT)
 #  define NEEDgetcwd
 # endif /* (SYSVREL > 0 && !OREO && !sgi && !linux && !sinix && !_AIX && !_UWIN) || NeXT */
@@ -532,7 +538,7 @@ typedef struct timeval timeval_t;
 #endif /* NeXT */
 
 #if !defined(BSD4_4) && !defined(__linux__) && !defined(__hpux) && \
-    !defined(sgi) && !defined(_AIX) && !defined(__GYGWIN__)
+    !defined(sgi) && !defined(_AIX) && !defined(__CYGWIN__)
 #ifndef NEEDgethostname
 extern int gethostname __P((char *, int));
 #endif /* NEEDgethostname */
@@ -558,7 +564,7 @@ extern caddr_t sbrk __P((int));
 extern int qsort();
 #  endif /* SYSVREL == 0 && !__lucid */
 # else /* !SUNOS4 */
-#  ifndef WINNT
+#  ifndef WINNT_NATIVE
 #   ifndef hpux
 #    if __GNUC__ != 2
 extern int abort();
@@ -570,7 +576,7 @@ extern int qsort();
 extern void abort();
 extern void qsort();
 #   endif /* hpux */
-#  endif /* !WINNT */
+#  endif /* !WINNT_NATIVE */
 # endif	/* SUNOS4 */
 #ifndef _CX_UX
 extern void perror();
