@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.prompt.c,v 3.39 2000/06/10 19:20:01 kim Exp $ */
+/* $Header: /src/pub/tcsh/tc.prompt.c,v 3.51 2004/11/20 20:30:47 christos Exp $ */
 /*
  * tc.prompt.c: Prompt printing stuff
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.prompt.c,v 3.39 2000/06/10 19:20:01 kim Exp $")
+RCSID("$Id: tc.prompt.c,v 3.51 2004/11/20 20:30:47 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -52,8 +48,8 @@ RCSID("$Id: tc.prompt.c,v 3.39 2000/06/10 19:20:01 kim Exp $")
  *	29-Dec-96	added rprompt support
  */
 
-static char   *month_list[12];
-static char   *day_list[7];
+static const char   *month_list[12];
+static const char   *day_list[7];
 
 void
 dateinit()
@@ -114,10 +110,10 @@ dateinit()
 void
 printprompt(promptno, str)
     int     promptno;
-    char   *str;
+    const char   *str;
 {
     static  Char *ocp = NULL;
-    static  char *ostr = NULL;
+    static  const char *ostr = NULL;
     time_t  lclock = time(NULL);
     Char   *cp;
 
@@ -149,10 +145,14 @@ printprompt(promptno, str)
 
     PromptBuf[0] = '\0';
     tprintf(FMT_PROMPT, PromptBuf, cp, 2 * INBUFSIZE - 2, str, lclock, NULL);
+#ifdef DSPMBYTE
+    if (dspmbyte_utf8)
+	Setutf8lit(PromptBuf, NULL);
+#endif
 
     if (!editing) {
 	for (cp = PromptBuf; *cp ; )
-	    (void) putraw(*cp++);
+	    (void) putwraw(*cp++);
 	SetAttributes(0);
 	flush();
     }
@@ -161,11 +161,15 @@ printprompt(promptno, str)
     if (promptno == 0) {	/* determine rprompt if using main prompt */
 	cp = varval(STRrprompt);
 	tprintf(FMT_PROMPT, RPromptBuf, cp, INBUFSIZE - 2, NULL, lclock, NULL);
+#ifdef DSPMBYTE
+	if (dspmbyte_utf8)
+	    Setutf8lit(RPromptBuf, NULL);
+#endif
 
 				/* if not editing, put rprompt after prompt */
 	if (!editing && RPromptBuf[0] != '\0') {
 	    for (cp = RPromptBuf; *cp ; )
-		(void) putraw(*cp++);
+		(void) putwraw(*cp++);
 	    SetAttributes(0);
 	    putraw(' ');
 	    flush();
@@ -179,7 +183,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
     Char *buf;
     const Char *fmt;
     size_t siz;
-    char *str;
+    const char *str;
     time_t tim;
     ptr_t info;
 {
@@ -199,9 +203,8 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 
 			/* prompt stuff */
     static Char *olddir = NULL, *olduser = NULL;
-    extern int tlength;	/* cache cleared */
-    int updirs, sz;
-    size_t pdirs;
+    int updirs;
+    size_t pdirs, sz;
 
     for (; *cp; cp++) {
 	if (p >= ep)
@@ -218,13 +221,20 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    cp++;
 	    switch (*cp) {
 	    case 'R':
-		if (what == FMT_HISTORY)
-		    fmthist('R', info, (char *) (cz = cbuff), sizeof(cbuff));
-		else
-		    cz = (unsigned char *) str;
+		if (what == FMT_HISTORY) {
+		    fmthist('R', info, (char *) cbuff, sizeof(cbuff));
+		    cz = cbuff;
+		} else
+		    cz = (const unsigned char *) str;
 		if (cz != NULL)
-		    for (; *cz; *p++ = attributes | *cz++)
-			if (p >= ep) break;
+		    for (; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		break;
 	    case '#':
 		*p++ = attributes | ((uid == 0) ? PRCHROOT : PRCH);
@@ -244,8 +254,14 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			eventno + 1);
 		    break;
 		}
-		for (cz = cbuff; *cz; *p++ = attributes | *cz++)
-		    if (p >= ep) break;
+		for (cz = cbuff; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+		    cz += one_mbtowc(p, cz, MB_LEN_MAX);
+		    *p |= attributes;
+#else
+		    *p = attributes | *cz++;
+#endif
+		}
 		break;
 	    case 'T':		/* 24 hour format	 */
 	    case '@':
@@ -302,33 +318,45 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'M':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO)
-		    cz = (unsigned char *) who_info(info, 'M',
+		    cz = (const unsigned char *) who_info(info, 'M',
 			(char *) cbuff, sizeof(cbuff));
 		else 
 #endif /* HAVENOUTMP */
-		    cz = (unsigned char *) getenv("HOST");
+		    cz = (const unsigned char *) getenv("HOST");
 		/*
 		 * Bug pointed out by Laurent Dami <dami@cui.unige.ch>: don't
 		 * derefrence that NULL (if HOST is not set)...
 		 */
 		if (cz != NULL)
-		    for (; *cz ; *p++ = attributes | *cz++)
-			if (p >= ep) break;
+		    for (; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		break;
 
 	    case 'm':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO)
-		    cz = (unsigned char *) who_info(info, 'm', (char *) cbuff,
-			sizeof(cbuff));
+		    cz = (const unsigned char *) who_info(info, 'm',
+			(char *) cbuff, sizeof(cbuff));
 		else 
 #endif /* HAVENOUTMP */
-		    cz = (unsigned char *) getenv("HOST");
+		    cz = (const unsigned char *) getenv("HOST");
 
 		if (cz != NULL)
-		    for ( ; *cz && (what == FMT_WHO || *cz != '.')
-			  ; *p++ = attributes | *cz++ )
-			if (p >= ep) break;
+		    for (; *cz && (what == FMT_WHO || *cz != '.') && p < ep;
+			 p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		break;
 
 			/* lukem: new directory prompt code */
@@ -357,7 +385,8 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			/* option to determine fixed # of dirs from path */
 		if (Scp == '.' || Scp == 'C') {
 		    int skip;
-#ifdef WINNT
+#ifdef WINNT_NATIVE
+		    Char *oldz = z;
 		    if (z[1] == ':') {
 		    	*p++ = attributes | *z++;
 		    	*p++ = attributes | *z++;
@@ -369,11 +398,23 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 					*p++ = attributes | *z++;
 				}while(*z != '/');
 			}
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
 		    q = z;
 		    while (*z)				/* calc # of /'s */
 			if (*z++ == '/')
 			    updirs++;
+
+#ifdef WINNT_NATIVE
+		    /*
+		     * for format type c, prompt will be following...
+		     * c:/path                => c:/path
+		     * c:/path/to             => c:to
+		     * //machine/share        => //machine/share
+		     * //machine/share/folder => //machine:folder
+		     */
+		    if (oldz[0] == '/' && oldz[1] == '/' && updirs > 1)
+			*p++ = attributes | ':';
+#endif /* WINNT_NATIVE */
 		    if ((Scp == 'C' && *q != '/'))
 			updirs++;
 
@@ -424,7 +465,7 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 			    *p++ = attributes | '+';
 			} else
 			    *p++ = attributes | ('0' + updirs);
-			*p++ = attributes | tcsh ? '>' : '%';
+			*p++ = attributes | '>';
 		    }
 		}
 		
@@ -436,10 +477,16 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'n':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO) {
-		    cz = (unsigned char *) who_info(info, 'n',
+		    cz = (const unsigned char *) who_info(info, 'n',
 			(char *) cbuff, sizeof(cbuff));
-		    for (; cz && *cz ; *p++ = attributes | *cz++)
-			if (p >= ep) break;
+		    for (; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		}
 		else  
 #endif /* HAVENOUTMP */
@@ -452,10 +499,16 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'l':
 #ifndef HAVENOUTMP
 		if (what == FMT_WHO) {
-		    cz = (unsigned char *) who_info(info, 'l',
+		    cz = (const unsigned char *) who_info(info, 'l',
 			(char *) cbuff, sizeof(cbuff));
-		    for (; cz && *cz ; *p++ = attributes | *cz++)
-			if (p >= ep) break;
+		    for (; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		}
 		else  
 #endif /* HAVENOUTMP */
@@ -466,9 +519,15 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 		}
 		break;
 	    case 'd':
-		for (cz = (unsigned char *) day_list[t->tm_wday]; *cz;
-		    *p++ = attributes | *cz++)
-		    if (p >= ep) break;
+		for (cz = (const unsigned char *) day_list[t->tm_wday];
+		     *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+		    cz += one_mbtowc(p, cz, MB_LEN_MAX);
+		    *p |= attributes;
+#else
+		    *p = attributes | *cz++;
+#endif
+		}
 		break;
 	    case 'D':
 		if (p >= ep - 3) break;
@@ -476,9 +535,15 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 		break;
 	    case 'w':
 		if (p >= ep - 5) break;
-		for (cz = (unsigned char *) month_list[t->tm_mon]; *cz;
-		    *p++ = attributes | *cz++)
-		    if (p >= ep) break;
+		for (cz = (const unsigned char *) month_list[t->tm_mon];
+		    *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+		    cz += one_mbtowc(p, cz, MB_LEN_MAX);
+		    *p |= attributes;
+#else
+		    *p = attributes | *cz++;
+#endif
+		}
 		break;
 	    case 'W':
 		if (p >= ep - 3) break;
@@ -513,14 +578,30 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    case 'L':
 		ClearToBottom();
 		break;
+
+	    case 'j':
+		{
+		    Char xbuf[128], *ebuf, *xq;
+		    int njobs = -1;
+		    struct process *pp;
+		    for (pp = proclist.p_next; pp; pp = pp->p_next)
+			njobs++;
+		    /* make sure we have space */
+		    ebuf = Itoa(njobs, buf, 1, attributes);
+		    for (xq = xbuf; xq < ebuf; *p++ = *xq++)
+			if (p >= ep) break;
+		    break;
+		}
 	    case '?':
 		if ((z = varval(STRstatus)) != STRNULL)
 		    for (; *z; *p++ = attributes | *z++)
 			if (p >= ep) break;
 		break;
 	    case '$':
-		sz = (int) (ep - p);
-		(void) expdollar(&p, &cp, &pdirs, attributes);
+		sz = ep - p;
+		(void) expdollar(&p, &cp, &sz, attributes);
+		/* cp should point the last char of currnet % sequence */
+		cp--;
 		break;
 	    case '%':
 		*p++ = attributes | '%';
@@ -542,9 +623,16 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 	    default:
 #ifndef HAVENOUTMP
 		if (*cp == 'a' && what == FMT_WHO) {
-		    cz = who_info(info, 'a', (char *) cbuff, sizeof(cbuff));
-		    for (; cz && *cz; *p++ = attributes | *cz++)
-			if (p >= ep) break;
+		    cz = (const unsigned char *) who_info(info, 'a',
+			(char *) cbuff, sizeof(cbuff));
+		    for (; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+			cz += one_mbtowc(p, cz, MB_LEN_MAX);
+			*p |= attributes;
+#else
+			*p = attributes | *cz++;
+#endif
+		    }
 		}
 		else 
 #endif /* HAVENOUTMP */
@@ -563,8 +651,14 @@ tprintf(what, buf, fmt, siz, str, tim, info)
 		fmthist('h', info, (char *) cbuff, sizeof(cbuff));
 	    else
 		(void) xsnprintf((char *) cbuff, sizeof(cbuff), "%d", eventno + 1);
-	    for (cz = cbuff; *cz; *p++ = attributes | *cz++)
-		if (p >= ep) break;
+	    for (cz = cbuff; *cz && p < ep; p++) {
+#ifdef WIDE_STRINGS
+		cz += one_mbtowc(p, cz, MB_LEN_MAX);
+		*p |= attributes;
+#else
+		*p = attributes | *cz++;
+#endif
+	    }
 	}
 	else 
 	    *p++ = attributes | *cp;	/* normal character */
@@ -604,7 +698,7 @@ expdollar(dstp, srcp, spp, attr)
 
     vp = adrof(var);
     val = (!vp) ? tgetenv(var) : NULL;
-    if (vp) {
+    if (vp && vp->vec) {
 	for (i = 0; vp->vec[i] != NULL; i++) {
 	    for (val = vp->vec[i]; *spp > 0 && *val; (*spp)--)
 		*dst++ = *val++ | attr;
