@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.sched.c,v 3.17 2000/07/04 19:46:23 christos Exp $ */
+/* $Header: /src/pub/tcsh/tc.sched.c,v 3.20 2004/08/04 17:12:31 christos Exp $ */
 /*
  * tc.sched.c: Scheduled command execution
  *
@@ -16,11 +16,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,9 +34,10 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.sched.c,v 3.17 2000/07/04 19:46:23 christos Exp $")
+RCSID("$Id: tc.sched.c,v 3.20 2004/08/04 17:12:31 christos Exp $")
 
 #include "ed.h"
+#include "tw.h"
 #include "tc.h"
 
 extern int just_signaled;
@@ -64,10 +61,10 @@ sched_next()
 /*ARGSUSED*/
 void
 dosched(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
-    register struct sched_event *tp, *tp1, *tp2;
+    struct sched_event *tp, *tp1, *tp2;
     time_t  cur_time;
     int     count, hours, minutes, dif_hour, dif_min;
     Char   *cp;
@@ -96,7 +93,7 @@ dosched(v, c)
 	    tprintf(FMT_SCHED, sbuf, fmt, sizeof(sbuf), 
 		    short2str(buf), tp->t_when, (ptr_t) &count);
 	    for (cp = sbuf; *cp;)
-		xputchar(*cp++);
+		xputwchar(*cp++);
 	}
 	return;
     }
@@ -203,11 +200,10 @@ sched_run(n)
     int n;
 {
     time_t   cur_time;
-    register struct sched_event *tp, *tp1;
+    struct sched_event *tp, *tp1;
     struct wordent cmd, *nextword, *lastword;
     struct command *t;
     Char  **v, *cp;
-    extern Char GettingInput;
 #ifdef BSDSIGS
     sigmask_t omask;
 
@@ -268,7 +264,7 @@ sched_run(n)
 	if (seterr)
 	    stderror(ERR_OLD);
 	/* execute the parse tree. */
-	execute(t, -1, NULL, NULL);
+	execute(t, -1, NULL, NULL, TRUE);
 	/* done. free the lex list and parse tree. */
 	freelex(&cmd), freesyn(t);
     }
