@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.sig.c,v 3.24 1998/04/08 17:57:36 christos Exp $ */
+/* $Header: /src/pub/tcsh/tc.sig.c,v 3.28 2004/08/04 17:12:31 christos Exp $ */
 /*
  * tc.sig.c: Signal routine emulations
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.sig.c,v 3.24 1998/04/08 17:57:36 christos Exp $")
+RCSID("$Id: tc.sig.c,v 3.28 2004/08/04 17:12:31 christos Exp $")
 
 #include "tc.wait.h"
 
@@ -161,7 +157,7 @@ ourwait(w)
     pid_t pid;
 
 #  ifdef JOBDEBUG
-    xprintf(CGETS(25, 1, "our wait %d\n", stk_ptr));
+    xprintf(CGETS(25, 1, "our wait %d\n"), stk_ptr);
     flush();
 #  endif /* JOBDEBUG */
 
@@ -281,7 +277,7 @@ sigsetmask(mask)
 {
     sigset_t set, oset;
     int     m;
-    register int i;
+    int i;
 
     (void) sigemptyset(&set);
     (void) sigemptyset(&oset);
@@ -297,7 +293,7 @@ sigsetmask(mask)
 
     m = 0;
     for (i = 1; i <= MAXSIG; i++)
-	if (sigismember(&oset, i))
+	if (sigismember(&oset, i) == 1)
 	    SETBIT(m, i);
 
     return (m);
@@ -317,7 +313,7 @@ sigblock(mask)
 {
     sigset_t set, oset;
     int     m;
-    register int i;
+    int i;
 
     (void) sigemptyset(&set);
     (void) sigemptyset(&oset);
@@ -337,7 +333,7 @@ sigblock(mask)
     /* Return old mask to user. */
     m = 0;
     for (i = 1; i <= MAXSIG; i++)
-	if (sigismember(&oset, i))
+	if (sigismember(&oset, i) == 1)
 	    SETBIT(m, i);
 
     return (m);
@@ -356,7 +352,7 @@ bsd_sigpause(mask)
     sigmask_t     mask;
 {
     sigset_t set;
-    register int i;
+    int i;
 
     (void) sigemptyset(&set);
 
