@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.who.c,v 3.30 2000/06/11 02:14:16 kim Exp $ */
+/* $Header: /src/pub/tcsh/tc.who.c,v 3.39 2004/11/21 04:34:22 christos Exp $ */
 /*
  * tc.who.c: Watch logins and logouts...
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.who.c,v 3.30 2000/06/11 02:14:16 kim Exp $")
+RCSID("$Id: tc.who.c,v 3.39 2004/11/21 04:34:22 christos Exp $")
 
 #include "tc.h"
 
@@ -56,6 +52,8 @@ RCSID("$Id: tc.who.c,v 3.30 2000/06/11 02:14:16 kim Exp $")
 # ifndef _PATH_UTMP
 #  if defined(__UTMPX_FILE) && !defined(UTMPX_FILE)
 #   define _PATH_UTMP __UTMPX_FILE
+#  elif defined(_PATH_UTMPX)
+#   define _PATH_UTMP _PATH_UTMPX
 #  else
 #   define _PATH_UTMP UTMPX_FILE
 #  endif /* __UTMPX_FILE && !UTMPX_FILE */
@@ -68,9 +66,9 @@ RCSID("$Id: tc.who.c,v 3.30 2000/06/11 02:14:16 kim Exp $")
 #  define ut_time ut_xtime
 # endif /* __MVS__ */
 #else /* !HAVEUTMPX */
-# ifndef WINNT
+# ifndef WINNT_NATIVE
 #  include <utmp.h>
-# endif /* WINNT */
+# endif /* WINNT_NATIVE */
 #endif /* HAVEUTMPX */
 
 #ifndef BROKEN_CC
@@ -167,8 +165,13 @@ void
 watch_login(force)
     int force;
 {
-    int     utmpfd, comp = -1, alldone;
+    int     comp = -1, alldone;
     int	    firsttime = stlast == 1;
+#ifdef HAVE_UTMPNAME
+    struct utmp *uptr;
+#else
+    int utmpfd;
+#endif
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif				/* BSDSIGS */
@@ -181,13 +184,13 @@ watch_login(force)
 #if defined(UTHOST) && defined(_SEQUENT_)
     char   *host, *ut_find_host();
 #endif
-#ifdef WINNT
+#ifdef WINNT_NATIVE
     static int ncbs_posted = 0;
     USE(utmp);
     USE(utmpfd);
     USE(sta);
     USE(wpnew);
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
 
     /* stop SIGINT, lest our login list get trashed. */
 #ifdef BSDSIGS
@@ -197,7 +200,7 @@ watch_login(force)
 #endif
 
     v = adrof(STRwatch);
-    if (v == NULL && !force) {
+    if ((v == NULL || v->vec == NULL) && !force) {
 #ifdef BSDSIGS
 	(void) sigsetmask(omask);
 #else
@@ -214,7 +217,7 @@ watch_login(force)
 	interval = 0;
 	
     (void) time(&t);
-#ifdef WINNT
+#ifdef WINNT_NATIVE
 	/*
 	 * Since NCB_ASTATs take time, start em async at least 90 secs
 	 * before we are due -amol 6/5/97
@@ -226,7 +229,7 @@ watch_login(force)
  		ncbs_posted = 1;
 	    }
 	}
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
     if (t - watch_period < interval) {
 #ifdef BSDSIGS
 	(void) sigsetmask(omask);
@@ -236,9 +239,9 @@ watch_login(force)
 	return;			/* not long enough yet... */
     }
     watch_period = t;
-#ifdef WINNT
+#ifdef WINNT_NATIVE
     ncbs_posted = 0;
-#else /* !WINNT */
+#else /* !WINNT_NATIVE */
 
     /*
      * From: Michael Schroeder <mlschroe@immd4.informatik.uni-erlangen.de>
@@ -265,7 +268,11 @@ watch_login(force)
 	return;
     }
     stlast = sta.st_mtime;
-    if ((utmpfd = open(_PATH_UTMP, O_RDONLY)) < 0) {
+#ifdef HAVE_UTMPNAME
+    utmpname(_PATH_UTMP);
+    setutent();
+#else
+    if ((utmpfd = open(_PATH_UTMP, O_RDONLY|O_LARGEFILE)) < 0) {
 	if (!force)
 	    xprintf(CGETS(26, 2,
 			  "%s cannot be opened.  Please \"unset watch\".\n"),
@@ -277,6 +284,7 @@ watch_login(force)
 # endif
 	return;
     }
+#endif
 
     /*
      * xterm clears the entire utmp entry - mark everyone on the status list
@@ -291,7 +299,12 @@ watch_login(force)
      * Read in the utmp file, sort the entries, and update existing entries or
      * add new entries to the status list.
      */
+#ifdef HAVE_UTMPNAME
+    while ((uptr = getutent()) != NULL) {
+        memcpy(&utmp, uptr, sizeof (utmp));
+#else
     while (read(utmpfd, (char *) &utmp, sizeof utmp) == sizeof utmp) {
+#endif
 
 # ifdef DEAD_PROCESS
 #  ifndef IRIS4D
@@ -387,11 +400,15 @@ watch_login(force)
 	    wp->who_prev = wpnew;	/* linked in now */
 	}
     }
+#ifdef HAVE_UTMPNAME
+    endutent();
+#else
     (void) close(utmpfd);
+#endif
 # if defined(UTHOST) && defined(_SEQUENT_)
     endutent();
 # endif
-#endif /* !WINNT */
+#endif /* !WINNT_NATIVE */
 
     if (force || vp == NULL)
 	return;
@@ -455,9 +472,9 @@ watch_login(force)
 #ifdef WHODEBUG
 static void
 debugwholist(new, wp)
-    register struct who *new, *wp;
+    struct who *new, *wp;
 {
-    register struct who *a;
+    struct who *a;
 
     a = whohead.who_next;
     while (a->who_next != NULL) {
@@ -500,12 +517,12 @@ print_who(wp)
     struct varent *vp = adrof(STRwho);
     Char buf[BUFSIZE];
 
-    if (vp && vp->vec[0])
+    if (vp && vp->vec && vp->vec[0])
 	cp = vp->vec[0];
 
     tprintf(FMT_WHO, buf, cp, BUFSIZE, NULL, wp->who_time, (ptr_t) wp);
     for (cp = buf; *cp;)
-	xputchar(*cp++);
+	xputwchar(*cp++);
     xputchar('\n');
 } /* end print_who */
 
@@ -558,13 +575,14 @@ who_info(ptr, c, wbuf, wbufsiz)
 	    return CGETS(26, 12, "local");
 	else {
 	    /* the ':' stuff is for <host>:<display>.<screen> */
-	    for (pb = wp->who_host, flg = Isdigit(*pb) ? '\0' : '.';
-		 *pb != '\0' &&
-		 (*pb != flg || ((pb = strchr(pb, ':')) != 0));
+	    for (pb = wp->who_host,
+		flg = isdigit((unsigned char)*pb) ? '\0' : '.';
+		*pb != '\0' && (*pb != flg || ((pb = strchr(pb, ':')) != 0));
 		 pb++) {
 		if (*pb == ':')
 		    flg = '\0';
-		*wb++ = Isupper(*pb) ? Tolower(*pb) : *pb;
+		*wb++ = isupper((unsigned char)*pb) ?
+		    tolower((unsigned char)*pb) : *pb;
 	    }
 	    *wb = '\0';
 	    return wbuf;
@@ -575,7 +593,8 @@ who_info(ptr, c, wbuf, wbufsiz)
 	    return CGETS(26, 12, "local");
 	else {
 	    for (pb = wp->who_host; *pb != '\0'; pb++)
-		*wb++ = Isupper(*pb) ? Tolower(*pb) : *pb;
+		*wb++ = isupper((unsigned char)*pb) ?
+		    tolower((unsigned char)*pb) : *pb;
 	    *wb = '\0';
 	    return wbuf;
 	}
@@ -616,6 +635,12 @@ struct command *c;
 }
 
 # ifdef UTHOST
+size_t
+utmphostsize()
+{
+    return UTHOSTLEN;
+}
+
 char *
 utmphost()
 {
@@ -635,7 +660,7 @@ utmphost()
 }
 # endif /* UTHOST */
 
-#ifdef WINNT
+#ifdef WINNT_NATIVE
 void add_to_who_list(name, mach_nm)
     char *name;
     char *mach_nm;
@@ -688,5 +713,5 @@ void add_to_who_list(name, mach_nm)
 	wp->who_prev = wpnew;	/* linked in now */
     }
 }
-#endif /* WINNT */
+#endif /* WINNT_NATIVE */
 #endif /* HAVENOUTMP */
