@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tw.init.c,v 3.26 2000/07/15 19:58:53 christos Exp $ */
+/* $Header: /src/pub/tcsh/tw.init.c,v 3.31 2004/08/08 06:42:29 christos Exp $ */
 /*
  * tw.init.c: Handle lists of things to complete
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.init.c,v 3.26 2000/07/15 19:58:53 christos Exp $")
+RCSID("$Id: tw.init.c,v 3.31 2004/08/08 06:42:29 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -115,7 +111,7 @@ static sigmask_t tw_omask;
 static Char	*tw_str_add		__P((stringlist_t *, int));
 static void	 tw_str_free		__P((stringlist_t *));
 static Char     *tw_dir_next		__P((DIR *));
-static void	 tw_cmd_add 		__P((Char *name));
+static void	 tw_cmd_add 		__P((const Char *name));
 static void 	 tw_cmd_cmd		__P((void));
 static void	 tw_cmd_builtin		__P((void));
 static void	 tw_cmd_alias		__P((void));
@@ -192,7 +188,7 @@ static Char *
 tw_dir_next(dfd)
     DIR *dfd;
 {
-    register struct dirent *dirp;
+    struct dirent *dirp;
 
     if (dfd == NULL)
 	return NULL;
@@ -210,7 +206,7 @@ tw_dir_next(dfd)
  */
 static void
 tw_cmd_add(name)
-    Char *name;
+    const Char *name;
 {
     int len;
 
@@ -236,16 +232,16 @@ tw_cmd_free()
 static void
 tw_cmd_cmd()
 {
-    register DIR *dirp;
-    register struct dirent *dp;
-    register Char *dir = NULL, *name;
-    register Char **pv;
+    DIR *dirp;
+    struct dirent *dp;
+    Char *dir = NULL, *name;
+    Char **pv;
     struct varent *v = adrof(STRpath);
     struct varent *recexec = adrof(STRrecognize_only_executables);
     int len;
 
 
-    if (v == NULL) /* if no path */
+    if (v == NULL || v->vec == NULL) /* if no path */
 	return;
 
     for (pv = v->vec; *pv; pv++) {
@@ -295,16 +291,16 @@ tw_cmd_cmd()
 static void
 tw_cmd_builtin()
 {
-    register struct biltins *bptr;
+    struct biltins *bptr;
 
     for (bptr = bfunc; bptr < &bfunc[nbfunc]; bptr++)
 	if (bptr->bname)
 	    tw_cmd_add(str2short(bptr->bname));
-#ifdef WINNT
+#ifdef WINNT_NATIVE
     for (bptr = nt_bfunc; bptr < &nt_bfunc[nt_nbfunc]; bptr++)
 	if (bptr->bname)
 	    tw_cmd_add(str2short(bptr->bname));
-#endif /* WINNT*/
+#endif /* WINNT_NATIVE*/
 } /* end tw_cmd_builtin */
 
 
@@ -314,8 +310,8 @@ tw_cmd_builtin()
 static void
 tw_cmd_alias()
 {
-    register struct varent *p;
-    register struct varent *c;
+    struct varent *p;
+    struct varent *c;
 
     p = &aliases;
     for (;;) {
@@ -505,9 +501,9 @@ tw_shvar_next(dir, flags)
     Char *dir;
     int	 *flags;
 {
-    register struct varent *p;
-    register struct varent *c;
-    register Char *cp;
+    struct varent *p;
+    struct varent *c;
+    Char *cp;
 
     USE(flags);
     USE(dir);
@@ -606,8 +602,6 @@ tw_complete_start(dfd, pat)
     DIR *dfd;
     Char *pat;
 {
-    extern struct varent completions;
-
     USE(pat);
     SETDIR(dfd)
     tw_vptr_start(&completions);
@@ -644,9 +638,9 @@ tw_logname_start(dfd, pat)
 {
     USE(pat);
     SETDIR(dfd)
-#if !defined(_VMS_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(WINNT_NATIVE)
     (void) setpwent();	/* Open passwd file */
-#endif /* !_VMS_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !WINNT_NATIVE */
 } /* end tw_logname_start */
 
 
@@ -670,12 +664,12 @@ tw_logname_next(dir, flags)
     USE(flags);
     USE(dir);
     TW_HOLD();
-#if !defined(_VMS_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(WINNT_NATIVE)
     /* ISC does not declare getpwent()? */
     pw = (struct passwd *) getpwent();
-#else /* _VMS_POSIX || WINNT */
+#else /* _VMS_POSIX || WINNT_NATIVE */
     pw = NULL;
-#endif /* !_VMS_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !WINNT_NATIVE */
     TW_RELS();
 
     if (pw == NULL) {
@@ -698,9 +692,9 @@ tw_logname_end()
 #ifdef YPBUGS
     fix_yp_bugs();
 #endif
-#if !defined(_VMS_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(WINNT_NATIVE)
    (void) endpwent();
-#endif /* !_VMS_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !WINNT_NATIVE */
 } /* end tw_logname_end */
 
 
@@ -715,9 +709,9 @@ tw_grpname_start(dfd, pat)
 {
     USE(pat);
     SETDIR(dfd)
-#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT_NATIVE)
     (void) setgrent();	/* Open group file */
-#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT_NATIVE */
 } /* end tw_grpname_start */
 
 
@@ -741,11 +735,11 @@ tw_grpname_next(dir, flags)
     USE(flags);
     USE(dir);
     TW_HOLD();
-#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT_NATIVE)
     gr = (struct group *) getgrent();
-#else /* _VMS_POSIX || _OSD_POSIX || WINNT */
+#else /* _VMS_POSIX || _OSD_POSIX || WINNT_NATIVE */
     gr = NULL;
-#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT_NATIVE */
     TW_RELS();
 
     if (gr == NULL) {
@@ -768,9 +762,9 @@ tw_grpname_end()
 #ifdef YPBUGS
     fix_yp_bugs();
 #endif
-#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT)
+#if !defined(_VMS_POSIX) && !defined(_OSD_POSIX) && !defined(WINNT_NATIVE)
    (void) endgrent();
-#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT */
+#endif /* !_VMS_POSIX && !_OSD_POSIX && !WINNT_NATIVE */
 } /* end tw_grpname_end */
 
 /* tw_file_start():
@@ -957,7 +951,7 @@ tw_bind_next(dir, flags)
     Char *dir;
     int *flags;
 {
-    char *ptr;
+    const char *ptr;
     USE(flags);
     if (tw_bind && tw_bind->name) {
 	for (ptr = tw_bind->name, dir = tw_retname;
@@ -997,7 +991,7 @@ tw_limit_next(dir, flags)
     int *flags;
 {
 #ifndef HAVENOLIMIT
-    char *ptr;
+    const char *ptr;
     if (tw_limit && tw_limit->limname) {
 	for (ptr = tw_limit->limname, dir = tw_retname; 
 	     (*dir++ = (Char) *ptr++) != '\0';)
@@ -1035,8 +1029,7 @@ tw_sig_next(dir, flags)
     Char *dir;
     int *flags;
 {
-    char *ptr;
-    extern int nsig;
+    const char *ptr;
     USE(flags);
     for (;tw_index < nsig; tw_index++) {
 
