@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tw.parse.c,v 3.93 2003/02/08 20:03:26 christos Exp $ */
+/* $Header: /src/pub/tcsh/tw.parse.c,v 3.100 2004/11/20 18:23:03 christos Exp $ */
 /*
  * tw.parse.c: Everyone has taken a shot in this futile effort to
  *	       lexically analyze a csh line... Well we cannot good
@@ -35,12 +35,17 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.parse.c,v 3.93 2003/02/08 20:03:26 christos Exp $")
+RCSID("$Id: tw.parse.c,v 3.100 2004/11/20 18:23:03 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
 #include "tc.h"
 
+#ifdef DSPMBYTE
+extern bool dspmbyte_utf8;
+extern int ColLen __P((Char *));
+#endif
+
 #ifdef WINNT_NATIVE
 #include "nt.const.h"
 #endif /* WINNT_NATIVE */
@@ -97,11 +102,6 @@ int InsideCompletion = 0;
 
 /* do the expand or list on the command line -- SHOULD BE REPLACED */
 
-extern Char NeedsRedraw;	/* from ed.h */
-extern int Tty_raw_mode;
-extern int TermH;		/* from the editor routines */
-extern int lbuffed;		/* from sh.print.c */
-
 static	void	 extract_dir_and_name	__P((Char *, Char *, Char *));
 static	int	 insert_meta		__P((Char *, Char *, Char *, bool));
 static	Char	*tilde			__P((Char *, Char *));
@@ -116,7 +116,7 @@ static	int	 is_prefix		__P((Char *, Char *));
 static	int	 is_prefixmatch		__P((Char *, Char *, int));
 static	int	 is_suffix		__P((Char *, Char *));
 static	int	 recognize		__P((Char *, Char *, int, int, int,
-    int));
+					     int));
 static	int	 ignored		__P((Char *));
 static	int	 isadirectory		__P((Char *, Char *));
 #ifndef __MVS__
@@ -423,7 +423,7 @@ tenematch(inputline, num_read, command)
  */
 static int
 t_glob(v, cmd)
-    register Char ***v;
+    Char ***v;
     int cmd;
 {
     jmp_buf_t osetexit;
@@ -477,7 +477,7 @@ t_glob(v, cmd)
  */
 static int
 c_glob(v)
-    register Char ***v;
+    Char ***v;
 {
     Char *pat = **v, *cmd, **av;
     Char dir[MAXPATHLEN+1];
@@ -527,7 +527,7 @@ insert_meta(cp, cpend, word, closequotes)
 {
     Char buffer[2 * FILSIZ + 1], *bptr, *wptr;
     int in_sync = (cp != NULL);
-    int qu = 0;
+    Char qu = 0;
     int ndel = (int) (cp ? cpend - cp : 0);
     Char w, wq;
 #ifdef DSPMBYTE
@@ -635,7 +635,7 @@ insert_meta(cp, cpend, word, closequotes)
  */
 static int
 is_prefix(check, template)
-    register Char *check, *template;
+    Char *check, *template;
 {
     for (; *check; check++, template++)
 	if ((*check & TRIM) != (*template & TRIM))
@@ -695,9 +695,9 @@ is_prefixmatch(check, template, igncase)
  */
 static int
 is_suffix(check, template)
-    register Char *check, *template;
+    Char *check, *template;
 {
-    register Char *t, *c;
+    Char *t, *c;
 
     for (t = template; *t++;)
 	continue;
@@ -717,10 +717,10 @@ is_suffix(check, template)
  */
 static int
 ignored(item)
-    register Char *item;
+    Char *item;
 {
     struct varent *vp;
-    register Char **cp;
+    Char **cp;
 
     if ((vp = adrof(STRfignore)) == NULL || (cp = vp->vec) == NULL)
 	return (FALSE);
@@ -737,9 +737,9 @@ ignored(item)
  */
 int
 starting_a_command(wordstart, inputline)
-    register Char *wordstart, *inputline;
+    Char *wordstart, *inputline;
 {
-    register Char *ptr, *ncmdstart;
+    Char *ptr, *ncmdstart;
     int     count;
     static  Char
             cmdstart[] = {'`', ';', '&', '(', '|', '\0'},
@@ -816,12 +816,11 @@ starting_a_command(wordstart, inputline)
 static int
 recognize(exp_name, item, name_length, numitems, enhanced, igncase)
     Char   *exp_name, *item;
-    int     name_length, numitems, enhanced;
+    int     name_length, numitems, enhanced, igncase;
 {
     Char MCH1, MCH2;
-    register Char *x, *ent;
-    register int len = 0;
-    struct varent *vp;
+    Char *x, *ent;
+    int len = 0;
 
     if (numitems == 1) {	/* 1st match */
 	copyn(exp_name, item, MAXNAMLEN);
@@ -882,7 +881,7 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
     Char *item, *ptr;
     Char buf[MAXPATHLEN+1];
     struct varent *vp;
-    int len, enhanced;
+    int len, enhanced = 0;
     int cnt = 0;
     int igncase = 0;
 
@@ -921,6 +920,23 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 	    break;
 
 	case TW_COMMAND:
+#if defined(_UWIN) || defined(__CYGWIN__)
+	    /* Turn foo.{exe,com,bat} into foo since UWIN's readdir returns
+	     * the file with the .exe, .com, .bat extension
+	     */
+	    {
+		size_t ext = strlen((char *)item) - 4;
+		if ((ext > 0) && (strcasecmp((char *)&item[ext], ".exe") == 0 ||
+				  strcasecmp((char *)&item[ext], ".bat") == 0 ||
+				  strcasecmp((char *)&item[ext], ".com") == 0))
+		    {
+			item[ext] = '\0';
+#if defined(__CYGWIN__)
+			strlwr((char *)item);
+#endif /* __CYGWIN__ */
+		    }
+	    }
+#endif /* _UWIN || __CYGWIN__ */
 	    exec_check = flags & TW_EXEC_CHK;
 	    dir_ok = flags & TW_DIR_OK;
 	    break;
@@ -980,15 +996,16 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 	case RECOGNIZE_ALL:
 	case RECOGNIZE_SCROLL:
 
- 	    if ((vp = adrof(STRcomplete)) != NULL && vp->vec != NULL) {
-		int i;
-		for (i = 0; vp->vec[i]; i++) {
-		    if (Strcmp(vp->vec[i], STRigncase) == 0)
-			igncase++;
-		    if (Strcmp(vp->vec[i], STRenhance) == 0)
-			enhanced++;
+	    if ((vp = adrof(STRcomplete)) != NULL && vp->vec != NULL) {
+		Char **cp;
+		for (cp = vp->vec; *cp; cp++) {
+		    if (Strcmp(*cp, STRigncase) == 0)
+			igncase = 1;
+		    if (Strcmp(*cp, STRenhance) == 0)
+			enhanced = 1;
 		}
 	    }
+
 	    if (enhanced || igncase) {
 	        if (!is_prefixmatch(target, item, igncase)) 
 		    break;
@@ -1073,8 +1090,8 @@ tw_collect_items(command, looking, exp_dir, exp_name, target, pat, flags)
 			break;
 		    }
 		}
-		if (recognize(exp_name, item, name_length, ++numitems, enhanced,
-		    igncase)) 
+		if (recognize(exp_name, item, name_length, ++numitems,
+		    enhanced, igncase)) 
 		    if (command != RECOGNIZE_SCROLL)
 			done = TRUE;
 		if (enhanced && (int)Strlen(exp_name) < name_length)
@@ -1716,7 +1733,7 @@ static void
 extract_dir_and_name(path, dir, name)
     Char   *path, *dir, *name;
 {
-    register Char *p;
+    Char *p;
 
     p = Strrchr(path, '/');
 #ifdef WINNT_NATIVE
@@ -1769,7 +1786,7 @@ static Char *
 tilde(new, old)
     Char   *new, *old;
 {
-    register Char *o, *p;
+    Char *o, *p;
 
     switch (old[0]) {
     case '~':
@@ -1879,7 +1896,7 @@ nostat(dir)
      Char *dir;
 {
     struct varent *vp;
-    register Char **cp;
+    Char **cp;
 
     if ((vp = adrof(STRnostat)) == NULL || (cp = vp->vec) == NULL)
 	return FALSE;
@@ -2016,7 +2033,7 @@ find_rows(items, count, no_file_suffix)
     Char *items[];
     int     count, no_file_suffix;
 {
-    register int i, columns, rows;
+    int i, columns, rows;
     unsigned int maxwidth = 0;
 
     for (i = 0; i < count; i++)	/* find widest string */
@@ -2039,10 +2056,10 @@ find_rows(items, count, no_file_suffix)
  */
 void
 print_by_column(dir, items, count, no_file_suffix)
-    register Char *dir, *items[];
+    Char *dir, *items[];
     int     count, no_file_suffix;
 {
-    register int i, r, c, columns, rows;
+    int i, r, c, columns, rows;
     unsigned int w, maxwidth = 0;
     Char *val;
     bool across;
@@ -2053,8 +2070,15 @@ print_by_column(dir, items, count, no_file_suffix)
     across = ((val = varval(STRlistflags)) != STRNULL) && 
 	     (Strchr(val, 'x') != NULL);
 
-    for (i = 0; i < count; i++)	/* find widest string */
+    for (i = 0; i < count; i++)	{ /* find widest string */
+#ifdef DSPMBYTE
+	if (dspmbyte_utf8) {
+	    maxwidth = max(maxwidth, (unsigned int) ColLen(items[i]));
+	    continue;
+	}
+#endif
 	maxwidth = max(maxwidth, (unsigned int) Strlen(items[i]));
+    }
 
     maxwidth += no_file_suffix ? 1 : 2;	/* for the file tag and space */
     columns = TermH / maxwidth;		/* PWP: terminal size change */
@@ -2089,15 +2113,20 @@ print_by_column(dir, items, count, no_file_suffix)
 		}
 		else {
 		    /* Print filename followed by '/' or '*' or ' ' */
-		    xprintf("%S%c", items[i],
+		    xprintf("\045S%c", items[i],
 			    filetype(dir, items[i]));
 		    w++;
 		}
 #endif /* COLOR_LS_F */
 
-		if (c < (columns - 1))	/* Not last column? */
+		if (c < (columns - 1)) {	/* Not last column? */
+#ifdef DSPMBYTE
+		    if (dspmbyte_utf8)
+			w = (unsigned int) ColLen(items[i]);
+#endif
 		    for (; w < maxwidth; w++)
 			xputchar(' ');
+		}
 	    }
 	    else if (across)
 		break;
@@ -2117,7 +2146,7 @@ print_by_column(dir, items, count, no_file_suffix)
  */
 int
 StrQcmp(str1, str2)
-    register Char *str1, *str2;
+    const Char *str1, *str2;
 {
     for (; *str1 && samecase(*str1 & TRIM) == samecase(*str2 & TRIM); 
 	 str1++, str2++)
@@ -2156,7 +2185,8 @@ fcompare(file1, file2)
  */
 void
 catn(des, src, count)
-    register Char *des, *src;
+    Char *des;
+    const Char *src;
     int count;
 {
     while (--count >= 0 && *des)
@@ -2174,7 +2204,7 @@ catn(des, src, count)
  */
 void
 copyn(des, src, count)
-    register Char *des, *src;
+    Char *des, *src;
     int count;
 {
     while (--count >= 0)
@@ -2193,9 +2223,10 @@ tgetenv(str)
     Char   *str;
 {
     Char  **var;
-    int     len, res;
+    size_t  len;
+    int     res;
 
-    len = (int) Strlen(str);
+    len = Strlen(str);
     /* Search the STR_environ for the entry matching str. */
     for (var = STR_environ; var != NULL && *var != NULL; var++)
 	if (Strlen(*var) >= len && (*var)[len] == '=') {
