@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/vms.termcap.c,v 1.5 1997/10/02 16:36:36 christos Exp $ */
+/* $Header: /src/pub/tcsh/vms.termcap.c,v 1.7 2004/08/04 17:12:32 christos Exp $ */
 /*
  *	termcap.c	1.1	20/7/87		agc	Joypace Ltd
  *
@@ -9,7 +9,7 @@
  *	A public domain implementation of the termcap(3) routines.
  */
 #include "sh.h"
-RCSID("$Id: vms.termcap.c,v 1.5 1997/10/02 16:36:36 christos Exp $")
+RCSID("$Id: vms.termcap.c,v 1.7 2004/08/04 17:12:32 christos Exp $")
 #if defined(_VMS_POSIX) || defined(_OSD_POSIX)
 /*    efth      1988-Apr-29
 
@@ -271,7 +271,7 @@ char	*cm;
 int	destcol;
 int	destline;
 {
-	register char	*rp;
+	char	*rp;
 	static char	ret[24];
 	int		incr = 0;
 	int 		argno = 0, numval;
@@ -323,7 +323,7 @@ int	destline;
  */
 int
 tputs(cp, affcnt, outc)
-register char	*cp;
+char	*cp;
 int		affcnt;
 int		(*outc)();
 {
@@ -334,4 +334,4 @@ int		(*outc)();
 		(*outc)(*cp++);
 	return(1);
 }
-#endif /* _VMS_POSIX */
+#endif /* _VMS_POSIX || _OSD_POSIX */
